%running matslise at the command line: example 0
%harmonic oscillator y''=(x^2-E)y on [0,10]

disp('*********************************')
disp('A truncated harmonic oscillator.')
disp('*********************************')


%First we specify the problem:
r=schrod('x^2',0,10,1,0,1,0);
%The schrod constructor is used, the first argument is a string
%representing the potential function, the second and third argument are
%the two endpoints of the integration interval and the last arguments
%specify the boundary conditions

%We want to solve the problem with the CPM{16,14} method.
%initialise the CPM{16,14}-method:
cp=cpm16_14(r,1e-10);
%We have chosen the tolerance to be 1e-10
plot_partition(cp);

%calculate the first 11 eigenvalues:
E=get_eigenvalues(cp,0,10,true); 
disp('The first 11 eigenvalues of the (truncated) harmonic oscillator problem:')
disp(sprintf('k \t  E_k \t\t\t\t  estimated error'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e')])
end

%eigenfunction associated to the first eigenvalue, evaluated in 100 points:
V=get_eigenfunction(cp,E.eigenvalues(1),100);
figure;
plot(V.x,V.y);
xlabel('x')
ylabel('y')
title('First eigenfunction of the (truncated) harmonic oscillator problem')

%calculate the eigenvalues between 10 and 20:
E=get_eigenvalues(cp,10,20,false); 
%note that the partition has not to be calculated again, the same partition
%is still used.
disp('The eigenvalues found in the interval [10,20] for the (truncated) harmonic oscillator problem:')
disp(sprintf('k \t  E_k \t\t\t\t  estimated error'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e')])
end

