%running matslise at the command line: example 1
%harmonic oscillator y''=(x^2-E)y on [-inf,inf]

disp('*******************************************************************************************')
disp('The harmonic oscillator: a Schrodinger problem defined on an infinite integration interval.')
disp('*******************************************************************************************')


%First we specify the problem:
r=schrod('x^2',-inf,inf,1,0,1,0);
%The schrod constructor is used, the first argument is a string
%representing the potential function, the second and third argument are
%the two endpoints of the integration interval and the last arguments
%specify the boundary conditions

%We want to solve the problem with the CPM{16,14} method.
%initialise the CPM{16,14}-method:
cp=cpm16_14(r,1e-10); 
%no partition is constructed at this stage (since we're dealing with an
%infinite problem)
%We have chosen the tolerance to be 1e-10

%calculate the first 11 eigenvalues:
[E,cp]=get_eigenvalues(cp,0,10,true); 
%cp contains now information on the truncated integration interval, this truncated
%integration interval is large enough to obtain the first 11 eigenvalues
%within an accuray of 1e-10
disp('The first 11 eigenvalues of the harmonic oscillator problem:')
disp(sprintf('k \t  E_k \t\t\t\t  estimated error'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e')])
end

%eigenfunction associated to the first eigenvalue, evaluated in 100 points:
V=get_eigenfunction(cp,E.eigenvalues(1),100);
plot(V.x,V.y);
xlabel('x')
ylabel('y')
title('First eigenfunction of the harmonic oscillator problem')

