%example4
%show how different PPM can be used:
format long;

disp('****************************')
disp('Harmonic oscillator problem.')
disp('****************************')


%define a Schrodinger problem (a truncated Harmonic oscillator)
sch=schrod('x^2',0,10,1,0,1,0);

%Is this problem singular?
get(sch,'infostring')

%we will calculate the first 10 eigenvalues
emin=0;
emax=9;
indices=true;

%with this tolerance:
tol=1e-10;

%first with the CPM{12,10} method
cp= cpm12_10(sch,tol);
%The reference_potential looks then like this:
plot_partition(cp);

%calculate eigenvalues:
E=get_eigenvalues(cp,emin,emax,indices);
disp('First 10 eigenvalues computed with the CPM{12,10} method:')
disp(sprintf('k \t  E_k \t\t\t\t  estimated error'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e')])
end

%now with the LPM[4,2] method:
lp = lpm10(sch,tol);
plot_partition(lp);
E=get_eigenvalues(lp,emin,emax,indices);
disp('First 10 eigenvalues computed with the LPM[4,2] method:')
disp(sprintf('k \t  E_k \t\t\t\t  estimated error'))
for i=1:length(E.eigenvalues)
    disp([num2str(E.indices(i),'%4.0f\t') '  ' num2str(E.eigenvalues(i),'% 16.12f\t') '  ' num2str(E.errors(i),'%+5.2e')])
end
