function E=calculate_eigenvalues(emin,emax,nmin,nmax)
%@CPM\CALCULATE_EIGENVALUES
%[eigvbasic,index,err,status]=calculate_eigenvalues(emin,emax,nmin,nmax)
% search of the eigenvalues with index n from nmin to nmax 
global cpm_child
bp = get(cpm_child,'bp'); %basic partition
rp = get(cpm_child,'rp'); %reference partition
Vmin = get(cpm_child,'Vmin');
Vmax = get(cpm_child,'Vmax');
tol = get(cpm_child,'tol'); %input tolerance
truncated = get(cpm_child,'truncated'); %true for a singular problem
distorted = isa(get(cpm_child,'schrod'),'distorted_coulomb');

e_low = emin;
n_low = nmin;
number_eigv = 0; 
h_en = (emax-emin)/(5*(nmax-nmin+1));
h_mem = h_en;
len=length(bp.h);

if truncated && nmax >nmin
   if nmax-nmin <50   
      nsteps=(nmax-nmin)/4;
   else
      nsteps=(nmax-nmin)/10; 
   end
   estep=abs((emax-emin)/nsteps);
   et=emin+estep;
   [trunca,truncb,inita,initb]=get_truncation_points(bp,et,bp.imatch,bp.imatch,distorted);
else
    trunca=1;
    truncb=len;
end
if isa(cpm_child,'cpm18_16')
    cpm18=true;
else
    cpm18=false;
end

%preallocation
eigv = zeros(1,nmax-nmin+1);
index = zeros(1,nmax-nmin+1);
indexr = zeros(1,nmax-nmin+1);
err = zeros(1,nmax-nmin+1);
status = zeros(1,nmax-nmin+1);

for n=nmin:nmax  
	number_eigv = number_eigv + 1;
    % an energy interval [e_low,e_up] is searched such that any e in
    % this interval represents a good initial guess for the Newton
    % iteration.
    while true
      h_en = abs(min(h_mem,h_en));  
	  e_up = e_low + h_en;
      if truncated && (trunca > 1 || truncb < len)
          %get new truncation points
        while e_up > et  
         et0=et;   
         et=et+estep;
         truncb0=truncb;
         inita0=inita;
         initb0=initb;
         [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita,initb,distorted);      
         if truncb-truncb0 >20
             estep=estep/2;
             et=et0+estep;
             [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita0,initb0,distorted);
         end
        end
      end
      n_up = locate_eigenvalue(e_up,bp,Vmin,trunca,truncb); %return index of e_up
      if h_en > 0 && n_up < n_low
          % something wrong
          e_up = e_low + 2*h_en;
          n_up = locate_eigenvalue(e_up,bp,Vmin,trunca,truncb);
      end
      rz2 = n_up-n-1;
      if rz2 < 0 
	     e_low = e_up;
         n_low = n_up;
         h_en = h_en*2;
      else
	     break; % stop while	
      end
    end %while
     % calculate the eigenvalue corresponding to the basic
     % partition:
     [ev,e_low,n_low,e_up,n_up,evpruf0,fail] = ...
        calculate_eigenvalue(e_low,n_low,e_up,n_up,n,bp,Vmin,Vmax,trunca,truncb);
     if fail && (e_low >= e_up || round(n_low -1) ~= n) 
         eigv(number_eigv) = ev;
         index(number_eigv) = n;
         indexr(number_eigv) = -1;
        continue;
     end
      % check whether ev is actually inside [e_low,e_up]:
      if (ev-e_low)*(ev-e_up) > 0 || round(evpruf0 -1) ~= n
          [ev,e_low,n_low,e_up,n_up,evpruf0]=...
              calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,bp,Vmin,Vmax,trunca,truncb);
      end
	 % compute reference eigenvalue: the basic one is used as
     % initial guess
      if cpm18
        [evx,evpruf]= calculate_eigenvalue(ev,rp,Vmin,Vmax,2*trunca-1,2*truncb-1);
      else
        [evx,evpruf]= calculate_eigenvalue(ev,rp,Vmin,Vmax,trunca,truncb);
        if round(evpruf -1) ~= n
          [evx,e_low,n_low,e_up,n_up,evpruf]=...
              calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,rp,Vmin,Vmax,trunca,truncb);
        end
      end
      eigv(number_eigv) = ev;
      index(number_eigv) = round(evpruf0 - 1);
      indexr(number_eigv) = round(evpruf - 1);	
	  err(number_eigv) = (ev-evx); %error estimation
      if abs(ev-evx) < eps*abs(evx)
          if ev<evx
               err(number_eigv) = -eps*abs(evx);
          else
               err(number_eigv) = eps*abs(evx);
          end
      end
      if abs(err(number_eigv))<eps
          err(number_eigv)=eps;
      end
      if abs(err(number_eigv)) > tol*10
          eigv(number_eigv) = ev;
          if abs(err(number_eigv)) > tol*1e4
              eigv(number_eigv)= calculate_eigenvalue(min(evx,ev),rp,Vmin,Vmax,trunca,truncb);
              if abs(eigv(number_eigv)-evx) < abs(eigv(number_eigv)-ev)
                   err(number_eigv)=eigv(number_eigv)-evx;
              else
                   err(number_eigv)=eigv(number_eigv)-ev;
              end
          end
      end
      h_en = min(e_up-e_low,h_en);
	  e_low = e_up;
	  n_low = n_up;
      pause(1e-200) % stop the calculation in the GUI?
      load stop
      if stop
        error('The calculation was interrupted');
      end
end %for


%   The results of the previous calculation are now scanned once again
%   and corrected if necessary. 
trunca=1;
truncb=length(bp.h);
for i=1:number_eigv
    if index(i) ~= indexr(i) || index(i)  ~= nmin+i-1
	  n = nmin+i-1;
      index(i) = n;
	  e1 = eigv(i);
	  h_en = 2*tol; 
	  while true
	    e1 = e1 - h_en;
        pruf1 = locate_eigenvalue(e1,bp,Vmin,trunca,truncb);	       	
	    h_en = 2*h_en;
        if (pruf1-n-1) <= 0
            break;
        end
	  end %while
      while true
	    e2 = e1 + h_en/2;
        pruf2 = locate_eigenvalue(e2,bp,Vmin,trunca,truncb);
        if (pruf2-n-1) >= 0 
          [ev0,e10,pruf10,e20,pruf20,pruf0]= calculate_eigenvalue_lin_interpolation(e1,pruf1,e2,pruf2,n,bp,Vmin,Vmax,trunca,truncb);
          [ev,e1,pruf1,e2,pruf2,pruf,fail] = calculate_eigenvalue(e1,pruf1,e2,pruf2,n,bp,Vmin,Vmax,trunca,truncb);
          if abs(ev0- ev) > tol
              if abs(round(pruf-1)-n) > abs(round(pruf0)-1)
                  ev = ev0;
                  pruf = pruf0;
              end
          end
          if indexr(i) ~= -1
              indexr(i) = round(pruf-1);
          end
          err(i) = (ev-eigv(i))*1.005;
          
          if abs(ev-eigv(i)) < eps
              err(i)=eps;
          end
          eigv(i) = ev;
          break;    
        else
	      e1 = e2;
	      h_en = 2*h_en;
	      pruf1 = pruf2;
        end
      end %while
    end
end %for

for i=1:number_eigv
    if indexr(i) == -1
        status(i)=1;
        continue;
    elseif indexr(i) ~= index(i)
        status(i)=2;
    elseif index(i)~= nmin+i-1
        status(i)=3;
    else
        status(i)=0;
        continue;
    end
    if status(i) == 2 && index(i) ~= nmin+i-1 
        status(i) = 4;
    end
end %for
E.eigenvalues=eigv;
E.errors=err;
E.indices=index;
E.status=status;


%-------------------------------------------------------------------------
function [trunca,truncb,inita,initb]=get_truncation_points(bp,E,inita,initb,distorted)
%returns the numbers of the intervals where the integration interval should be truncated
%using WKB-approach
trunca=1;
truncb=length(bp.h);
  sums=0;
  V0=bp.V0;
  h=bp.h;
  for i=initb:length(h)-1 
    v0i=V0(i);  
    if v0i>=E && V0(i+1) > v0i
      if sums==0
          initb=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 18
        truncb=i;
        for j=length(h):-1:i+1
            if V0(j)>=v0i && V0(j-1)<v0i
                truncb=j;
            end
        end
        break;
    end
  end
if ~distorted
  sums=0;
  for i=inita:-1:2
    v0i=V0(i);  
    if v0i>=E && V0(i-1) > v0i
      if sums==0
          inita=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 18
        trunca=i;
        for j=1:i-1
            if V0(j)>=v0i && V0(j+1)<v0i
                trunca=j;
            end
        end
        break;
    end
  end
end


%--------------------------------------------------------------------------
function [varargout]= calculate_eigenvalue(varargin)
%@CPM\CALCULATE_EIGENVALUE
% (1) [e,e1,n1,e2,n2,pruf,small]= calculate_eigenvalue(e1,n1,e2,n2,n,part,vmin,vmax,trunca,truncb)
% or:
% (2) [e,pruf]= calculate_eigenvalue(e,part,vmin,vmax,trunca,truncb)
%
%   This calculates the eigenvalue (e_n) by the Newton iteration procedure.
%    (1): first an initial guess is generated for the Newton procedure.
%    (2): the start point for the Newton iteration is already given in the
%    input (varargin{1})
%
%     e1 / e2     - in input: lower and upper limits for the eigenvalue e_n 
%                 - in output: sharper lower and upper limits
%     n1 / n2     - these are the pruefer theta/pi values corresponding to e1 and e2.
%     part        - data concerning the partition of the equation domain.
%     tol         - the requested tolerance.
%     vmin /vmax  - approximations of the min. and max. of the potential 
%
%      pruf       - the Pruefer theta/pi corresponding to the eigenvalue e.
%      small      - small = false indicates that there was no difficulty 
%                          to localise ev. 
%                   small = true says that the Pruefer phase varies
%                          with e so violently that the original [e1,e2] had
%                          to be diminished up to a width consistent with 
%                          eps.
global schrod_object cpm_child 
small = false;
if nargin == 10
 % init_ev : generates a value ev in [e1,e2] to be used as the initial guess
 % for the Newton procedure 
   [e,varargout{2},varargout{3},varargout{4},varargout{5},small]...
       = init_ev(varargin{1},varargin{2},varargin{3},varargin{4},varargin{5},...
       varargin{6},varargin{7},varargin{8},varargin{9},varargin{10});
   varargout{7}=small;
else
    % e = initial guess 
    e = varargin{1};
end
truncb = varargin{nargin};
trunca = varargin{nargin-1};
vmin = varargin{nargin-3};
tol0=get(cpm_child,'tol');
part = varargin{nargin-4};
if isa(schrod_object,'distorted_coulomb')
    distorted=true;
else 
    distorted=false;
end
newton=1e300;
if ~small || (varargout{2} ~= varargout{4} && round(varargout{3}-1) == varargin{5})
   if distorted
     [yi,yei,yf,yef,truncb] = get_initConditions(e,part);
   else
     [yi,yf] = get_initConditions;
     yei=[0; 0];
     yef=[0; 0];
   end
   it = 0;
   match=min(part.imatch,truncb-1);  %matching point
     
   while true
       %Newton iteration
    it = it+1;
    if it >= 70
        break;
    end
    scalev = max(1,abs(e));
	tol = scalev*tol0;
    %forward:
    [yl,yel] = propagate_solution(e,yi,yei,trunca,match,part,vmin,true);
    %backward:
    [yr,yer] = propagate_solution(e,yf,yef,match+1,truncb,part,vmin,false);
    %the data obtained at each side of xmatch are combined:
    yl1=yl(1);
    yr1=yr(1);
    yl2=yl(2);
    yr2=yr(2);
    yel1=yel(1);
    yer1=yer(1);
    yel2=yel(2);
    yer2=yer(2);
    phi = yl1*yr2-yr1*yl2; %miss-match 
    dphi = yel1*yr2+yl1*yer2-yer1*yl2-yr1*yel2;
	aux = yr1*yl1;
	auxp = yr2*yl2;
	daux = yer1*yl1+yr1*yel1;
	dauxp = yer2*yl2+yr2*yel2;
    if isinf(phi) %avoid overflow
        h1=(yl1/10^150)*(yr2/10^150);
        h2=(yr1/10^150)*(yl2/10^150);
        phi = (h1-h2);
        dphi = (yel1/10^150)*(yr(2)/10^150)+(yl1/10^150)*(yer2/10^150)-...
            (yer1/10^150)*(yl2/10^150)-(yr1/10^150)*(yel2/10^150);
        aux = (yr1/10^150)*(yl1/10^150);
	    auxp = (yr2/10^150)*(yl2/10^150);
	    daux = (yer1/10^150)*(yl1/10^150)+(yr1/10^150)*(yel1/10^150);
	    dauxp = (yer2/10^150)*(yl2/10^150)+(yr2/10^150)*(yel2/10^150);
    end
    newton0=newton;
    if abs(aux) >= abs(auxp)
       newton = -phi*aux/(dphi*aux-daux*phi);
    else %(abs(aux) < abs(auxp))
       newton = -phi*auxp/(dphi*auxp-dauxp*phi);
    end
    if abs(dphi)>abs(phi)
       testnewton=-phi/dphi;
       if newton*testnewton<0
           newton=testnewton;
       end
    end
    if isnan(newton)
        newton= abs(e)/100000;
    end
    if abs(newton)>abs(newton0) && abs(newton/newton0)>10
        break;
    end
	e = e + newton;
    if abs(newton) <= tol
        break;
    end
    if distorted
       [yi,yei,yf,yef,truncb] = get_initConditions(e,part);
    end
  end %while  
end
varargout{1} = e;
if nargout == 2
   varargout{2} = locate_eigenvalue(e,part,vmin,trunca,truncb);
else
    varargout{6} = locate_eigenvalue(e,part,vmin,trunca,truncb);
end

%--------------------------------------------------------------------------
function [ev,e1,pruf1,e2,pruf2,small]= init_ev(e1,pruf1,e2,pruf2,n,part,vmin,vmax,trunca,truncb)
%@CPM\INIT_EV
% [ev,e1,pruf1,e2,pruf2,small]= init_ev(e1,pruf1,e2,pruf2,n,part)
%  For given input e1 and e2 for which it is known that the eigenvalue
%  e_n is between e1 and e2, function init_ev furnishes a value ev to be 
%  used as the initial value in the Newton procedure of
%  calculate_eigenvalue for locating e_n. 
%  ev is calculated by alternative use of the linear interpolation and 
%  of halving.
%
%   INPUT:
%      n                 - the value of the eigenvalue index
%    part                - the set of data corresponding to the 
%                          partition of the domain of the equation.
%     e1,e2              - lower and upper limits for the 
%                          eigenvalue e_n;
%     pruf1,pruf2       -  these are the pruefer 
%                          theta/pi values corresponding to e1 and e2.
%	                       Conditions pruf1-n-1 < 0 and pruf2-n-1 > 0 are 
%                          assumed to be satisfied in the input.
%
%   OUTPUT:
%      ev                - the approximate value of the eigenvalue e_n.
%      small             - small = false indicates that there was no difficulty 
%                          to localise ev. 
%                          small = true says that the Pruefer phase varies
%                          with e so violently that the original [e1,e2] had
%                          to be diminished up to a width consistent with 
%                          eps.
%      e1,e2             - in output, sharper lower and upper limits.
%      pruf1,pruf2       - the pruefer theta/pi values corresponding to e1 and e2.

small = false;
phi1 = pruf1-(n+1);
phi2 = pruf2-(n+1);
it = 0;
while true
   ec = (phi2*e1-phi1*e2)/(phi2-phi1);  % linear interpolation
   if (abs(phi1) <= 0.1 && abs(phi2) <= 0.1)  %#1
       ev = ec;
	   return;		    	
   else
       scalev = max(1,max(abs(vmin-ec),abs(vmax-ec)));
       if scalev*eps  < (e2-e1) %#2
            it = it + 1;
            if rem(it,2) == 0  %even 
                ec = (e1+e2)/2;
            end
            [prufc,enew] = locate_eigenvalue(ec,part,vmin,trunca,truncb);
            if (abs(phi1)+abs(phi2) < 0.25 && (enew-e1)*(enew-e2) < 0) 	%#3
	           ev = enew;
	           return;
	        else
	           phic = prufc-(n+1);
               if (phic < (phi1-(n+1)*eps)) || (phic > (phi2+(n+1)*eps))
         	        warning(sprintf('INIT_EV: Inaccurate computation of the Pruefer phase is detected at e = %s ',ec))
               end
               if phi1*phic <= 0 
	              e2 = ec;
	              phi2 = phic;
	              pruf2 = prufc;
	           else
		          e1 = ec;
		          phi1 = phic;	
		          pruf1 = prufc;
               end
             end	%#3
             if abs(e1 - e2) < 2*n*eps
                 ev = enew;
                 return;
             end
	    else %#2
	         small = true;		
	         ev = ec;
	         return	      
        end%#2
    end %#1
 end %while

 %-------------------------------------------------------------------------
 
 function [e,e_low,n_low,e_up,n_up,pruf]=calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,part,vmin,vmax,trunca,truncb)
%@CPM\CALCULATE_EIGENVALUE2
% [e,e_low,e_up,pruf]=calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,part,vmin,vmax,trunca,truncb)
%   For given index n and lower and upper limits e_low and e_up, this 
%   function calculates the eigenvalue e_n (argument e) by using 
%   the Pruefer phase function. Since this function uses linear 
%   interpolation, it is more robust but slower than calculate_eigenvalue(). 
%   For these reasons calculate_eigenvalue_lin_interpolation() is called only 
%   when there are some hints that the results produced by 
%   calculate_eigenvalue() may be inaccurate. 

tol=eps;
p_low = n_low-(n+1);
p_up = n_up-(n+1);
it=0;	
while true
    pause(1e-200) % stop the calculation in the GUI?
    load stop
    if stop
        error('The calculation was interrupted');
    end
    it = it + 1;
	e = (p_up*e_low-p_low*e_up)/(p_up-p_low); %linear interpolation
    if rem(it,2) == 0
        e = (e_low+e_up)/2;
    end
    pruf = locate_eigenvalue(e,part,vmin,trunca,truncb);
	pc = pruf-(n+1);
    if pc < p_low-(n+1)*eps  || pc > p_up+(n+1)*eps
       warning(['calculate_eigenvalue_lin_interpolation: Inaccurate computation of the Pruefer phase is detected at e = ' e ])
    end
	scalev = max(1,max(abs(vmin-e),abs(vmax-e)));
	tol = scalev*tol;
    if abs(e_up-e_low) > tol %#1
        if p_low*pc < 0 %#2
         p_up = pc;
	     e_up = e;
         n_up = pruf;
	    else %#2
          if p_low == 0
            e = e_low;
            pruf = n_low;
            return;
          end
          if pc == 0
            return;
          end
	      e_low = e;
          n_low = pruf;
	      p_low = pc;
        end %#2
	else %#1
	  break;
    end %#1
end %while
