function E=calculate_eigenvalues_singular(emin,emax,nmin,nmax)
%@CPM\CALCULATE_EIGENVALUES_SINGULAR
%[eigvbasic,index,err,status]=calculate_eigenvalues_singular(emin,emax,nmin,nmax)
% search of the eigenvalues with index n from nmin to nmax 

global cpm_child
bp = get(cpm_child,'bp'); %basic partition
rp = get(cpm_child,'rp'); %reference partition
ref_cpm = get(cpm_child,'ref_cpm');
Vmin = get(cpm_child,'Vmin');
Vmax = get(cpm_child,'Vmax');
tol = get(cpm_child,'tol');
truncated = get(cpm_child,'truncated');
schrod=get(cpm_child,'schrod');
distorted = isa(schrod,'distorted_coulomb');
transf_slp=isa(schrod,'transformed_slp');
singularities=get(schrod,'singularities');
x_min=get(schrod,'min');
x_max=get(schrod,'max');
sing_a=false;
sing_b=false;
if (singularities(1)~= x_min) && ~isinf(singularities(1))
    sing_a=true; %then endpoint a is singular (not infinite)
end
if (singularities(2)~= x_max) && ~isinf(singularities(2))
    sing_b=true; %then endpoint b is singular (not infinite)
end   
  
e_low = emin;
n_low = nmin;
number_eigv = 0; 
h_en = (emax-emin)/(5*(nmax-nmin+1));
h_mem = h_en;
len=length(bp.h);

if truncated && nmax >nmin
   if nmax-nmin <50   
      nsteps=(nmax-nmin)/4;
   else
      nsteps=(nmax-nmin)/10; 
   end
   estep=abs((emax-emin)/nsteps);
   et=emin+estep;
   [trunca,truncb,inita,initb]=get_truncation_points(bp,et,bp.imatch,bp.imatch,distorted);
else
    trunca=1;
    truncb=len;
end
if isa(cpm_child,'cpm18_16')
    cpm18=true;
else
    cpm18=false;
end

for n=nmin:nmax  
	number_eigv = number_eigv + 1;
    % an energy interval [e_low,e_up] is searched such that any e in
    % this interval represents a good initial guess for the Newton
    % iteration.
    while true
      h_en = abs(min(h_mem,h_en));  
	  e_up = e_low + h_en;
      if truncated && (trunca > 1 || truncb < len)
        while e_up > et  
         et0=et;   
         et=et+estep;
         truncb0=truncb;
         inita0=inita;
         initb0=initb;
         [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita,initb,distorted);
         if truncb-truncb0 >20
             estep=estep/2;
             et=et0+estep;
             [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita0,initb0,distorted);
         end
        end
      end
      n_up = locate_eigenvalue(e_up,bp,Vmin,trunca,truncb);
      if h_en > 0 && n_up < n_low
          % something wrong
          e_up = e_low + 2*h_en;
          n_up = locate_eigenvalue(e_up,bp,Vmin,trunca,truncb);
      end
      rz2 = n_up-n-1;
      if rz2 < 0 
	     e_low = e_up;
         n_low = n_up;
         h_en = h_en*2;
      else
	     break; % stop while	
      end
    end %while
     % calculate the eigenvalue corresponding to the basic
     % partition:
      [ev,e_low,n_low,e_up,n_up,evpruf0] = ...
        calculate_eigenvalue(e_low,n_low,e_up,n_up,n,bp,Vmin,Vmax,trunca,truncb);
      % check whether ev is actually inside [e_low,e_up]:
      if (ev-e_low)*(ev-e_up) > 0
          [ev,e_low,n_low,e_up,n_up,evpruf0]=...
              calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,bp,Vmin,Vmax,trunca,truncb);
      end
      
      truncbs=truncb;
      truncas=trunca;
      
      % ev = first approximation of eigenvalue 
      
      if abs(evpruf0-1-n)>0.5
          error('Error: Some problems were detected')
      end
      
      %the (truncated) endpoint is moved closer to the singular endpoint to
      %compute a new eigenvalue approximation
      
      bp0=bp;
      rp0=rp;
      truncbsr=0;
      warn=false;
      if sing_a
       %endpoint a is singular
       %search good truncation point
       Vmins=Vmin;
       ev_prev=ev-1;
       ev_save=ev;
       pruf_save=n+1;
       x_mins=x_min;
       M=4;
       while true
          ev_prev_prev=ev_prev; 
          ev_prev=ev;
          x_min0=x_mins;
          h = min((x_min0-singularities(1))/M,bp0.h(1));
          if h < eps && (ev_prev>ev_prev_prev || abs(ev_prev_prev-ev_prev)>abs(ev_prev/50))
              ev=ev_prev_prev;
              warn=true;
              warning('Double precision is not sufficient to calculate this eigenvalue accurately');
              break;
          end
          x_mins = x_min0-h; %new truncated endpoint
          % add [x_mins,x_min0] to partition
          [V0,V] = calculate_V(x_mins,h); 
          [cu,cup,cv,cvp] = get_coeff(cpm_child,V);
          [rcu,rcup,rcv,rcvp] = get_coeff(ref_cpm,V);
          bps.h=[h bp0.h];
          bps.V0=[V0 bp0.V0];
          bps.cu=[cu; bp0.cu];
          bps.cup=[cup; bp0.cup];
          bps.cv=[cv; bp0.cv];      
          bps.cvp=[cvp; bp0.cvp];      
          rps.cu=[rcu; rp0.cu];
          rps.cup=[rcup; rp0.cup];
          rps.cv=[rcv; rp0.cv];      
          rps.cvp=[rcvp; rp0.cvp]; 
          rps.h=[h rp0.h];
          rps.V0=[V0 rp0.V0];
          [Vmins,bps.imatch]=min(bps.V0);
          truncbs=truncbs+1;
          %compute new eigenvalue approximation over the new (larger)
          %partition:
          [ev,evpruf0]= calculate_eigenvalue(ev_prev,bps,Vmins,Vmax,1,truncbs);
          if ~transf_slp && ev > ev_prev
              [ev,e_low,n_low,e_up,n_up,evpruf0]=...
              calculate_eigenvalue_lin_interpolation(e_low,n_low,e_up,n_up,n,bps,Vmins,Vmax,1,truncbs);
           end
          if abs(evpruf0-pruf_save)>0.2 && M<20 %dan is het toegevoegde intervalleke niet klein genoeg
              truncbs=truncbs-1;
              evpruf0=pruf_save;
              ev=ev_save;
              x_mins = x_min0;
              M=M*2;
              bps=[];
              rps=[];
              continue;
          end
          bp0=bps;
          rp0=rps;
          bps=[];
          rps=[];
          pruf_save=evpruf0;
          ev_save=ev;
          tols=max(min(tol,tol*abs(ev)),1e-15);
          if abs(ev-ev_prev)<tols && abs(ev_prev_prev-ev_prev)<tols 
             if cpm18
                error('this singular problem can not yet be solved with the CPM{18,16} method, try CPM{16,14}') 
             else
                %adapt reference partition: 
                h1=min((x_mins-singularities(1))/M,bp0.h(1));
                x_mins=x_mins-h1;
                [V01,V] = calculate_V(x_mins,h1);
                [cu1,cup1,cv1,cvp1] = get_coeff(ref_cpm,V);
                h2=min((x_mins-singularities(1))/M,bp0.h(1));
                x_mins=x_mins-h2;
                [V02,V] = calculate_V(x_mins,h2);
                [cu2,cup2,cv2,cvp2] = get_coeff(ref_cpm,V);
                rp0.cu=[cu1; cu2; rp0.cu];
                rp0.cup=[ cup1; cup2; rp0.cup];
                rp0.cv=[ cv1; cv2; rp0.cv ];      
                rp0.cvp=[cvp1; cvp2; rp0.cvp];     
                rp0.h = [h1 h2 rp0.h];  
                rp0.V0 = [V01 V02 rp0.V0];  
                [Vmins,rp0.imatch]=min(rp0.V0);
                truncbsr=truncbs+2;
             end
             break; %truncated endpoint is close enough to 
             %the singular endpoint to obtain a good eigenvalue approximation
         end
       end
      end
        
      %now the same for endpoint b:
       if sing_b
       %endpoint b is singular
       %search good truncation point
       Vmins=Vmin;
       ev_prev=ev-1;
       ev_save=ev;
       pruf_save=n+1;
       x_maxs=x_max;
       M=4;
       while true 
          ev_prev_prev=ev_prev; 
          ev_prev=ev;
          x_max0=x_maxs;
          h = min((singularities(2)-x_max0)/M,bp0.h(end)); 
          if h < eps && (ev_prev>ev_prev_prev || abs(ev_prev_prev-ev_prev)>abs(ev_prev/50))
              ev=ev_prev_prev;
              warn=true;
              warning('Double precision is not sufficient to calculate this eigenvalue accurately');
              break;
          end
          x_maxs = x_max0+h; %move closer to the singular endpoint
          %add interval to the partition:
          [V0,V] = calculate_V(x_max0,h);
          [cu,cup,cv,cvp] = get_coeff(cpm_child,V);
          [rcu,rcup,rcv,rcvp] = get_coeff(ref_cpm,V);
          bps.h=[bp0.h h];
          bps.V0=[bp0.V0 V0];
          bps.cu=[bp0.cu; cu];
          bps.cup=[bp0.cup; cup];
          bps.cv=[bp0.cv; cv];      
          bps.cvp=[bp0.cvp; cvp];      
          rps.cu=[rp0.cu; rcu];
          rps.cup=[rp0.cup; rcup];
          rps.cv=[rp0.cv; rcv];      
          rps.cvp=[rp0.cvp; rcvp];
          rps.h=[rp0.h h];
          rps.V0=[rp0.V0 V0];
          [Vmins,bps.imatch]=min(bps.V0);
          truncbs=truncbs+1;
          %compute new eigenvalue approximation:
          [ev,evpruf0]= calculate_eigenvalue(ev_prev,bps,Vmins,Vmax,truncas,truncbs);
          if abs(evpruf0-pruf_save)>0.2 && M<20 %dan is het toegevoegde intervalleke niet klein genoeg
              truncbs=truncbs-1;
              evpruf0=pruf_save;
              ev=ev_save;
              x_maxs = x_max0;
              M=M*2;
              bps=[];
              rps=[];
              continue;
          end
          bp0=bps;
          rp0=rps;
          bps=[];
          rps=[];
          pruf_save=evpruf0;
          ev_save=ev;
          tols=max(min(tol,tol*abs(ev)),1e-15);
          if abs(ev-ev_prev)<tols && abs(ev_prev_prev-ev_prev)<tols 
              %a good truncated endpoint has been found
             if cpm18
                error('this singular problem can not yet be solved with the CPM{18,16} method, try CPM{16,14}') 
             else
                 %adapt reference parition
                h1=min((singularities(2)-x_maxs)/M,bp0.h(end));
                x_maxs=x_maxs+h1;
                [V01,V] = calculate_V(x_maxs-h1,h1);
                [cu1,cup1,cv1,cvp1] = get_coeff(ref_cpm,V);
                h2=min((singularities(2)-x_maxs)/M,bp0.h(end));
                x_maxs=x_maxs+h2;
                [V02,V] = calculate_V(x_maxs-h2,h2);
                [cu2,cup2,cv2,cvp2] = get_coeff(ref_cpm,V);
                rp0.cu=[rp0.cu; cu1; cu2];
                rp0.cup=[rp0.cup; cup1; cup2];
                rp0.cv=[rp0.cv; cv1; cv2 ];      
                rp0.cvp=[rp0.cvp; cvp1; cvp2];     
                rp0.h = [rp0.h h1 h2];  
                rp0.V0 = [rp0.V0 V01 V02];  
                [Vmins,rp0.imatch]=min(rp0.V0);
                if sing_a
                   truncbsr=truncbs+4;
                else
                   truncbsr=truncbs+2;
                end
             end
             break;
         end
       end
      end
      
	 % compute reference eigenvalue: the basic one is used as
     % initial guess
      if cpm18
        [evx,evpruf]= calculate_eigenvalue(ev,rp0,Vmins,Vmax,2*trunca-1,2*truncb-1);
      else
          if truncbsr==0
              truncbsr=length(rp0.h);
              [Vmins,rp0.imatch]=min(rp0.V0);
          end
        [evx,evpruf]= calculate_eigenvalue(ev,rp0,Vmins,Vmax,truncas,truncbsr);
      end
      eigv(number_eigv) = ev;
      index(number_eigv) = round(evpruf0 - 1);
      if warn
        indexr(number_eigv) = -1;	  
      else
        indexr(number_eigv) = round(evpruf - 1);	
      end
	  err(number_eigv) = (ev-evx)*1.005;%*1.05;
      pwr=max(log(abs(evx))/log(10),0);
      if abs(evx-ev) < eps*10^(pwr) && tol>1e-12
          err(number_eigv)=eps*10^pwr;
          if number_eigv > 1
              err(number_eigv)=(err(number_eigv)+err(number_eigv-1))/2;
          end
      elseif abs(evx-ev) < eps*10^(max(pwr,0))
          err(number_eigv)=eps*10^(max(pwr,0));
      elseif abs(evx-ev) < eps*10 
          err(number_eigv) = err(number_eigv)*2;
      end 
      h_en = min(e_up-e_low,h_en);
	  e_low = e_up;
	  n_low = n_up;
      pause(0) % stop the calculation in the GUI?
      load stop
      if stop
        error('The calculation was interrupted');
      end
end %for

for i=1:number_eigv
    if indexr(i) == -1
        status(i)=1;
        continue;
    elseif indexr(i) ~= index(i)
        status(i)=2;
    elseif index(i)~= nmin+i-1
        status(i)=3;
    else
        status(i)=0;
        continue;
    end
    if status(i) == 2 && index(i) ~= nmin+i-1 
        status(i) = 4;
    end
end %for
E.eigenvalues=eigv;
E.errors=err;
E.indices=index;
E.status=status;


function [trunca,truncb,inita,initb]=get_truncation_points(bp,E,inita,initb,distorted)
trunca=1;
truncb=length(bp.h);
  sums=0;
  V0=bp.V0;
  h=bp.h;
  for i=initb:length(h)-1 
    v0i=V0(i);  
    if v0i>=E && V0(i+1) > v0i
      if sums==0
          initb=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 15
        truncb=i;
        for j=length(h):-1:i+1
            if V0(j)>=v0i && V0(j-1)<v0i
                truncb=j;
            end
        end
        break;
    end
  end
if ~distorted
  sums=0;
  for i=inita:-1:2
    v0i=V0(i);  
    if v0i>=E && V0(i-1) > v0i
      if sums==0
          inita=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 15
        trunca=i;
        for j=1:i-1
            if V0(j)>=v0i && V0(j+1)<v0i
                trunca=j;
            end
        end
        break;
    end
  end
end

