function [varargout]=get_initConditions(varargin)
% @PPM\GET_INITCONDITIONS
% [yi,yf]=get_initConditions
% [yi,yei,yf,yef,number_interval]=get_initConditions(e,part)
% [yi,yei,yf,yef,number_interval,phase]=get_initConditions(e,part)
% the boundary conditions are converted as initial conditions for the
% forwards and backwards integration of the Schrodinger equation from the
% two endpoints to the matching point.

global schrod_object;
if nargin == 0
  if isa(schrod_object,'schrod')
    a0 = get(schrod_object,'a0');
    b0 = get(schrod_object,'b0');
    a1 = get(schrod_object,'a1');
    b1 = get(schrod_object,'b1');
  elseif isa(schrod_object,'distorted_coulomb')
      a0 = 1;
      b0 = 0;
      a1 = 1;
      b1 = 0;
  end
  sq1 = sqrt(a0^2+b0^2);
  sq2 = sqrt(a1^2+b1^2);
  yi = [-b0 / sq1;a0 / sq1];
  yf = [-b1 / sq2;a1 / sq2];
  if (yi(1) <= 0)
    varargout{1} = -yi;
  else
    varargout{1} = yi;  
  end
  if (yf(1) <= 0)
    varargout{2} = -yf;
  else
    varargout{2} = yf;  
  end
elseif isa(schrod_object,'distorted_coulomb')
    S=get(schrod_object,'S');
    Si=S(1e10);
    R=get(schrod_object,'R');
    classi=get(schrod_object,'classification'); 
    %get truncation point to replace the infinite endpoint:
    if (strcmp(char(R),'0') && (classi.cutoff==0 && classi.category==2)) && ~isinf(Si) && ~isnan(Si) %&& isempty(strfind(S,'x'))
       part=varargin{2};
       [r,number_intervals]=get_turning_point(varargin{1},part,get(schrod_object,'min'),S,R);
       if number_intervals<length(part.h) && S(r) == Si 
         varargout{5}=number_intervals;
         [varargout{3},varargout{4}] = get_asymptotic_solution(varargin{1},r,-Si,get(schrod_object,'l'));
       else
         [varargout{3},varargout{4},varargout{5}] = laststep(varargin{1},varargin{2});  
       end
    else
        [varargout{3},varargout{4},varargout{5}] = laststep(varargin{1},varargin{2});
    end
    %use special algorithm to deal with the singularity in the origin:
    rmin=get(schrod_object,'min');
    if rmin>get(schrod_object,'max')/50
      if nargout == 6
        [varargout{1},varargout{2},varargout{6}] = firststep(varargin{1},rmin); 
      elseif nargout == 5
       [varargout{1},varargout{2}] = firststep(varargin{1},rmin);
      end
    else
      if nargout == 6
        [varargout{1},varargout{2},varargout{6}]=initstep(varargin{1},get(schrod_object,'min'),get(schrod_object,'Vs'),get(schrod_object,'l'));
      elseif nargout == 5
        [varargout{1},varargout{2}]=initstep(varargin{1},get(schrod_object,'min'),get(schrod_object,'Vs'),get(schrod_object,'l'));
      end
    end
end


%--------------------------------------------------------------------------
function [varargout]=initstep(e,r,Vs,l)
%specially tuned algorithm consistent with the singular nature around the
%origin
global cpm_child
max_number_corrections=15;
Vm1=Vs(1);
V0=Vs(2);
V1=Vs(3);
V2=Vs(4);
y0=r^(l+1);
y0p=(l+1)*r^l;
Vbar=V0-e;
i=1;
Vs=[Vm1 Vbar V1 V2]';
Vsd=[0 -1 0 0]';
y=y0;
ye=0;
maxn=max_number_corrections*4;
tmpB=zeros(1,maxn);
tmpBe=zeros(1,maxn);
for p=i:i*4
       tmp=zeros(1,4);
       tmp(p)=1;
       tmpB(p)=(tmp*Vs)/((l+p)*(l+p+1)-l*(l+1));
       tmpBe(p)=(tmp*Vsd)/((l+p)*(l+p+1)-l*(l+1));
       y=y+tmpB(p)*r^(l+p+1);
       ye=ye+tmpBe(p)*r^(l+p+1);
       %yep=yep+(l+p+1)*Be(i,p)*r^(l+p);
end
y1p=Vm1*r^(l+2)*(l+2)/r/(2*l+2)+Vbar*r^(l+3)*(l+3)/r/(6+4*l)+V1*r^(l+4)*(l+4)/r/(6*l+12)+V2*r^(l+5)*(l+5)/r/(8*l+20);
yp=y0p+y1p;
yep=-r^(l+3)*(l+3)/r/(4*l+6);

Vmatrix=diag(Vm1*ones(4,1));
Vmatrix(1,2:4)=[Vbar V1 V2];
Vmatrix(2,3:4)=[Vbar V1];
Vmatrix(3,4)=Vbar;
ll=l*(l+1);
tmpA=(1+l+(1:maxn))';
rrs=(r.^tmpA);
rrsd=(tmpA.*rrs)/r;

for i=2:max_number_corrections
          Bs=[tmpB(i-1:(i-1)*4) 0 0 0 0];
          Bse=[tmpBe(i-1:(i-1)*4) 0 0 0 0];
          tmpB=zeros(1,maxn);
          tmpBe=zeros(1,maxn);
          %voor p=i
          tmpA = Vm1*Bs(1);
          tmpAe = Vm1*Bse(1);
          lp=l+i;
          tn=(lp*(lp+1)-ll);
          tmpB(i)=tmpA/tn;
          tmpBe(i)=tmpAe/tn;
          for p=i+1:i*4
            lp=lp+1;  
            tmpi=max(1,p-i-2):p-i+1;  
            tmp = [Bs(tmpi) 0 0 0 0];
            tmpV = Vmatrix(:,min(p-i+1,4));
            tmpA = tmp(1:4)*tmpV;
            tmp2= [Bse(tmpi) 0 0 0 0];
            tmpAe = tmp2(1:4)*tmpV-tmp(2);
            tn=(lp)*(lp+1)-ll;
            tmpB(p) = tmpA/tn;
            tmpBe(p) = tmpAe/tn;
          end
          ytmp=tmpB*rrs;
          y=y+ytmp;
          yp=yp+tmpB*rrsd;
          ye=ye+tmpBe*rrs;
          yep=yep+tmpBe*rrsd;
          if abs(ytmp)/abs(y)<1e-15  %tkan zijn dat y zelf zeer klein is dus treshold moet misschien van y afhangen?
              break;
          end
end
varargout{1}=[y;yp];
varargout{2}=[ye;yep];

if nargout>2
    vmin = get(cpm_child,'Vmin');
    if e-vmin > 1
      sn = sqrt(e-vmin);
    else
      sn = 1;
    end
    step=r/10;
    r=step;
    y1=r^(l+1);
    nzeros=0;
    for i=2:10
        r=i*step;
        y_prev=y1;
        y1=r^(l+1);
        if y1*y_prev<=0
            nzeros=nzeros+1;
        end
    end
    % Generation of the Prufer phase
    tmp = atan(sn*y/yp);
    if (-1)^nzeros*yp < 0
      tmp = pi + tmp;
    end
    varargout{3} = nzeros*pi + tmp;  
end



function [yf,yef,number_intervals]=laststep(e,part)
%@CPM/LASTSTEP
% [yf,yef,number_intervals]=laststep(e,part)
% For the test value e of the eigenvalue searched for,
% a lower limit r' for rmax is choosen. For that 
% eigenvalue only the interval (r0,r'] is actually involved.
% using WKB approach
global cpm_child schrod_object;
rmin = get(schrod_object,'min'); 
wkbmax =20;
wkb = 0;
i = part.imatch-1;
while 1
    i = i+1;
    if e <= part.V0(i)
        wkb = wkb + part.h(i)*sqrt(part.V0(i)-e);
    end
    if wkb >= wkbmax || i == length(part.h)
        %als wkb > wkbmax dan wil dit zeggen dat de E-lijn de potentiaal
        %gekruisd heeft en er al beetje voorbij is (hoeveel voorbij? zoveel
        %dat wkb >= wkbmax
        dobreak=true;
        for j=i:length(part.h)
            if e > part.V0(j) %dan mag het nog niet afgekapt worden
                %want de energie zal de potentiaal verder nog eens snijden.
                dobreak=false;
                break;
            end
        end
        if dobreak || i == length(part.h)
           break;
        end
    end
end
number_intervals = i;
if part.V0(number_intervals) >= e
    yf = [1;-sqrt(part.V0(number_intervals)-e)];
else %at rmax
    yf = [0;1];  %then: number_intervals = length(part.h) 
end
yef = [0;0];
if wkb >= 1.5*wkbmax
    d = 1.5 * wkbmax - wkb + part.h(number_intervals)*sqrt(part.V0(number_intervals)-e);
    a.h = part.h(number_intervals) * d / (part.h(number_intervals)*sqrt(part.V0(number_intervals)-e));
    number_intervals = number_intervals-1;
    r = rmin + sum(part.h(1:number_intervals));
    [a.V0,AV]=calculate_V(r,a.h);
    wkbnew = a.h * sqrt(a.V0-e);
    if wkbnew < d
        a.h = a.h *d /wkbnew;
        [a.V0,AV]=calculate_V(r,a.h);
    end
    [a.cu,a.cup,a.cv,a.cvp]=get_coeff(cpm_child,AV);
    [yf,yef]=propagate_solution(e,yf,yef,1,1,a,a.V0,false);
end
if yf(1) < 0
    yf = -yf;
    yef = -yef;
end
if number_intervals < part.imatch
    warning('number_intervals < part.imatch!')
end


%------------------------------------------------------------------
function [r,number_intervals]=get_turning_point(e,part,xmin,S,R)
x=[xmin xmin+cumsum(part.h)];
lenx=length(x)-1;
iturn=1;
V0=part.V0;
h=part.h;
%r=x(2);
for i=lenx:-1:2
   if V0(i-1)<e  
      iturn=i;
      break;      
   end
end
if iturn<length(h)   
 wkb=0;
 wkbt=min(max(1/abs(e),1),5);
 if abs((V0(iturn)-V0(iturn+1))/h(iturn))>1
     wkbt=wkbt*2;
 end
 imax=iturn+1;
 while wkb<wkbt && imax<lenx
    %r=x(imax);
    dif=V0(imax-1)-e;    %quadrature rule
    if dif>0
        wkb=wkb+h(imax-1)*sqrt(dif);
    end
    imax=imax+1;
 end
else
    imax=lenx;
end
imax=max(imax,part.imatch+1);
imax=min(imax+1,lenx+1);
r=x(imax);
while imax < lenx && ( abs(R(r))>1e-15 || (abs((S(r)-S(x(imax-1)))/S(r)))>1e-7 || abs((V0(imax)-V0(imax-1))/h(imax))>10)
    %R moet naar 0 gaan en S moet naar constante gaan.
    imax=imax+1;
    r=x(imax);
end
number_intervals=imax-1;


function [yf,yef]=get_asymptotic_solution(e,r,Z,l)
arlo=zeros(1,100);
arup=zeros(1,100);
arlod=zeros(1,100);
arupd=zeros(1,100);
sqe=sqrt(-e);
x=2*r*sqe;
k=Z/sqe/2;  
c=1;
c1=-(1-k+l)*(-l-k)*c; 
%ro=c1*x^(-1);
%rod=-c1*x^(-2);
summ=1;
sumd=0;
huge=1e60;
tiny=1e-60;
elimo=1;
delimo=0;
inds=0;
indsd=0;
for n=1:100   
 c=-(n-k+l)*(n-1-l-k)*c/n;
 term=c*x^(-n);
 termd=-n*c*x^(-n-1);
 n1=n+1;
 c1=-(n1-k+l)*(n1-1-l-k)*c1/n1;
 ro=c1*x^(-n1);
 rod=-n1*c1*x^(-n1-1);
 if isinf(termd) ||isinf(term)
     elimn=summ;
     delimn=sumd;
     indsd=1;
     inds=1;
 end
 summ=summ+term; %ok
 sumd=sumd+termd;%ok
 if abs(term)<1e-200
     elimn=summ;
     inds=1;
 end
 if abs(termd)<1e-200
     delimn=sumd;
     indsd=1;
 end
 if inds == 0 && indsd == 0
     arup(n+1)   = summ / ro;
     arlo(n+1)   = 1  / ro;
     arup(n) = arup(n+1) - arup(n);
     arlo(n) = arlo(n+1) - arlo(n);
     arupd(n+1)   = sumd / rod;
     arlod(n+1)   = 1  / rod;
     arupd(n) = arupd(n+1) - arupd(n);
     arlod(n) = arlod(n+1) - arlod(n);
     if n>1 
          coef    = n / (1+n);
          for j = 2:n
            tmp1=1+n-j;  
            fact  = tmp1 * coef^(j-2) / (n+1);
      %  perform the calculation of the new elements of the two tables.
            arup(tmp1)  = arup(tmp1+1) - fact*arup(tmp1);            
            arlo(tmp1)  = arlo(tmp1+1) - fact*arlo(tmp1);
            arupd(tmp1)  = arupd(tmp1+1) - fact*arupd(tmp1);            
            arlod(tmp1)  = arlod(tmp1+1) - fact*arlod(tmp1);
          end
     end 
  %  compute the estimate for the limit of the sequence.
     if abs(arlo(1))<tiny 
       elimn = huge;
     else
       elimn = arup(1)/arlo(1);
     end
     if abs(arlod(1))<tiny 
       delimn = huge;
     else
       delimn = arupd(1)/arlod(1);
     end     
     if abs(elimn-elimo)< eps*abs(elimn)
         inds=1;
     end
     if abs(delimn-delimo) < eps*abs(delimn)
         indsd=1;
     end
     delimo=delimn;
 else
   if inds==0
     %[estlim,arlo,arup]=levin(summ,ro,beta,n,arlo,arup);             
     arup(n+1)   = summ / ro;
     arlo(n+1)   = 1  / ro;
     arup(n) = arup(n+1) - arup(n);
     arlo(n) = arlo(n+1) - arlo(n);
     if n>1 
          bn1     = n;
          bn2     = 1 + n;
          coef    = bn1 / bn2;
          for j = 2:n
            fact  = (1+n-j) * coef^(j-2) / bn2;
      %  perform the calculation of the new elements of the two tables.
            arup(n-j+1)  = arup(n-j+2) - fact*arup(n-j+1);            
            arlo(n-j+1)  = arlo(n-j+2) - fact*arlo(n-j+1);
          end
     end 
  %  compute the estimate for the limit of the sequence.
    if abs(arlo(1))<tiny 
       estlim = huge;
    else
       estlim = arup(1)/arlo(1);
    end
     
     elimn=estlim;
     if abs(elimn-elimo)< eps*abs(elimn)
         inds=1;
     end
     elimo=elimn;
   end
   if indsd == 0
     %[destlim,arlod,arupd]=levin(sumd,rod,beta,n,arlod,arupd);
     arupd(n+1)   = sumd / rod;
     arlod(n+1)   = 1  / rod;
     arupd(n) = arupd(n+1) - arupd(n);
     arlod(n) = arlod(n+1) - arlod(n);
     if n>1 
          bn1     = n;
          bn2     = 1 + n;
          coef    = bn1 / bn2;
          for j = 2:n
            fact  = (1+n-j) * coef^(j-2) / bn2;
      %  perform the calculation of the new elements of the two tables.
            arupd(n-j+1)  = arupd(n-j+2) - fact*arupd(n-j+1);            
            arlod(n-j+1)  = arlod(n-j+2) - fact*arlod(n-j+1);
          end
     end 
     %  compute the estimate for the limit of the sequence.
     if abs(arlod(1))<tiny 
       destlim = huge;
     else
       destlim = arupd(1)/arlod(1);
     end
     
     delimn=destlim;
     if abs(delimn-delimo) < eps*abs(delimn)
         indsd=1;
     end
     delimo=delimn;
   end
 end
 if indsd==1 && inds == 1
     break;
 end
end
s=elimn;
sd=delimn;
wh=1;   %genormalizeerd
whd1=k/x-1/2+sd/s;
whd=2*sqe*whd1;
yf=[wh;whd];
m=l+1/2;
whe=-r/sqe*whd1;
whde=-2*r*(1/4-k/x-(1/4-m^2)/x^2);
yef=[whe;whde];



% function [estlim,arlo,arup]=levin(sofn,rofn,beta,n,arlo,arup)
% % computes levin's transformation.
% %  (d. levin (j. comput. math., vol. b3, 1973, 371-388 ) using the
% %  partial sums s(n) and terms a(n) of a given series as input data.
% %
% %  here, the definition of levin's sequence transformation according to
% %  eq. (7.1-7) of e. j. weniger, comput.. phys. rep., vol. 10 (1989),
% %  189 - 371, is used.
% %
% %  in levin's sequence transformation it is assumed that the partial
% %  sums s(n) of the series to be transformed can be written as
% %
% %  s(n) = s + r(n),
% %
% %  with s being the limit or antilimit of this series, and that the
% %  exact remainder r(n) can be approximated by a remainder estimate
% %  omega(n) in such a way that the ratio
% %
% %  [ s(n) - s ] / omega(n)
% %
% %  can be expressed as a by a poincare-type asymptotic expansion in
% %  inverse powers of (n + beta).
% %
% %  levin suggested some simple remainder estimates omega(n) which can
% %  be computed be determined from at most two terms a(n) of the series
% %  to be transformed. with the help of these remainder estimates the
% %  following variants of levin's sequence transformations result
% %  (compare section 7.3 of e. j. weniger, comput. phys. rep.,
% %  vol. 10 (1989), 189):
% %
% %  (n+beta) * a(n)                     : u transformation, eq. (7.3-5)
% %  a(n)                                : t transformation, eq. (7.3-7)
% %  a(n+1)                              : d transformation, eq. (7.3-7)
% %  a(n + 1) * a(n) / [a(n) - a(n + 1)] : v transformation, eq. (7.3-11)
% %
% %  it is always assumed that the index n of the terms and the partial
% %  sums s(n) satisfies n >= 0.
% %
% %  both numerator and denominator of this transformation are calculated 
% %  with the help of a simple extension of a 2-dimensional nonlinear
% %  3-term recurrence formula which was originally derived by 
% %  t.fessler, w.f.ford and w.f.smith 
% %  ( acm trans.math.software ,vol. 9 ( 1983 ), pp.346-354 ).           *
% %                                                                      *
% %  in this subroutine a variant of the so-called moving lozenge tech-  *
% %  nique ( p.wynn, r.f.t.i.-chiffres, vol.9, pp.327-362 ( 1966 )) is   *
% %  used, i.e., only one counterdiagonal of the numerator and denomina- *
% %  tor tables have to be stored at a time. consequently, only two      *
% %  1-dimensional arrays of appropriate length are needed in this       *
% %  subroutine.                                                         *
% %  in each call this subroutine calculates new counterdiagonals of the *
% %  numerator and denominator tables and overwrites the previous entries.
% %  in addition, a new estimate of the limit is calculated which is the *
% %  in addition, a new estimate of the limit is calculated which is the *
% %  ratio of the array elements arup(0) and arlo(0). hence, for every   *
% %  new sequence element s-of-n this subroutine has to be called again.
% %                                                                      *
% %  input parameters :                                                  *
% %  ----------------                                                    *
% %                                                                      *
% %  sofn   : element s(n) of the sequence to be accelerated             *
% %                                                                      *
% %  rofn   : estimate of the leading term of the poincare-type asympto- *
% %           tic expansion of the remainder r(n) in inverse powers of 
% %           (n+beta), i.e. omega(n).    
% %           it is tacitly assumed that rofn is different from zero.    *
% %                                                                      *
% %  beta   : parameter contained in the inverse powers                  *
% %           ( n + beta ) ** ( -j )                                     *
% %           which describe the higher order contributions in the       *
% %           poincare-type expansion of the remainder r(n).             *
% %                                                                      *
% %  n      : number of the last sequence element (n has to satisfy the  *
% %           inequality n .ge. 0 )                                      *
% %                                                                      *
% %  arup   : 1-dimensional array to store the actual counterdiagonal of *
% %           the numerator table.                                       *
% %                                                                      *
% %  arlo   : 1-dimensional array to store the actual counterdiagonal of *
% %           the denominator table.                                     *
% %                                                                      *
% %  larray : length of the 1-dimensional arrays arup and arlo.          *
% %                                                                      *
% %  output parameter :                                                  *
% %  ----------------                                                    *
% %                                                                      *
% %  estlim : approximation to the limit of the sequence s(n) which is   *
% %           to be accelerated.                                         *
% %           we have here estlim = arup(0)/arlo(0).                     *
% %                                                                      *
% %  machine-dependent parameters:                                       *
% %  ----------------------------                                        *
% %                                                                      *
% %  huge,tiny : huge should be set close to but not identical with the  *
% %              largest floating point number representable on the com- *
% %              puter, and tiny should be set close to but not identical*
% %              with the smallest floating point number representable   *
% %              on the computer.                                        *
% %                                                                      *
% huge = 1e+60;
% tiny = 1e-60;
% arup(n+1)   = sofn / rofn;
% arlo(n+1)   = 1  / rofn;
% % check whether n .gt. 0 holds. in that case the recursive computation
% %  of the numerators and denominators of the general levin transformatio
% %  is to be continued.
% if  n > 0 
%         arup(n) = arup(n+1) - arup(n);
%         arlo(n) = arlo(n+1) - arlo(n);
% %  check whether n .gt. 1 holds. in that case the actual recursion
% %  of the numerators and denominators starts.
%         if n>1 
%           bn1     = beta + n-1;
%           bn2     = beta + n;
%           coef    = bn1 / bn2;
%           for j = 2:n
%             fact  = (beta+n-j) * coef^(j-2) / bn2;
% %  perform the calculation of the new elements of the two tables.
%             arup(n-j+1)  = arup(n-j+2) - fact*arup(n-j+1);            
%             arlo(n-j+1)  = arlo(n-j+2) - fact*arlo(n-j+1);
%           end
%         end 
% end 
% %  compute the estimate for the limit of the sequence.
% if abs(arlo(1))<tiny 
%      estlim = huge;
% else
%      estlim = arup(1)/arlo(1);
% end


function [varargout]=firststep(e,rr)
%@CPM/FIRSTSTEP
% [y,ye]=firststep(e,r)
% [y,ye,phase]=firststep(e,r)
% calculates the solution on the interval I_1: (0,r]
% for a distorted Coulomb problem.
%
% r could be a vector. Then the procedure is repeated for every element in
% r.

global cpm_child schrod_object
length_sum = 60;  

vmin = get(cpm_child,'Vmin');
if ~isa(schrod_object,'distorted_coulomb')
    error('firststep could only be applied when the schrodinger object is a distorted_coulomb object')
end
l = get(schrod_object,'l');
if l ~= round(l) || l < 0
    integer = false;
else
    integer = true;
end
Vs = get(schrod_object,'Vs');
if e-vmin > 1
    sn = sqrt(e-vmin);
else
    sn = 1;
end
for nterm=length(Vs):-1:1  
    if Vs(nterm) ~= 0
        break;
    end
end
M = max(nterm-1,1);

len = length(rr);
Y = zeros(2,len);
Ye = zeros(2,len);
e0=e;
vs0=Vs(2);
l0=l;
tmpM=-1:M-1;
tmpM2=tmpM+2;
phase=zeros(1,len);
g=zeros(3,M+1);
w=zeros(3,M+1);
for h=1:len
r=rr(h); 
e = e0;
Vs(2)=vs0;
l=l0;
sch = l*(l+1)/(r*r)-e;
sch = sch + Vs(tmpM2)*(r.^tmpM)';
difev = (e-Vs(2))*r^2;
if(abs(difev) <= 0.00001 || abs(e)<0.00001) %remark 4     
    %ik had hier eerst 0.01 maar 0.00001 geeft betere resultaten, klopt opm4 wel?
      if difev >= 0                                 
          delta= e - (0.01/(r^2));
      else
          delta = e + (0.01/(r^2)); 
      end
      Vs(2) = Vs(2)-delta;
      e = e - delta;
end
if e < 0  %&& integer %remark 5 
    is = -1;
else
    is = 1;
end
%if integer
  x = sqrt(abs(e))*r;
  alpha = 1/sqrt(abs(e));
  %else
 % x = sqrt(e)*r;
 % alpha = 1/sqrt(e);
 %end
v = Vs(tmpM2).*alpha.^(tmpM2);
if imag(l) ~= 0 %avoid complex l-values % does this work???
    v(2)=v(2)+l*(l+1)/x^2; 
    l = 0;
end
if integer
    [csi,eta] = get_eta(-x*x*is,l+length_sum,1);             
end
g(1,:) = zeros(1,M+1);
w(1,:) = zeros(1,M+1);
g(1,1)=1;
g(2,:)=((is.^tmpM2)./(tmpM2.*(2*l+tmpM+3))).*((2*l+3)*v(tmpM2)); %=g_{1,j}
w(2,:)=-tmpM2.*g(2,:);  %=w_{1,j}
y = 0;
yp = 0;
ypp = 0;
cye = 0;
cype = 0;
xpowin = x^(l-2);
tpruf=zeros(length_sum+1,M+1);
for k=0:length_sum
      xpowin = xpowin*is*x;  
      xpow = xpowin;
      s = k+2;
      for j=0:M
         tmp1 = (2*(l+s)+1)/((s+j)*(2*l+j+s+1));  % (3.9) en (3.13) 
         g1j=g(1,j+1);
         if 2*l + 2*s == 3
             g(3,j+1)=0;
             w(3,j+1)=0;
         else
           hulp=j+1:-1:1;
           ghulp=g(2,hulp);
           hulp2=1:j+1;
           vhulp2=v(hulp2);
           hulp3=is.^(-1:1:j-1);
           tmp2 = (s-j-2)*(2*l+s-j-1)/(2*l+2*s-3); 
           sum = vhulp2*(ghulp.*(hulp3))';
           sump = vhulp2*((w(2,hulp)-((hulp2).*ghulp)).*hulp3)'; 
           g(3,j+1) = tmp1 * (sum-tmp2*is*g1j);
           w(3,j+1) = tmp1 * (sump-tmp2*is*w(1,j+1));
         end
         if j <= 1
             tpruf(k+1,j+1)=g1j;
         end
         xpow = xpow*is*x;
         t = l+k;
         u = t-j;
         if integer
           eta1 = eta(t+1);
           if t > 0
             eta2 = eta(t);
           else
             eta2 = csi;
           end
         else
             etat = get_eta_noninteger(-x*x*is,[t t-1]);
             eta1 = etat(1);
             eta2 = etat(2);
         end
         y = y + g1j*eta1*xpow;    % term in phi   
         yp = yp + g1j*(-u*eta1+eta2)*xpow;    
         cype = cype+w(1,j+1)*(-u*eta1+eta2)*xpow;
         cye = cye+w(1,j+1)*eta1*xpow;
         if integer
           if t > 1  
            dypp = g1j*(u*(u+1)*eta1+(-2*u+1)*eta2+eta(t-1))*xpow;      
           elseif t == 1
              dypp = g1j*(u*(u+1)*eta1+(-2*u+1)*eta2+csi)*xpow; 
           elseif t == 0
              dypp = g1j*((-x*x*is)*(j*eta(2)+eta1)+j*(j*eta1+csi))*xpow;  
           end 
         else
           eta3 =  get_eta_noninteger(-x*x*is,t-2);
           dypp = g1j*(u*(u+1)*eta1+(-2*u+1)*eta2+eta3)*xpow; 
        end
         ypp = ypp+dypp;
      end
      if(abs(ypp-sch*y*r^2) <= 1e-14*abs(ypp))
          break;
      end
      g(1,:) = g(2,:);
      g(2,:) = g(3,:);
      w(1,:) = w(2,:);
      w(2,:) = w(3,:);      
 end

  
%   Finally y and cye are multiplied by x, yp and cype are unchanged
%   and ypp is divided by x. 
  y = y*x;
  cye = cye*x;
  ypp = ypp/x;
%	The partial derivatives against e are constructed
  ye = (x*yp+cye)/(2*e);   %(3.11)
  ype = (x*ypp+cype)/(2*e);  %(3.12)
%   The values are converted in the original r variable. Only yp,ype
%	and ypp are affected.
  yp = yp/alpha;     
  ype = ype/alpha;
  ypp = ypp*e*is;
%   Renormalization by an energy dependent multiplicative factor (fren)
  fnorm = 1e-100;
  fact = fnorm * prod(2*(1:l)+1); 
  fren = fact*alpha^(l+1);
  ye = fren*(-(l+1)*y/(2*e)+ye);
  ype = fren*(-(l+1)*yp/(2*e)+ype);   
  y = fren*y;  
  yp = fren*yp;
  ypp = fren*ypp;
    
  Y(:,h) = [y; yp];
  Ye(:,h) =[ye; ype];
if nargout == 3
 % estimation of the number of zeros in y     
  nzeros = 0;
  y1 = 1;
  hx = x/10;
  tmpk=1:k+1;
  mmax = 30;
  mmaxm = 29;
  m=l+k+1;
  wght2(1) = 1 / prod(2*(1:m-1)+1); 
  wght1(1) = wght2(1) / (2*m+1);
  wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
  wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
  for i=1:10
      x = i*hx;
      %b = get_eta(-x*x*is,l+length_sum,1);
      if integer
        [csi,eta] = get_eta(-x*x*is,l+k+1,wght1,wght2);
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta(l+1:l+k+1)'));
      else
        eta = get_eta_noninteger(-x*x*is,l+(0:k));
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta'));
      end
      xx = is*1/x;
      y2 = (xx*((x*is).^tmpk))*hlp;
      if y1*y2 <= 0
          nzeros = nzeros + 1;
      end
      y1=y2;
  end
  err = 0;
  if k==length_sum+1
      err=1;
  end
  
  
% Generation of the Prufer phase
  tmp = atan(sn*y/yp);
  if (-1)^nzeros*yp < 0
      tmp = pi + tmp;
  end
  phase(h) = nzeros*pi + tmp;  

 %	Error test
 if err == 1
      if tmp/pi >= 0.02 && tmp/pi <= 0.98	
        if l==0
          den=ypp;
        else
	      den=ypp;
          if abs(den)<fnorm*l*(l+1)*r^(l+1)
              den = fnorm*l*(l+1)*r^(l+1);
          end
        end
        relerr = abs((ypp-sch*y)/den);  %relative error
        if relerr >= 1e-14
           disp('accuracy loss on I1');
        end
      end
 end
end
end%for h
varargout{1}=Y;
varargout{2}=Ye;
if nargout ==3
   varargout{3}=phase; 
end


function eta = get_eta_noninteger(x,s)
% used when index of eta-function is not an integer
% expresses this function as a besselfunction
x = sqrt(-x);
les=length(s);
eta=zeros(1,les);
if les>1
  for i=1:les   
    eta(i) = sqrt(pi/2)*(x)^(-(s(i)+1/2))*besselj(s(i)+1/2,x);
  end
else
  eta = sqrt(pi/2)*(x)^(-(s+1/2))*besselj(s+1/2,x);
end



function [csi,eta] = get_eta(varargin)
%GET_ETA [csi,eta] = get_eta(Z,m)
% this calculates csi, eta_i, i=0..m, for given Z
Z = varargin{1};
m = varargin{2};
tz = 1;
eta=zeros(1,m+1);
mmax = 30;
mmaxm = 29;
if nargin ==4
    wght1=varargin{3};
    wght2=varargin{4};
else
    wght2(1) = 1 / prod(2*(1:m-1)+1); 
    wght1(1) = wght2(1) / (2*m+1);
    wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
    wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
end
    
if imag(Z)==0
if nargin>2 || abs(Z) < tz 
    %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
    %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
    if nargin >2
      if wght1(mmax) == 0
        tz = 1;    
      else
        tz = (eps*wght1(1)/wght1(mmax))^(1/(mmaxm));
      end
    end
    if(abs(Z) < tz)
      tmp=(Z.^(0:mmaxm))';  
      eta(m+1) = wght1*tmp;
      eta(m) = wght2*tmp;
      for i=(m-1):-1:1
        eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
      end
      csi = Z * eta(2) + eta(1);
      return
    end
end
if Z >= tz  
      sq = sqrt(Z);
      csi = cosh(sq);
      if isinf(csi)
          wght2(1) = 1 / prod(2*(1:m-1)+1); 
          wght1(1) = wght2(1) / (2*m+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(m+1) = wght1(1:mmax)*tmp;
          eta(m) = wght2(1:mmax)*tmp;
          for i=(m-1):-1:1
              eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
          end
          csi = Z * eta(2) + eta(1);
          return;
      end
      eta(1) = sinh(sq)/sq;
else
      sq = sqrt(abs(Z));
      csi = cos(sq);
      eta(1) = sin(sq)/sq;
end    
eta(2)=(csi-eta(1))/Z;
for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
end

 
else %complex
    csi=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi-eta(1))/Z;
    for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
    end
end
