function part = get_partition(varargin)
% part = get_partition(varargin)
%used to construct meshpoints where an eigenfunction should be computed
global cpm_child
e = varargin{1};
n = varargin{end};
schr = get(cpm_child,'schrod');
x_min = get(schr,'min');
x_max = get(schr,'max');
bp = get(cpm_child,'bp');
if isempty(bp)
    error('There is something wrong with the partition. The integration interval is maybe infinite.');
end
part.cu = [];
part.cup = [];
part.cv = [];
part.cvp = [];
part.V0 = [];
if isa(schr,'distorted_coulomb')
  [yi,yei,yf,yef,number_intervals,phase] = get_initConditions(e,bp); 
  x_max = x_min + sum(bp.h(1:number_intervals));
end

if nargin == 2 && isa(n,'double') %construct partition with n meshpoints
  x = x_min;
  xs = x_min + cumsum(bp.h);
  i_a=1;
  hmin=bp.h(i_a);
  i=0;
  h = (x_max-x_min)/n;
  while x < x_max
    i=i+1;
    part.h(i)=min(min(h,x_max-x),hmin); %die hmin is nodig omdat vb rond singulariteit de stap niet te groot mag zijn
    [part.V0(i),V] = calculate_V(x,part.h(i));
    [part.cu(i,:),part.cup(i,:),part.cv(i,:),part.cvp(i,:)] = get_coeff(cpm_child,V); 
    x = x + part.h(i);
    if i_a<length(xs) && x> xs(i_a+1)
          i_a=i_a+1;
          hmin=bp.h(i_a);
    end
  end

  [Vmin,part.imatch] = min(part.V0);

  V = get(schr,'V'); 
  symm = true;
  mid = (x_max+x_min)/2;
  for i=mid:(x_max-x_min)/200:x_max
    if V(mid+i) ~= V(mid-i)
        symm =false;
        break;
    end
  end
  if symm
      summ = x_min;
      for i=1:length(part.h)
          summ = summ + part.h(i); 
          if summ > mid
              break;
          end
      end
      part.imatch = i;
  end
elseif nargin == 2 
    part=n;
    if x_min+sum(part.h) < x_max %enlarge partition
        x = x_min+sum(part.h);
        i=0;
        if length(part.h)>2
          h = part.h(length(part.h)-1)*1.5;
            if h<1e-10
                h = part.h(length(part.h)-2)*1.7;
            end
        else
          h = (x_max - x_min)/5;  
        end
       while x < x_max
        i=i+1;
        step(i)=min(h,x_max-x);
        [V0(i),V] = calculate_V(x,step(i));
        [cu(i,:),cup(i,:),cv(i,:),cvp(i,:)] = get_coeff(cpm_child,V); 
        x = x + step(i);
       end
       part.h = [part.h step];
       part.cu = [part.cu; cu];
       part.cup = [part.cup; cup];
       part.cv = [part.cv; cv];
       part.cvp= [part.cvp; cvp];
       part.V0 = [part.V0 V0];
    elseif x_min+sum(part.h) == x_max
        return;
    else %shrink partition
       x=x_min+sum(part.h);
       while x >x_max
           x = x-part.h(end);
           part.h = part.h(1:length(part.h)-1);
       end
       m=length(part.h);
       part.V0 = part.V0(1:m);
       part.cu=part.cu(1:m,:);
       part.cup=part.cup(1:m,:);
       part.cv=part.cv(1:m,:);
       part.cvp=part.cvp(1:m,:);
       if x < x_max
        part.h(m+1)=x_max-x;
        [part.V0(m+1),V] = calculate_V(x,x_max-x);
        [part.cu(m+1,:),part.cup(m+1,:),part.cv(m+1,:),part.cvp(m+1,:)] = get_coeff(cpm_child,V);  
       end
       [Vmin,part.imatch] = min(part.V0);
    end
else
    a = max(x_min,varargin{2}); %necessary for distorted_coulomb
    b = varargin{3};
    x = x_min+bp.h(1);
    part.h=[];
    i=0;
    while x <= a %|| abs(a-x)<1e-15
        i=i+1;
        part.h(i) = bp.h(i);
        x = x + bp.h(i+1);
    end
    m=length(part.h);
    part.cu = bp.cu(1:m,:);
    part.cup = bp.cup(1:m,:);
    part.cv = bp.cv(1:m,:);
    part.cvp = bp.cvp(1:m,:);
    part.V0 = bp.V0(1:m);
    
    if x > a && a>x_min %&& abs(x-a)>1e-15
      i = i+1;
      x = x - bp.h(i); %stapke terug
      if abs(a-x)>1e-15
        part.h(i) = a-x;
        [part.V0(i),V] = calculate_V(x,a-x);
        [part.cu(i,:),part.cup(i,:),part.cv(i,:),part.cvp(i,:)] = get_coeff(cpm_child,V); 
      end
    end
    
    %nu zitten we al in a

    xs = x_min + cumsum(bp.h);
    i_a=0;
    for i=1:length(xs)
        if xs(i)>a
            i_a=1;
            break;
        end
    end
    hmin=bp.h(i_a);
    x = a;
    i=0;
    step=[];
    h = (b-a)/n;
    while x < b && abs(x-b)>1e-15
      i=i+1;
      step(i)= min(min(h,b-x),hmin);  %kan te groot zijn vb rond singulariteit! daarom ook nr hmin kijken
      [V0(i),V] = calculate_V(x,step(i));
      [cu(i,:),cup(i,:),cv(i,:),cvp(i,:)] = get_coeff(cpm_child,V); 
      x = x + step(i);
      if i_a<length(xs) && x> xs(i_a+1)
          i_a=i_a+1;
          hmin=bp.h(i_a);
      end
    end
    part.h = [part.h step];
    part.cu = [part.cu; cu];
    part.cup = [part.cup; cup];
    part.cv = [part.cv; cv];
    part.cvp= [part.cvp; cvp];
    part.V0 = [part.V0 V0];
        
    
    i_b=0;
    for i =1:length(xs)
        if xs(i)>b && abs(xs(i)-b)>1e-15
            i_b=i;
            break;
        end
    end
    if i_b > 0
    i=length(part.V0)+1;  
    part.h(i)=xs(i_b)-b;
    [part.V0(i),V] = calculate_V(b,xs(i_b)-b);
    [part.cu(i,:),part.cup(i,:),part.cv(i,:),part.cvp(i,:)] = get_coeff(cpm_child,V); 
        
    if i_b < length(xs)
    x = xs(i_b+1);
    i=i_b; 
    h=[];
    while x < x_max && x_max-x >1e-13 
        i=i+1;
        h = [h bp.h(i)];
        x = x + bp.h(i+1);
    end
    m=length(h);
    t=i_b+1:i_b+1+m;
    part.h = [part.h bp.h(t)];
    part.cu = [part.cu ; bp.cu(t,:)];
    part.cup = [part.cup ; bp.cup(t,:)];
    part.cv =[part.cv ; bp.cv(t,:)];
    part.cvp = [part.cvp ; bp.cvp(t,:)];
    part.V0 = [part.V0 bp.V0(t)];
    end
    end
    
    [Vmin,part.imatch] = min(part.V0);
end
 