function [T,Te]=get_propagation_matrices(e,part,t,forward)
%@CPM\GET_PROPAGATION_MATRICES
%[T,Te]=get_propmatrices(e,part,t,forward)
% returns the propagation matrices
hm=part.h;
cum=part.cu;
cupm=part.cup;
cvm=part.cv;
cvpm=part.cvp;
vbarm=part.V0;

M = size(cum,2);
Zm = (vbarm(t)-e).*hm(t).^2;
[csim,etam] = get_eta2(Zm,M); %compute the basic functions (xsi and eta)

for i=1:length(t)
    tt=t(i);
    cu=cum(tt,:);
    cup=cupm(tt,:);
    cv=cvm(tt,:);
    cvp=cvpm(tt,:);
    h=hm(tt);
    csi=csim(i);
    eta=etam(i,:);
    Z=Zm(i);
    tmp = eta(1:M)';

    if csi > 1e100
        csi = csi/1e140;
        eta = eta./1e140;
    end

    u  = csi + cu*tmp;
    up = (Z * eta(1) + cup*tmp)/h;
    v = (eta(1) + cv*tmp)*h;
    vp = csi + cvp*tmp;

    %using eq.(A.8):  
    ue = eta(1) + cu(2:M)*eta(3:M+1)';  
    uep = csi + eta(1) + cup*eta(2:M+1)';
    ve = eta(2) + cv(3:M)*eta(4:M+1)';
    vep = eta(1) + cvp(2:M)*eta(3:M+1)';
    ue = -ue*h^2/2; 
    uep = -uep*h/2; 
    ve = -ve*(h^3)/2; 
    vep = -vep*h^2/2; 
    if forward
        T(:,:,i)=[u, v ; up, vp];
        Te(:,:,i)=[ue, ve ; uep, vep];
    else
        T(:,:,i)=[vp, -v ; -up, u];
        Te(:,:,i)=[vep, -ve ; -uep, ue];
    end

end


function [csi,etam] = get_eta2(Zm,m)
%GET_ETA [csi,eta] = get_eta(Z,m)
% this calculates csi, eta_i, i=0..m, for given Z

mmax = 30;
mmaxm = 29;
lenZm = length(Zm);
etam = zeros(lenZm,m+1);
csi = zeros(lenZm,1);
for k=1:lenZm
Z = Zm(k);
tz = 1;
eta=zeros(1,m+1);

if imag(Z)==0

if abs(Z) < tz 
    %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
    %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
    wght2(1) = 1 / prod(2*(1:m-1)+1); 
    wght1(1) = wght2(1) / (2*m+1);
    wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
    wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
    tmp=(Z.^(0:mmaxm))';  
    eta(m+1) = wght1(1:mmax)*tmp;
    eta(m) = wght2(1:mmax)*tmp;
    for i=(m-1):-1:1
        eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
    end
    etam(k,:)=eta;
    csi(k) = Z * eta(2) + eta(1);
    continue;
end
if Z >= tz  
      sq = sqrt(Z);
      csi(k) = cosh(sq);
      if isinf(csi(k))
          wght2(1) = 1 / prod(2*(1:m-1)+1); 
          wght1(1) = wght2(1) / (2*m+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(m+1) = wght1(1:mmax)*tmp;
          eta(m) = wght2(1:mmax)*tmp;
          for i=(m-1):-1:1
              eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
          end
          etam(k,:)=eta;
          csi(k) = Z * eta(2) + eta(1);
          continue;
      end
      eta(1) = sinh(sq)/sq;
else
      sq = sqrt(abs(Z));
      csi(k) = cos(sq);
      eta(1) = sin(sq)/sq;
end    
eta(2)=(csi(k)-eta(1))/Z;
for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
end

else %complex
    csi(k)=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi(k)-eta(1))/Z;
    for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
    end
end
etam(k,:)=eta;
end



