function [cp]=get_regular_problem(cp,pmax,indices)
% GET_REGULAR_PROBLEM : REGULARIZATION OF SINGULAR PROBLEMS
% 1) transforms a problem with endpoint(s)=inf into a
% problem with finite endpoints = truncation 
% 2) truncates a singular problem

schrod=get(cp,'schrod');
singularities=get(schrod,'singularities');
classif=get(schrod,'classification');
cutoff=classif.cutoff;
regular=classif.regular;
V=get(schrod,'V');
x_min = get(schrod,'min');
x_max = get(schrod,'max');
infmax=false;
infmin=false;
%infinite integration interval?
if isinf(x_max)
    infmax=true;
end
if isinf(x_min)
    infmin=true;
end
%singular endpoints?
singular_min=false;
singular_max=false;
if ~isempty(singularities) && ~infmin && regular(1)==0 
    singular_min=true; %endpoint a is singular
end
if ~isempty(singularities) && ~infmax && regular(2)==0  
    singular_max=true; % endpoint b is singular
end

%%get (initial) truncation points for (finite) singular endpoints/
% i.e. a* and b*
%endpoint a is singular:
if singular_min
     if cutoff <1e300
         he=1e-5;
         x=x_min+he;
         while V(x)>cutoff
             x=x+he;
             he=he*1.2;
         end
         if he==1e-5
             sums=0;
             x=x_min+he;
             while sums<2
               he=he*1.1;  
               x=x+he;
               sums=sums+(cutoff-V(x))*he;
             end
         else
             wkb=0;
             x=x-he;
             he=1e-5;
             while wkb<2
                 he=he*1.05;
                 x=x-he;
                 if (V(x)-cutoff)<0
                     break;
                 end
                 wkb=wkb+sqrt(V(x)-cutoff)*he;
             end
         end
     else
          he=1e-5;
          x=x_min+he;
          vx=V(x);
          rico=1e11;
          while rico>1e10
               he=he*1.1;  
               x=x+he;
               vx2=V(x);
               rico=(vx-vx2)/he;
               vx=vx2;
          end
     end
     x_min=x;
end
%endpoint b is singular:
if singular_max
     if cutoff <1e300
         he=1e-5;
         x=x_max-he;
         while V(x)>cutoff
             x=x-he;
             he=he*1.2;
         end
         if he==1e-5
             sums=0;
             x=x_max-he;
             while sums<2
               he=he*1.1;  
               x=x-he;
               sums=sums+(cutoff-V(x))*he;
             end
         else
             wkb=0;
             x=x+he;
             he=1e-5;
             while wkb<2
                 he=he*1.05;
                 x=x+he;
                 if (V(x)-cutoff)<0
                     break;
                 end
                 wkb=wkb+sqrt(V(x)-cutoff)*he;
             end
         end
     else
          he=1e-5;
          x=x_max+he;
          vx=V(x);
          rico=1e11;
          while rico>1e10
               he=he*1.1;  
               x=x-he;
               vx2=V(x);
               rico=(vx-vx2)/he;
               vx=vx2;
          end
     end
     x_max=x;
end

if indices && (infmin || infmax) % infinite integration interval
   wkb=max((pmax+1),18);
   if infmax 
     %get initial truncation point for b  
     if infmin
       [x_max,emax0,vminx]=get_x_max(V,wkb,0,-1e300,0,V(0));
     else
       if cutoff<1e300
           x=x_min;
           he=1e-5;
           vx=V(x+he);
           vx2=V(x);
           while (vx-vx2)/he>1e2 && cutoff-vx>10
               he=he*1.2;
               x=x+he;
               vx2=vx;
               vx=V(x);
           end
           [x_max,emax0,vminx]=get_x_max(V,wkb,x,-1e300,0,V(x));  
        else
           [x_max,emax0,vminx]=get_x_max(V,wkb,x_min,-1e300,0,V(x_min));  
        end
     end
   end
   if infmin
       %get initial truncation point for a 
       if infmax
         [x_min,emax1,vminxm]=get_x_min(V,wkb,0,-1e300,0,V(0));
       else
         if cutoff<1e300
           x=x_max;
           he=1e-5;
           vx=V(x-he);
           vx2=V(x);
           while (vx-vx2)/he>1e2 && cutoff-vx>10
               he=he*1.2;
               x=x-he;
               vx2=vx;
               vx=V(x);
           end  
           [x_min,emax1,vminxm]=get_x_min(V,wkb,x,-1e300,0,V(x));
         else
           [x_min,emax1,vminxm]=get_x_min(V,wkb,x_max,-1e300,0,V(x_max));  
         end
       end
   end
   [bp,Vmin]=get_coarse_mesh(cp,x_min,x_max);  %construct coarse mesh
   if infmax  %what is highest eigenvalue which can be calculated on this mesh?
      nmax0 = locate_eigenvalue(emax0,bp,Vmin,1,length(bp.h));
   else
      nmax0 =1e300; 
   end
   if infmin 
      nmax1 = locate_eigenvalue(emax1,bp,Vmin,1,length(bp.h));
   else
       nmax1 = nmax0;
   end
   equals=0;
   it=0;
   while min(nmax0,nmax1)<pmax
       %integration interval was too small
       it=it+1;
       x_mino=x_min;
       x_maxo=x_max;
       if infmax && nmax0<pmax+1
           %get a value for b which is larger than previous b
         [x_max,emax0,vminx]=get_x_max(V,max((pmax-nmax0)/2,1),x_maxo,emax0,vminx,cutoff);
         if x_maxo < x_max %enlarge mesh
            x = x_maxo;
            i=length(bp.h);
            trial_h = (x_max-x_maxo)/50;  
            while x < x_max
                i=i+1;
                [bp.h(i),bp.V0(i),Vs] = get_step(x,trial_h,x_max-x,1e-3);
                [bp.cu(i,:),bp.cup(i,:),bp.cv(i,:),bp.cvp(i,:)] = get_coeff(cp,Vs); 
                x = x + bp.h(i);
                trial_h = bp.h(i);
            end
         end
       end
       if infmin && nmax1<pmax+1
          [x_min,emax1,vminxm]=get_x_min(V,max((pmax-nmax1)/2,1),x_mino,emax1,vminxm,cutoff);
          if infmax && abs((x_min+x_max)/x_max) <0.5
              if V(x_max)==V(-x_max) && V(x_min)==V(-x_min)
                  x_max=max(x_max,-x_min);
                  x_min=-x_max;
              end
          end
          if x_mino > x_min
             x = x_min;
             trial_h = (x_maxo-x_mino)/50;  
             %enlarge the coarse mesh:
             while x < x_mino
                [h,Vb,Vs] = get_step(x,trial_h,x_mino-x,1e-3);
                bp.h = [h bp.h];
                bp.V0 = [Vb bp.V0];
                [cu,cup,cv,cvp] = get_coeff(cp,Vs); 
                bp.cu = [cu;bp.cu];
                bp.cup = [cup;bp.cup];
                bp.cv = [cv;bp.cv];
                bp.cvp = [cvp;bp.cvp];
                x = x + h;
                trial_h = h;
             end     
          end
        end 
        [Vmin,bp.imatch] = min(bp.V0);
        if infmax
           nmax0 = locate_eigenvalue(emax0,bp,Vmin,1,length(bp.h));
        else
           nmax0=1e300; 
        end
        if infmin
           nmax1 = locate_eigenvalue(emax1,bp,Vmin,1,length(bp.h));
        else
           nmax1 = nmax0; 
        end
        if it>25
            if equals==0
                nmax_eq=round(min(nmax0,nmax1));
            end
            if round(min(nmax0,nmax1)) == nmax_eq
                equals=equals+1;
                if equals>10
                    error(['Error!!! :  Sorry, the eigenvalues with indices larger than ' num2str(nmax_eq-1) ' can ''t be calculated'])
                end
            else
                equals=0;
            end
        end
        pause(0) % stop the calculation in the GUI?
        load stop
        if stop
          error('Error!!! :  The calculation was interrupted');
        end
   end
   if infmax && floor(nmax0)==pmax %voor als de gevraagde eigenwaarde juist de laatste eigenwaarde in het spectrum is
       if cutoff <1e300 && V(x_max)<cutoff
           x_max=get_x_max2(V,1,x_max,cutoff);
           x_max=x_max+abs(x_max)/4;
           if pmax==0 && V(x_max)<cutoff
               x_max=get_x_max2(V,1,x_max,cutoff);
           end
           if classif.category == 2 && pmax == classif.lastev-1
               x_max=x_max+abs(x_max)/2;
               if V(2*x_max)-V(x_max)<1e-5
                   x_max=x_max+abs(x_max)/2;
               end
           end
       end
   end
   if infmin && floor(nmax1)==pmax
       if cutoff <1e300 && V(x_min)<cutoff
           x_min=get_x_min2(V,1,x_min,cutoff);
           x_min=x_min-abs(x_min)/4;
           if pmax==0 && V(x_min)<cutoff
               x_min=get_x_min2(V,1,x_min,cutoff);
           end
           if classif.category == 2 && pmax == classif.lastev-1
               x_min=x_min-abs(x_min)/2;
           end
       end
   end
else
 pmax10=abs(pmax/15);   
 if isinf(x_max)
    %find xmax
    if isinf(x_min)
      x=0;         %beter minimum van V
    else
      x=x_min;
    end
    he=0.1;
    vx=V(x);
    vmin=vx;
    vx_prev=vx;
    while abs(vx-pmax)>1e-80 && (vx<pmax || V(x+min(0.1,he))<vx) 
      x=x+he;
      vx=V(x);
      if (vx-vx_prev)/he >-1e-4  && (vx-pmax)*(vmin-pmax)<0
          break;
      end
      if vx<vmin
          vmin=vx;
      end
      vx_prev=vx;
      he=he*1.2;
    end
    if abs(vx-pmax)<1e-80 && vx<pmax && abs(vx-V(x+he/2))/(he/2)<1e-50
        x_max=x;
    else
      xlow=x-he*2/3;
      xup=x;
      it = 0;
      while it<10
        it=it+1;  
        xc=(xup+xlow)/2;
        vxc=V(xc);
        if vxc < pmax
          xlow = xc;
        else
          xup = xc;
          if abs(vxc - pmax)<pmax10 
            break;
          end
        end
      end
      x=xup;
      sums=0;
      he=abs(x/20);
      vx1=V(x);
      while sums<18
        vx0=vx1;  
        x=x+he;
        vx1=V(x);
        vm=(vx0+vx1)/2;
        if vx1==vx0 || ((vx1-vx0)/he < 0 && (vx1-vx0)/he >-1e-3)
          x=x+2*he;
          break;
        end
        if vm-pmax >=0
         sums=sums+he*sqrt(vm-pmax); %trapezoidal rule
        else
          sums=0;
        end
      end
      x_max=x;
    end
 end
 if isinf(x_min)
    %find xmin
    x=min(0,x_max);  %beter ipv 0 de plaats waar het minimum in V ligt
    he=0.1;
    vx=V(x);
    vmin=vx;
    vx_prev=vx;
    while abs(vx-pmax)>1e-80 && (vx<pmax || V(x-min(0.1,he))<vx)
      x=x-he;
      vx=V(x);
      if (vx-vx_prev)/he >-1e-4  && (vx-pmax)*(vmin-pmax)<0
          break;
      end
      if vx<vmin
          vmin=vx;
      end
      vx_prev=vx;
      he=he*1.2;
    end
    if abs(vx-pmax)<1e-80 && vx<pmax && abs(vx-V(x-he/2))/(he/2)< 1e-50
        x_min=x;
    else
      xlow=x+he*2/3;
      xup=x;
      it=0;
      while it<10
        it=it+1; 
        xc=(xup+xlow)/2;
        vxc=V(xc);
        if vxc < pmax
          xlow = xc;
        else
          xup = xc;
          if abs(vxc - pmax)<pmax10 
            break;
          end
        end
      end
      x=xup;
      sums=0;
      he=abs(x)/20;
      vx1=V(x);
      while sums<18
        vx0=vx1;  
        x=x-he;
        vx1=V(x);
        vm=(vx0+vx1)/2;
        if vx1==vx0 || ((vx1-vx0)/he < 0 && (vx1-vx0)/he >-1e-3)
          x=x-2*he;
          break;
        end
        if vm-pmax >=0
         sums=sums+he*sqrt(vm-pmax);
        else
          sums=0;
        end
      end
      x_min=x;
    end
 end
end
problem = get(cp,'schrod');
if infmax || singular_max
      problem = set(problem,'max',x_max);
end
if infmin || singular_min
      problem = set(problem,'min',x_min);
end
%problem = set(problem,'singularities',singularities); 
cp = set(cp,'schrod',problem);
cp = set(cp,'truncated',true);  


function [x_max,emax,vminx] = get_x_max(V,wkb,x,emax0,vminx,cutoff)
sums = 0;
vmin = min(V(x),cutoff);
he=0.1;
vx_prev=vmin;
x0=x;
while true
       x=x+he;
       vx=V(x);
       if isnan(vx) %te ver
           x=x-he;
           break;
       end
       if vx > vmin && (vx>vx_prev || (vx-vx_prev)/he >-1e-3)  
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         if vmin == vx
            vminx= x;
         end
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10 && x>x0+2*he) || x>x0+max(5e3,abs(x0)*1e2)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x+1)/10);
end
x_max=x;
if abs(vx-V(x+he))<1e-2
    x_max=x_max+abs(x)/10;
end
x0 = max(0.7*x_max,vminx);
x=x0;
he=(x_max-x0)/40;
emax=V(x);
xu=x_max;
xl=0;
while true
 xx=x0+(0:39)*he;
 vx=V(xx)-emax;
 tmp=[0 find(vx<0)];
 te=tmp(end);
 sums=sum(sqrt(vx(te+1:end))*he);
 if sums >16
     xl=x0;
     if ((xu-xl)/xl <0.25 || xl==0 && abs(xu-xl)<1e-2 )&& sums <50  
       break;
     end
 else
    xu=x0;
 end
 if x0 == vminx || ((xu-xl)/xl <0.1 || xl==0 && abs(xu-xl)<1e-2 )
     emax=V(x0);
     break;
 end
 x0=max((xl+xu)/2,vminx);
 x=x0;
 emax=V(x);
 he=(x_max-x)/40;
end
emax=max(emax,emax0*1.0001);

function x_max = get_x_max2(V,wkb,x,cutoff)
sums = 0;
vmin =V(x);
he=0.1;
vx_prev=vmin;
x0=x;
while true
       x=x+he;
       vx=V(x);
       if isnan(vx) %te ver
           x=x-he;
           break;
       end
       if vx > vmin && (vx>vx_prev || (vx-vx_prev)/he >-1e-3)  
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10 && x>x0+2*he) || x>x0+max(5e3,abs(x0)*1e2)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x+1)/10);
end
x_max=x;


function [x_min,emax,vminx] = get_x_min(V,wkb,x,emax0,vminx,cutoff)
sums=0;
vmin = min(V(x),cutoff);
he=0.1;
vx_prev=vmin;
x0=x;
while true
       x=x-he;
       vx=V(x);  
       if isnan(vx) %te ver
           x=x+he;
           break;
       end
       if vx > vmin && (vx>vx_prev  || (vx-vx_prev)/he >-1e-3 )
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         if vmin == vx
             vminx=x;
         end
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10 && x<x0-2*he) || x<x0-max(5e3,abs(x0)*1e2)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x-1)/10);
end
x_min=x;
if abs(vx-V(x-he))<1e-2
    x_min=x_min-abs(x)/10;
end
x0=min(0.7*x_min,vminx);
x=x0;
he=abs(x_min-x0)/40;
emax=V(x);
xu=x_min;
xl=0;
while true
 xx=x0-(0:39)*he;
 vx=V(xx)-emax;
 tmp=[1 find(vx<0)];
 te=tmp(end);
 sums=sum(sqrt(vx(te+1:end))*he);
 if sums >16
    xl=x0;
     if (abs((xu-xl)/xl) <0.25 || xl==0 && abs(xu-xl)<1e-2) && sums <50  
       break;
     end
 else
    xu=x0;
 end
 if x0 == vminx || (abs((xu-xl)/xl) <0.1 || xl==0 && abs(xu-xl)<1e-2 )
     emax=V(x0);
     break;
 end
 x0=(xl+xu)/2; 
 x=x0;
 emax=V(x);
 he=abs(x_min-x)/40;
end
emax=max(emax,emax0*1.0001);



function x_min = get_x_min2(V,wkb,x,cutoff)
sums=0;
vmin = min(V(x),cutoff);
he=0.1;
vx_prev=vmin;
x0=x;
while true
       x=x-he;
       vx=V(x);  
       if isnan(vx) %te ver
           x=x+he;
           break;
       end
       if (vx > vmin && vx>vx_prev ) || (vx-vx_prev)/he >-1e-3  
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10 && x<x0-2*he) || x<x0-max(5e3,abs(x0)*1e2)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x-1)/10);
end
x_min=x;
 

%-------------------------------------------------
function [bp,Vmin]=get_coarse_mesh(cpm_child,x_min,x_max)
x = x_min;
i=0;
trial_h = (x_max-x_min)/50;
if ~isreal(x_max) && imag(x_max) > 0
    error('Error!!! : no success');
end
while x < x_max
    i=i+1;
    [bp.h(i),bp.V0(i),V] = get_step(x,trial_h,x_max-x,1e-3);
    [bp.cu(i,:),bp.cup(i,:),bp.cv(i,:),bp.cvp(i,:)] = get_coeff(cpm_child,V);
    x = x + bp.h(i);
    trial_h = bp.h(i);
end
[Vmin,bp.imatch] = min(bp.V0);