function varargout=locate_eigenvalue(eold,part,Vmatch,trunca,truncb)
% @PPM\LOCATE_EIGENVALUE
% [pruf,enew]=locate_eigenvalue(eold,part,Vmatch)
%   
% For given e (eold) the Schrodinger equation is integrated from the
% both ends up to the matching point to produce the Prufer's 
% theta(eold)/pi (pruf) and an extrapolated value of 
% e (enew), hopefully closer to an eigenvalue.    

global schrod_object

S = 1;
if((eold-Vmatch)> 1) 
      S = sqrt(eold-Vmatch);
end
if isa(schrod_object,'distorted_coulomb')
   [yi,yei,yf,yef,number_intervals,thforw] = get_initConditions(eold,part);
   truncb = min(truncb,number_intervals);
else
   [yi,yf] = get_initConditions;
   yei=[0; 0];
   yef=[0; 0];
   [thforw]= prufer(yi(1),yi(2),yi(1),yi(2),eold,part.V0(trunca),0,S);
   if thforw < 0
     thforw = pi+thforw; % thforw = theta_min 
   end
end

[yl,yel,theta] = propagate_solution(eold,yi,yei,trunca,part.imatch,part,Vmatch,true);
thforw = thforw + theta; % thforw = thforw + delta_i;

%backward
[thbackw] = prufer(yf(1),yf(2),yf(1),yf(2),eold,part.V0(truncb),0,S);
if thbackw > 0
    thbackw = thbackw-pi;
end

if truncb > part.imatch
      [yr,yer,theta] = propagate_solution(eold,yf,yef,part.imatch+1,truncb,part,Vmatch,false);
      thbackw = thbackw - theta; % thbackw = thbackw - delta_i; 
else
    yr = yf;   %in xmatch
    yer = [0; 0];
end

varargout{1} = (thforw-thbackw)/pi;
if imag(varargout{1}) < eps
    varargout{1} = real(varargout{1});
end
if nargout == 1
    return;
end

%the data obtained at each side of xmatch are combined:
phi = yl(1)*yr(2)-yr(1)*yl(2); %miss-distance / miss-match
%derivative with respect to E:
dphi = yel(1)*yr(2)+yl(1)*yer(2)-yer(1)*yl(2)-yr(1)*yel(2);

aux = yr(1)*yl(1);
auxp = yr(2)*yl(2);
%derivatives with respect to E:
daux = yer(1)*yl(1)+yr(1)*yel(1);
dauxp = yer(2)*yl(2)+yr(2)*yel(2);

if abs(aux) >= abs(auxp)
    newton = -phi*aux/(dphi*aux-daux*phi);
else %(abs(aux) < abs(auxp))
    newton = -phi*auxp/(dphi*auxp-dauxp*phi);
end
varargout{2} = eold + newton; % Newton/Raphson-step 