function varargout = propagate_solution(e,y,ye,imin,imax,part,Vmin,forward)
% @CPM\PROPAGATE_SOLUTION 
% [y,ye,theta] = propagate_solution(e,y,ye,imin,imax,part,forward)
% This propagates the solution from the leftmost point of interval
% imin up to the rightmost point of interval imax if forward = true
% and viceversa if forward = false. y(1), y(2), ye(1) and ye(2) 
% represent y, y',y_e and y'_e both in the input and in the output.
% Argument part contains the standard data regarding the partition.
pruf=false;

if nargout==3
 S = 1;
 if((e-Vmin)> 1) 
      S = sqrt(e-Vmin);
 end
 pruf=true;
 theta=0;
end

if forward
 t = imin:imax;
else
 t = imax:-1:imin;    
end
if imin>imax
    varargout{1}=y;
    varargout{2}=ye;
    return;
end

hm=part.h;
cum=part.cu;
cupm=part.cup;
cvm=part.cv;
cvpm=part.cvp;
vbarm=part.V0;
M = size(cum,2);
mmax = 30;
mmaxm = 29;

for i = 1:length(t)
    y1=y;
    ye1=ye;
    tt=t(i);
    cu=cum(tt,:);
    cup=cupm(tt,:);
    cv=cvm(tt,:);
    cvp=cvpm(tt,:);
    vbar=vbarm(tt);
    h=hm(tt);
    Z=(vbar-e)*h^2;
    eta=zeros(1,M+1);
    %calculation of eta functions:
    if imag(Z)==0
      if abs(Z) < 0.5 
       %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
       %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
       if M==9
        wght1 =[1.527349308567059e-009 0.36365459727787e-10 0.00395276736172e-10 0.00002635178241e-10 0.00000012199899e-10 0.00000000042069e-10...
            0.00000000000113e-10 0];
        wght2 = [2.901963686277412e-008  0.76367465428353e-9 0.00909136493195e-9 0.00006587945603e-9 0.00000032939728e-9 0.00000000121999e-9...
            0.00000000000351e-9 0.00000000000001e-9];
        mmaxm=7;
        mmax=8;
       elseif M==8
            wght1 = [2.901963686277412e-008 0.76367465428353e-9   0.00909136493195e-9   0.00006587945603e-9   0.00000032939728e-9   0.00000000121999e-9   0.00000000000351e-9];
            wght2 = [4.933338266671600e-007 0.14509818431387e-7   0.00190918663571e-7   0.00001515227489e-7   0.00000008234932e-7   0.00000000032940e-7   0.00000000000102e-7];
            mmaxm=6;
            mmax=7;
       else
        wght2(1) = 1 / prod(2*(1:M-1)+1); 
        wght1(1) = wght2(1) / (2*M+1);
        wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*M+1).*(1:mmaxm)));
        wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*M-1).*(1:mmaxm)));
       end
       tmp=(Z.^(0:mmaxm))';
       eta(M+1) = wght1*tmp;
       eta(M) = wght2*tmp;
       for k=(M-1):-1:1
         eta(k) = Z * eta(k+2) + (2*k+1) * eta(k+1);
       end
       csi = Z * eta(2) + eta(1);
      else
       if Z >= 0.5
        sq = sqrt(Z);
        csi = cosh(sq);    
        eta(1) = sinh(sq)/sq; 
       else
        sq = sqrt(abs(Z));
        csi = cos(sq);    
        eta(1) = sin(sq)/sq;
       end
       eta(2)=(csi-eta(1))/Z;
       for k=3:M+1
          eta(k)=(eta(k-2)-(2*k-3)*eta(k-1))/Z;
       end
       if isinf(csi)
          mmax = 20;
          mmaxm = 19;
          wght2(1) = 1 / prod(2*(1:M-1)+1); 
          wght1(1) = wght2(1) / (2*M+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*M+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*M-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(M+1) = wght1(1:mmax)*tmp;
          eta(M) = wght2(1:mmax)*tmp;
          for k=(M-1):-1:1
              eta(k) = Z * eta(k+2) + (2*k+1) * eta(k+1);
          end
          csi = Z * eta(2) + eta(1);
       end
      end    
   else %complex
    csi=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi-eta(1))/Z;
    for k=3:M+1
     eta(k)=(eta(k-2)-(2*k-3)*eta(k-1))/Z;
    end
   end 
   %end calculation eta functions.    
    tmp = eta(1:M)';
    eta1=eta(1);
    while csi > 1e100
        csi = csi/1e140;
        eta = eta./1e140;
    end
    u  = csi + cu*tmp;
    up = (Z * eta1 + cup*tmp)/h;
    v = (eta1 + cv*tmp)*h;
    vp = csi + cvp*tmp;
    %using eq.(A.8):  
    tmp=eta(2:end)';
    ue = eta1 + cu*tmp;  
    uep = csi + eta1 + cup*tmp;
    ve = eta(2) + cv*tmp;
    vep = eta1 + cvp*tmp;
    h2=h*h;
    ue = -ue*h2/2; 
    uep = -uep*h/2; 
    ve = -ve*(h2*h)/2; 
    vep = -vep*h2/2; 
    if isinf(up)
        eta1t=eta1/1e200;
        ct=cup*tmp/1e200;
        csit=csi/1e200;
        u  = csit + (cu*tmp)/1e200;
        v = (eta1t + (cv*tmp)/1e200)*h;
        vp = csit + (cvp*tmp)/1e200;
        up=Z * eta1t/(h) + ct/h;
    end
    if forward
        T=[u, v ; up, vp];
        Te=[ue, ve ; uep, vep];
    else
        T=[vp, -v ; -up, u];
        Te=[vep, -ve ; -uep, ue];
    end
    tmp1 = Te*y+T*ye;
    tmp2 = T*y;
    if abs(tmp2(1))>10^50 || abs(tmp2(2))>10^50 || abs(tmp1(1))>10^50 || abs(tmp1(2))>10^50 || isnan(tmp1(1)) || isnan(tmp2(1))
        %avoid overflow
        m = max(abs([y;ye]));
        if m == 0
            m = 10^50;
        end
        y = y/m;
        ye = ye/m;
        tmp1 = Te*y+T*ye;
        tmp2 = T*y;
        if abs(tmp2(1))>10^100 || abs(tmp2(2))>10^100 || abs(tmp1(1))>10^100 || abs(tmp1(2))>10^100
            y = y/10^100;
            ye = ye/10^100;
            tmp1 = Te*y+T*ye;
            tmp2 = T*y;
        end
    end
    ye = tmp1;
    y = tmp2;
    if isinf(y(1)) || isnan(y(1))
        T=T./1e200;
        Te=Te./1e200;
        ye = Te*y1+T*ye1;
        y = T*y1;
        if isinf(y)  
         error('y = Inf OVERFLOW / It may be better to make the integration interval smaller');
        end
    end
    if pruf
      if forward
          [theta0,theta1] = prufer(y1(1),y1(2),y(1),y(2),e,vbar,h,S);
      else
          [theta0,theta1] = prufer(y(1),y(2),y1(1),y1(2),e,vbar,h,S);
      end
      theta=theta+theta1-theta0;
    end
end
varargout{1}=y;
varargout{2}=ye;
if pruf
    varargout{3}=theta;
end

