function d = distorted_coulomb(varargin)
%distorted_coulomb class constructor
% d = distorted_coulomb(l,S,R,rmax)
% d = distorted_coulomb(l,S,R,rmax,var)
%
% y'' = (l*(l+1)/x^2 + S(x)/x + R(x) - E)*y 

switch nargin
case 0
    % if no input arguments, create a default object
   parent = schrod;
   d.l = 0;
   d.S =inline('1');
   d.R = inline('1');
   d = class(d,'distorted_coulomb',parent);
case 1    
 if isa(varargin{1},'distorted_coulomb')
     d = varargin{1};
 else
     error('distorted_coulomb constructor: Input argument is not a distorted_coulomb object')
 end
case {4,5}
  if ~isa(varargin{1},'double') 
       error('distorted_coulomb constructor:  arg1: You need to pass a double-precision floating point number');
  end
  if ~isa(varargin{2},'char')
       error('distorted_coulomb constructor: arg2: You need to pass the function as string');  
  end     
  if ~isa(varargin{3},'char') 
       error('distorted_coulomb constructor: arg3: You need to pass the function as string');
  end
  if nargin == 5
       if ~(isa(varargin{5},'char') || isa(varargin{5}, 'sym'))
          error('distorted_coulomb constructor: arg5: You need to pass the unknown variable as character or as sym '); 
       end
       d.S = inline(char(subs(varargin{2},sym(varargin{5}),sym('x'))));
       d.R = inline(char(subs(varargin{3},sym(varargin{5}),sym('x'))));
  else
      d.S = inline(varargin{2});
      d.R = inline(varargin{3});
  end
  d.l = varargin{1};
  if d.l < 0
        tmp=d.l*(d.l+1);
        if 4*tmp >= 0
            d.l = (-1+sqrt(1+4*tmp))/2;
        end
  end
  Sf=d.S;
  Rf=d.R;
  %test_singularity(Rf,0,10,true);  %gives problems in Matlab2013
  %test_singularity(Sf,0,10,false);
  d.min=0;
  d.max=varargin{4};
  parent = schrod(sprintf('(%d*(%d+1))/x^2+(%s)/x + (%s)',varargin{1},...
      varargin{1},varargin{2},varargin{3}),0,varargin{4},1,0,1,0);
  %find a good value for r0:
  r0 = (12 / (1+20/(1+exp(d.l))))/2;
  if r0 >= d.max
      r0 = d.max/6;
  end
  ll=d.l*(d.l+1);
  step=0.001;
  r=0+step;
  while abs(Sf(r)/r+Rf(r))/abs(ll/r^2)<0.05
    r=r+step;
    step=step*1.5;
  end
  d.min=min(min(r0,r),0.1);
  
  d.Vs=fitpot(r,Sf,Rf);  %voor PERSYS procedure
  d = class(d,'distorted_coulomb',parent);
otherwise
    error('distorted_coulomb: Wrong number of input arguments')
end;


%---------------------------------------------
function test=test_limits(S)
x=1000;
h=1;
rico=abs((S(x+h)-S(x))/h);
prev_rico=1e14;
while rico < prev_rico  
    %the S function is going to a limit
        prev_rico = rico;
        x = x+h;
        h=h*2;
        rico=abs((S(x+h)-S(x))/h);
        if rico<1e-5
            break;
        end
end
if rico < 1e-5
    test=false;
else
    test=true;
end
if isinf(S(0))
    test=true;
end

% %--------------------------------------------
% function test_singularity(R,a,b,Rf)
% if Rf
%     tmp='R';
% else
%     tmp='S';
% end
% str=maple(['singular(' char(R) ')']);
% hlp=findstr('=',str);
% hlp2=findstr('}',str);
% singularities=[];
% for i=1:length(hlp)
%     s = str(hlp(i)+1:hlp2(i)-1);
%     if ~(isempty(findstr('_Z',s)) && isempty(findstr('_N',s))) && isempty(findstr('*i',s)) &&(isinf(sch.max)||isinf(sch.min))
%        error(['Error!!! : The ' tmp ' function is singular in ' s]);
%     end
%     singularity = str2num(s);
%     if isempty(singularity)
%         break;
%     end
%     if ~isinf(singularity) && singularity >= a && singularity <= b && imag(singularity) ==0
%         if isinf(R(singularity))
%            singularities = [singularities singularity];
%         else
%                warning(['Removable singularity in ' num2str(singularity)']);
%                pause(0) % GUI
%                load stop
%                if ~stop
%                  warndlg(['Removable singularity in ' num2str(singularity)]);
%                end
%         end
%     end
% end
% if ~isempty(singularities)
%      s = num2str(singularities(1));
%      for i=2:length(singularities)
%          s = [ s ' and ' num2str(singularities(i))];
%      end
%      error(['Error!!! : The ' tmp ' function is singular in ' s]);
% end

 function Vs=fitpot(cr,funS,funR)
% This subroutine gives the fitting coefficients by a parabola of 
% each component of the matrix potential specified by function FUN.
%     input parameters:
%     cr         = the radius at which the solution is computed
%     fun        = the function which should be approximated. 
%     output parameters:
%     c0, c1, c2 = The coefficients of the shifted Legendre polynomials 
%                  of degrees 0, 1, 2 respectively.

a=cr/2;    
b=cr;          
c=0.43056815579702629*b;    
xg(1)=a+c;
xg(2)=a-c;
c=0.16999052179242813*b;   
xg(3)=a+c;
xg(4)=a-c;
wg(1)=b*0.17392742256872693;
wg(2)=b*0.32607257743127307;
for k=1:4
  vfun(k)=funS(xg(k));
  ch=xg(k)/cr;
  vpol(k)=2*ch-1;
end
s0=wg(1)*(vfun(1)+vfun(2))+wg(2)*(vfun(3)+vfun(4));
s0=s0/cr;
s1=wg(1)*(vfun(1)*vpol(1)+vfun(2)*vpol(2))+wg(2)*(vfun(3)*vpol(3)+vfun(4)*vpol(4));
s1=3*s1/cr;
for k = 1:4
	ch=xg(k)/cr;
	vpol(k)=6*ch^2-6*ch+1;
end
s2=wg(1)*(vfun(1)*vpol(1)+vfun(2)*vpol(2))+wg(2)*(vfun(3)*vpol(3)+vfun(4)*vpol(4));
s2=5*s2/cr;

for k=1:4
  vfun(k)=funR(xg(k));
  ch=xg(k)/cr;
  vpol(k)=2*ch-1;
end
r0=wg(1)*(vfun(1)+vfun(2))+wg(2)*(vfun(3)+vfun(4));
r0=r0/cr;
r1=wg(1)*(vfun(1)*vpol(1)+vfun(2)*vpol(2))+wg(2)*(vfun(3)*vpol(3)+vfun(4)*vpol(4));
r1=3*r1/cr;
for k = 1:4
	ch=xg(k)/cr;
	vpol(k)=6*ch^2-6*ch+1;
end
r2=wg(1)*(vfun(1)*vpol(1)+vfun(2)*vpol(2))+wg(2)*(vfun(3)*vpol(3)+vfun(4)*vpol(4));
r2=5*r2/cr;


% obtain the corresponding coefficients of the terms proportional 
%    to r^{-1}, r^0, r^1 and r^2 
%     wc         = The coefficient corresponding to r^{-1} 
%     w0         = The coefficient corresponding to r^0 
%     w1         = The coefficient corresponding to r^1 
%     w2         = The coefficient corresponding to r^2 
Vs(1)=s0-s1+s2;
Vs(2)=r0-r1+r2+(2*s1-6*s2)/cr;
Vs(3)=(2*r1-6*r2)/cr+6*s2/cr^2;
Vs(4)=6*r2/cr^2;
