function E=calculate_eigenvalues(emin,emax,nmin,nmax)
%CALCULATE_EIGENVALUES
%[eigvbasic,index,err,status]=calculate_eigenvalues(lpm_child,emin,emax,nmin,nmax)
% search of the eigenvalues with index n from nmin to nmax 

global lpm_child
bp = get(lpm_child,'bp');
rp = get(lpm_child,'rp');
Vmin = get(lpm_child,'Vmin');
truncated = get(lpm_child,'truncated');
distorted = isa(get(lpm_child,'schrod'),'distorted_coulomb');

e_low = emin;
n_low = nmin;
len=length(bp.h);
number_eigv = 0; 
h_en = (emax-emin)/(5*(nmax-nmin+1));
h_mem = h_en;

if truncated && nmax >nmin
   if nmax-nmin <50   
      nsteps=(nmax-nmin)/4;
   else
      nsteps=(nmax-nmin)/10; 
   end
   estep=abs((emax-emin)/nsteps);
   et=emin+estep;
   [trunca,truncb,inita,initb]=get_truncation_points(bp,et,bp.imatch,bp.imatch,distorted);
else
    trunca=1;
    truncb=len;
end

for n=nmin:nmax  
	number_eigv = number_eigv + 1;
    % an energy interval [e_low,e_up] is searched such that any e in
    % this interval represents a good initial guess. (there is only one
    % eigenvalue in this interval).
    h_en = abs(min(h_mem,h_en));
    while true
      %h_en = abs(min(h_mem,h_en))  
	  e_up = e_low + h_en;
      if truncated && (trunca > 1 || truncb < len)
        while e_up > et  
         et0=et;   
         et=et+estep;
         truncb0=truncb;
         inita0=inita;
         initb0=initb;
         [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita,initb,distorted);
         if truncb-truncb0 >20
             estep=estep/2;
             et=et0+estep;
             [trunca,truncb,inita,initb]=get_truncation_points(bp,et,inita0,initb0,distorted);
         end
        end
      end
      [n_up,phi_up] = locate_eigenvalue(e_up,bp,Vmin,trunca,truncb);
      if h_en > 0 && n_up < n_low
          % something wrong
          e_up1 = e_low + 2*h_en;
          [n_up1,phi_up1] = locate_eigenvalue(e_up1,bp,Vmin,trunca,truncb);
          e_up2 = e_low + 3*h_en;
          [n_up2,phi_up2] = locate_eigenvalue(e_up2,bp,Vmin,trunca,truncb);
          if n_up2<n_up1
              n_up=n_up2;
              phi_up=phi_up2;
              e_up=e_up2;
          else
              n_up=n_up1;
              phi_up=phi_up1;
              e_up=e_up1;
          end
      end
      rz2 = n_up-n-1;
	  if rz2>1 %|| phi_up*phi_low<0 
          h_en=h_en/2;
      elseif rz2 < 0 
	     e_low = e_up;
         n_low = n_up;
         h_en = h_en*2;
      else
         e_upt = e_up + h_en/2;
         [n_upt] = locate_eigenvalue(e_upt,bp,Vmin,trunca,truncb);
         if n_upt-n-1 > 0
             break; % stop while	
         else
             h_en=1.2*h_en;
         end
      end
     end %while
     % calculate the eigenvalue corresponding to the reference
     % partition:
      [ev,evpruf0] = calculate_eigenvalue(e_low,e_up,phi_up,rp,Vmin,2*trunca-1,2*truncb-1);
    
     % compute basic eigenvalue: the reference one is used as
     % initial guess
      [evx,evpruf]= calculate_eigenvalue(ev,bp,Vmin,trunca,truncb);
      eigv(number_eigv) = evx;
      index(number_eigv) = round(evpruf - 1);
      indexr(number_eigv) = round(evpruf0 - 1);	
	  err(number_eigv) = (evx-ev)*1.005;%*1.05;
      h_en = min(e_up-e_low,h_en);
	  e_low = max(e_up,min(ev,evx));
      phi_low=locate_eigenvalue(e_low,bp,Vmin,trunca,truncb);
	  n_low = n_up;
end %for


for i=1:number_eigv
    if indexr(i) == -1
        status(i)=1;
        continue;
    elseif indexr(i) ~= index(i)
        status(i)=2;
    elseif index(i)~= nmin+i-1
        status(i)=3;
    else
        status(i)=0;
        continue;
    end
    if status(i) == 2 && index(i) ~= nmin+i-1 
        status(i) = 4;
    end
end %for

E.eigenvalues=eigv;
E.indices=index;
E.errors=err;
E.status=status;


function [trunca,truncb,inita,initb]=get_truncation_points(bp,E,inita,initb,distorted)
% determine intervals where the infinite problem should be truncated
trunca=1;
truncb=length(bp.h);
  sums=0;
  V0=bp.V0;
  h=bp.h;
  for i=initb:length(h)-1 
    v0i=V0(i);  
    if v0i>=E && V0(i+1) > v0i
      if sums==0
          initb=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 15
        truncb=i;
        for j=length(h):-1:i+1
            if V0(j)>=v0i && V0(j-1)<v0i
                truncb=j;
            end
        end
        break;
    end
  end
if ~distorted
  sums=0;
  for i=inita:-1:2
    v0i=V0(i);  
    if v0i>=E && V0(i-1) > v0i
      if sums==0
          inita=i;
      end
      sums=sums+sqrt(v0i-E)*h(i);
    else
      sums=0;  
    end
    if sums > 15
        trunca=i;
        for j=1:i-1
            if V0(j)>=v0i && V0(j+1)<v0i
                trunca=j;
            end
        end
        break;
    end
  end
end


%-------------------------------------------------------------------------
function [e,pruf]= calculate_eigenvalue(varargin)
%@LPM\CALCULATE_EIGENVALUE
% (1) [e,pruf]= calculate_eigenvalue(e1,e2,phi2,part,vmin,trunca,truncb)
% or:
% (2) [e,pruf]= calculate_eigenvalue(e,part,vmin,trunca,truncb)Blatt iteration procedure.
%    (1): first an initial guess is generated by the bisection method.
%    (2): the start point for the iteration is already given in the
%    input (varargin{1})
%
%     e1 / e2     - in input: lower and upper limits for the eigenvalue e_n 
%                 - in output: sharper lower and upper limits
%     part        - data concerning the partition of the equation domain.
%     tol         - the requested tolerance.
%     vmin /vmax  - approximations of the min. and max. of the potential 
%
%      pruf       - the Pruefer theta/pi corresponding to the eigenvalue e.

global schrod_object 
x_min = get(schrod_object,'min');
if isa(schrod_object,'distorted_coulomb')
    distorted=true;
     yi = 0;
     yf = 0;
else
     distorted=false;
     [yi,yf] = get_initConditions;
end
if nargin == 7
  %bisection is used to obtain good initial estimation
   e = bisection(varargin{1},varargin{2},varargin{3},varargin{4},yi,yf,x_min,distorted,varargin{6},varargin{7});
   part = varargin{4};
   vmin = varargin{5};
else
    e = varargin{1};
    part = varargin{2};
    vmin = varargin{3};
end
truncb = varargin{nargin};
trunca = varargin{nargin-1};
%Blatt
e0=e;
de0=inf;
itb=0;
im=min(part.imatch,truncb-1);

while true
   de0prev=de0;
   if distorted
       [yi,yf] = get_initConditions2(e0,part,x_min); 
   end
   [suml,yl]=propagate_solution_blatt(e0,yi,trunca,im,part,true);
   [sumr,yr]=propagate_solution_blatt(e0,yf,im+1,truncb,part,false);
   cmat=yl(1)/yr(1);
   if ~isinf(cmat)
     yr(2)=yr(2)*cmat;
     sumr=sumr*cmat^2;
   end
   de0=real(yl(1)*(yl(2)-yr(2))/(suml+sumr));  
   if isnan(de0)
        error('Some problems occured')
    end
   itb=itb+1;
   if abs(de0)>abs(de0prev)
       de0=blatt(e0,yi,yf,part,trunca,truncb);
       if abs(de0)>abs(de0prev)
              if itb<5 && abs(de0prev)>1e-4
                e0=e0-0.9*de0prev;
                de0=blatt(e0,yi,yf,part,trunca,truncb);
              else
                e = ecor;
                break;
              end
       end
   end
    ecor=e0+de0;
    if abs(de0)>max(1e-15*abs(ecor),eps)
             e0 = ecor;
    else
              e = ecor;
              break;
    end
end
pruf = locate_eigenvalue(e,part,vmin,trunca,truncb);

%--------------------------------------------------------------------------
function [e,e0,e1]=bisection(e0,e1,phi1,bp,yi,yf,x_min,distorted,trunca,truncb)
% bisection method to find root of mismatch function
it=0;
im=min(bp.imatch,truncb-1);
tol0=1e-4;
while true 
    e=(e0+e1)/2;
    if distorted
       [yi,yf] = get_initConditions2(e,bp,x_min); 
    end
    it = it+1;  
    if it >= 70
        break;
    end
	tol = tol0*max(1e-7,abs(e));
    %forward:
    yl = propagate_solution(e,yi,trunca,im,bp,true);
    %backward:
    yr = propagate_solution(e,yf,im+1,truncb,bp,false);
    %the data obtained at each side of xmatch are combined:
    phi = yl(1)*yr(2)-yr(1)*yl(2);
    if phi*phi1>0
        phi1=phi;
        e1=e;
    else
        e0=e;
    end
    if abs(e1-e0)< tol  %min(max((tol*abs(e0)),1e-10),1e-4) 
        break;
    end
 end %while
 
%------------------------------------------------------------------- 
function d=blatt(e,yi,yf,bp,trunca,truncb)
% blatt iteration procedure to locate the root of the mismatch function
[suml,yl]=propagate_solution_blatt(e,yi,trunca,bp.imatch,bp,true);
[sumr,yr]=propagate_solution_blatt(e,yf,bp.imatch+1,truncb,bp,false);
while abs(yl(1))<1e-2 && abs(yl(2))>1e-1
    bp.imatch=bp.imatch+2;
    if bp.imatch>truncb
        break;
    end
    [suml,yl]=propagate_solution_blatt(e,yi,trunca,bp.imatch,bp,true);
    [sumr,yr]=propagate_solution_blatt(e,yf,bp.imatch+1,truncb,bp,false);
end
cmat=yl(1)/yr(1);
if ~isinf(cmat)
  yr(2)=yr(2)*cmat;
  sumr=sumr*cmat^2;
end
d=real(yl(1)*(yl(2)-yr(2))/(suml+sumr));



function [sum,y] = propagate_solution_blatt(e,y,imin,imax,bp,forward)
if forward
 t = imin:imax;
else
 t = imax:-1:imin;    
end
ht=bp.h;
sum=0;
if ~isempty(t)
 Tm=get_propagation_matrices(e,bp,t,forward);
 for j = 1:length(t)
    h=ht(t(j));
    sint=y(1)*y(1);
    T=Tm(:,:,j);
    %y = T*y;
    tmp2 = T*y;
    if abs(tmp2(1))>10^50 || abs(tmp2(2))>10^50 || isnan(tmp2(1))
        %avoid overflow
        m = max(abs(y));
        if m == 0
            m = 10^50;
        end
        y = y/m;
        sint=sint/m^2;
        sum=sum/m^2;
        tmp2 = T*y;
    end
    y = tmp2;
    sint=sint+y(1)*y(1);
    sum=sum+sint*h*0.5; %trapezoidal rule
 end
end


%-------------------------------------------------------------------------

 function [yi,yf,number_intervals]=get_initConditions2(e,part,x_min)
[yf,number_intervals] = laststep(e,part);
yi = firststep(e,x_min); 
 

function [varargout]=firststep(e,rr)
%FIRSTSTEP
% [y]=firststep(e,r)
% [y,phase]=firststep(e,r)
% calculates the solution on the interval I_1: (0,r]
% for a distorted Coulomb problem.
%
% r could be a vector. Then the procedure is repeated for every element in
% r.

global lpm_child schrod_object
length_sum = 60;  

vmin = get(lpm_child,'Vmin');
if ~isa(schrod_object,'distorted_coulomb')
    error('firststep could only be applied when the schrodinger object is a distorted_coulomb object')
end
l = get(schrod_object,'l');
if l ~= round(l) || l < 0
    integer = false;
else
    integer = true;
end
Vs = get(schrod_object,'Vs');
if e-vmin > 1
    sn = sqrt(e-vmin);
else
    sn = 1;
end
for nterm=length(Vs):-1:1  
    if Vs(nterm) ~= 0
        break;
    end
end
M = max(nterm-1,1);

len = length(rr);
Y = zeros(2,len);
Ye = zeros(2,len);
e0=e;
vs0=Vs(2);
l0=l;
tmpM=1:M+1;
for h=1:len
r=rr(h); 
e = e0;
Vs(2)=vs0;
l=l0;
sch = l*(l+1)/(r*r)-e;
sch = sch + Vs(tmpM)*(r.^(-1:M-1))';
difev = (e-Vs(2))*r^2;
if(abs(difev) <= 0.00001 || abs(e)<0.00001) %remark 4     
    %ik had hier eerst 0.01 maar 0.00001 geeft betere resultaten, klopt opm4 wel?
      if difev >= 0                                 
          delta= e - (0.01/(r^2));
      else
          delta = e + (0.01/(r^2)); 
      end
      Vs(2) = Vs(2)-delta;
      e = e - delta;
end
if e < 0  %&& integer %remark 5 
    is = -1;
else
    is = 1;
end
x = sqrt(abs(e))*r;
alpha = 1/sqrt(abs(e));
v = Vs(tmpM).*alpha.^tmpM;
if imag(l) ~= 0 %avoid complex l-values % does this work???
    v(2)=v(2)+l*(l+1)/x^2; 
    l = 0;
end
if integer
    [csi,eta] = get_eta(-x*x*is,l+length_sum,1);             
end
g(1,:) = zeros(1,M+1);
w(1,:) = zeros(1,M+1);
g(1,1)=1;
g(2,:)=((is.^tmpM)./(tmpM.*(2*l+tmpM+1))).*((2*l+3)*v(tmpM)); %=g_{1,j}
w(2,:)=-tmpM.*g(2,:);  %=w_{1,j}
y = 0;
yp = 0;
ypp = 0;
xpowin = x^(l-2);
for k=0:length_sum
      s = k+2;
      xpowin = xpowin*is*x;
      xpow = xpowin;
      for j=0:M
         tmp1 = (2*(l+s)+1)/((s+j)*(2*l+j+s+1));  % (3.9) en (3.13) 
         if 2*l + 2*s == 3
             g(3,j+1)=0;
             w(3,j+1)=0;
         else
           tmp2 = (s-j-2)*(2*l+s-j-1)/(2*l+2*s-3); 
           sum = v(1:j+1)*(g(2,j+1:-1:1).*(is.^[-1:1:j-1]))';
           sump = v(1:j+1)*((w(2,j+1:-1:1)-([1:j+1].*g(2,j+1:-1:1))).*is.^[-1:j-1])'; 
           g(3,j+1) = tmp1 * (sum-tmp2*is*g(1,j+1));
           w(3,j+1) = tmp1 * (sump-tmp2*is*w(1,j+1));
         end
         if j <= 1
             tpruf(k+1,j+1)=g(1,j+1);
         end
         xpow = xpow*is*x;
         t = l+k;
         u = t-j;
         if integer
           eta1 = eta(t+1);
           if t > 0
             eta2 = eta(t);
           else
             eta2 = csi;
           end
         else
             etat = get_eta_noninteger(-x*x*is,[t t-1]);
             eta1 = etat(1);
             eta2 = etat(2);
         end
         y = y + g(1,j+1)*eta1*xpow;    % term in phi   
         yp = yp + g(1,j+1)*(-u*eta1+eta2)*xpow;    
         if integer
           if t > 1  
            dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta(t-1))*xpow;      
           elseif t == 1
              dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+csi)*xpow; 
           elseif t == 0
              dypp = g(1,j+1)*((-x*x*is)*(j*eta(2)+eta1)+j*(j*eta1+csi))*xpow;  
           end 
         else
           eta3 =  get_eta_noninteger(-x*x*is,t-2);
           dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta3)*xpow; 
        end
         ypp = ypp+dypp;
      end
      if(abs(ypp-sch*y*r^2) <= 1e-14*abs(ypp))
          break;
      end
      g(1,:) = g(2,:);
      g(2,:) = g(3,:);
      w(1,:) = w(2,:);
      w(2,:) = w(3,:);      
  end

  
%   Finally y and cye are multiplied by x, yp and cype are unchanged
%   and ypp is divided by x. 
  y = y*x;
  ypp = ypp/x;
%   The values are converted in the original r variable. Only yp
%	and ypp are affected.
  yp = yp/alpha;     
  ypp = ypp*e*is;
%   Renormalization by an energy dependent multiplicative factor (fren)
  fnorm = 1e-100;
  fact = fnorm * prod(2*(1:l)+1); 
  fren = fact*alpha^(l+1);
  y = fren*y;  
  yp = fren*yp;
  ypp = fren*ypp;
    
  Y(:,h) = [y; yp];
  if nargout == 2
 % estimation of the number of zeros in y     
  nzeros = 0;
  y1 = 1;
  hx = x/10;
  xin = 0;
  for i=1:10
      x = i*hx;
      %b = get_eta(-x*x*is,l+length_sum,1);
      if integer
        [csi,eta] = get_eta(-x*x*is,l+k+1,1);
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta(l+1:l+k+1)'));
      else
%         for s=0:k
%           eta(s+1) = get_eta_noninteger(-x*x*is,l+s);
%         end
        eta = get_eta_noninteger(-x*x*is,l+(0:k));
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta'));
      end
      xx = is*1/x;
      y2 = (xx*((x*is).^[1:k+1]))*hlp;
      if y1*y2 <= 0
          nzeros = nzeros + 1;
      end
      y1=y2;
  end
  err = 0;
  if k==length_sum+1
      disp('err=1')
      err=1;
  end
  
  
% Generation of the Prufer phase
  tmp = atan(sn*y/yp);
  if (-1)^nzeros*yp < 0
      tmp = pi + tmp;
  end
  phase(h) = nzeros*pi + tmp;  

 %	Error test
 if err == 1
      if tmp/pi >= 0.02 && tmp/pi <= 0.98	
        if l==0
          den=ypp;
        else
	      den=ypp;
          if abs(den)<fnorm*l*(l+1)*r^(l+1)
              den = fnorm*l*(l+1)*r^(l+1);
          end
        end
        relerr = abs((ypp-sch*y)/den);  %relative error
        if relerr >= 1e-14
           disp('accuracy loss on I1');	
        end
      end
 end
end
end%for h
varargout{1}=Y;
if nargout == 2
   varargout{2}=phase; 
end
return;
    

function [yf,number_intervals]=laststep(e,part)
%LASTSTEP
% [yf,number_intervals]=laststep(e,part)
% For the test value e of the eigenvalue searched for,
% a lower limit r' for rmax is choosen. For that 
% eigenvalue only the interval (r0,r'] is actually involved.
global lpm_child schrod_object;
rmin = get(schrod_object,'min'); 
wkbmax =20;
wkb = 0;
i = part.imatch-1;
while 1
    i = i+1;
    if e <= part.V0(i)
        wkb = wkb + part.h(i)*sqrt(part.V0(i)-e);
    end
    if wkb >= wkbmax || i == length(part.h)
        %als wkb > wkbmax dan wil dit zeggen dat de E-lijn de potentiaal
        %gekruisd heeft en er al beetje voorbij is (hoeveel voorbij? zoveel
        %dat wkb >= wkbmax, dit is goede maat die wkb: als de V snel stijgt zal
        %wkb snel even groot zijn als wkbmax, als V traag stijgt dan zal
        %het minder snel zijn) en zal het een stuk verder afgekapt worden
        dobreak=true;
        for j=i:length(part.h)
            if e > part.V0(j) %dan mag het nog niet afgekapt worden
                %want de energie zal de potentiaal verder nog eens snijden.
                dobreak=false;
                break;
            end
        end
        if dobreak || i == length(part.h)
           break;
        end
    end
end
number_intervals = i;
if part.V0(number_intervals) >= e
    yf = [1;-sqrt(part.V0(number_intervals)-e)];
else %at rmax
    yf = [0;1];  %then: number_intervals = length(part.h) 
end
if wkb >= 1.5*wkbmax
    d = 1.5 * wkbmax - wkb + part.h(number_intervals)*sqrt(part.V0(number_intervals)-e);
    a.h = part.h(number_intervals) * d / (part.h(number_intervals)*sqrt(part.V0(number_intervals)-e));
    number_intervals = number_intervals-1;
    r = rmin + sum(part.h(1:number_intervals));
    [a.V0]=calculate_V(r,a.h);
    wkbnew = a.h * sqrt(a.V0-e);
    if wkbnew < d
        a.h = a.h *d /wkbnew;
        [a.V0,a.V(:,1)]=calculate_V(r,a.h);
    end
    [a.cu,a.cup,a.cv,a.cvp]=get_coeff(lpm_child,a.V(:,1),a.h);
    yf=propagate_solution(e,yf,1,1,a,false);
end
if yf(1) < 0
    yf = -yf;
end
if number_intervals < part.imatch
    warning('number_intervals < part.imatch!')
end



function [csi,eta] = get_eta(varargin)
%GET_ETA [csi,eta] = get_eta(Z,m)
% this calculates csi, eta_i, i=0..m, for given Z
Z = varargin{1};
m = varargin{2};
tz = 1;
eta=zeros(1,m+1);
mmax = 30;
mmaxm = 29;
if imag(Z)==0
if nargin==3 || abs(Z) < tz 
    %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
    %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
    wght2(1) = 1 / prod(2*(1:m-1)+1); 
    wght1(1) = wght2(1) / (2*m+1);
    wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
    wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
    if nargin == 3
      if wght1(mmax) == 0
        tz = 1;    
      else
        tz = (eps*wght1(1)/wght1(mmax))^(1/(mmaxm));
      end
    end
    if(abs(Z) < tz)
      tmp=(Z.^(0:mmaxm))';  
      eta(m+1) = wght1(1:mmax)*tmp;
      eta(m) = wght2(1:mmax)*tmp;
      for i=(m-1):-1:1
        eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
      end
      csi = Z * eta(2) + eta(1);
      return
    end
end
if Z >= tz  
      sq = sqrt(Z);
      csi = cosh(sq);
      if isinf(csi)
          wght2(1) = 1 / prod(2*(1:m-1)+1); 
          wght1(1) = wght2(1) / (2*m+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(m+1) = wght1(1:mmax)*tmp;
          eta(m) = wght2(1:mmax)*tmp;
          for i=(m-1):-1:1
              eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
          end
          csi = Z * eta(2) + eta(1);
          return;
      end
      eta(1) = sinh(sq)/sq;
else
      sq = sqrt(abs(Z));
      csi = cos(sq);
      eta(1) = sin(sq)/sq;
end    
eta(2)=(csi-eta(1))/Z;
for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
end

 
else %complex
    csi=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi-eta(1))/Z;
    for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
    end
end

