function [bp,rp,Vmin,Vmax]=calculate_partitions(varargin)
% generates the partition and the associated set of 
% quantities 
global lpm_child 
lpm_child = varargin{1};
schr = get(lpm_child,'schrod');
x_min = get(schr,'min');
x_max = get(schr,'max');
tol = get(lpm_child,'tol');


inf_max=false;
inf_min=false;
if isinf(x_max)
    inf_max=true;
end
if isinf(x_min)
    inf_min=true;    
end
if inf_min || inf_max
    bp=[];
    rp=[];
    Vmin=[];
    Vmax=[];
    return;
end

if nargin==1
%calculate partition:
% this furnishes the vector of stepsizes (bp.h) and the elements of the one
% step propagator (matrices bp.cu, bp.cup, bp.cv, bp.cvp) + bp.Vbar
% and bp.imatch (the interval at the rhs of which is the matching point)
rp=[];
x = x_min;
i=0;
trial_h = (x_max-x_min)/50;
while x < x_max
    i=i+1;
    [h,bp.V0(i),bp.V(:,i)] = get_step(x,trial_h,x_max-x,tol);
    [V01,V1] = calculate_V(x,h/2);
    [V02,V2] = calculate_V(x+h/2,h/2);
    [bp.cu(:,:,i),bp.cup(:,:,i),bp.cv(:,:,i),bp.cvp(:,:,i)] = get_coeff(lpm_child,bp.V(:,i),h);
    k=2*i-1;
    rp.h(k) = h/2;
    rp.V0(k)= V01;
    rp.V(:,k)= V1;
    [rp.cu(:,:,k),rp.cup(:,:,k),rp.cv(:,:,k),rp.cvp(:,:,k)] = get_coeff(lpm_child,rp.V(:,k),h/2); 
    k=2*i;
    rp.h(k) = h/2;
    rp.V0(k)= V02;
    rp.V(:,k)= V2;
    [rp.cu(:,:,k),rp.cup(:,:,k),rp.cv(:,:,k),rp.cvp(:,:,k)] = get_coeff(lpm_child,rp.V(:,k),h/2);
    x = x + h;
    trial_h = h;
    bp.h(i)=h;
end

else %nargin==5
    % part of the partition has already been calculated, it only has to be 
    % enlarged or diminished now.  
    bp = varargin{2};
    rp = varargin{3};
    x_min0 = varargin{4};
    x_max0 = varargin{5};
    if x_min0 < x_min
        %make the integration interval + mesh smaller
        x=x_min0;
        i=0;
        while x < x_min
            i=i+1;
            x = x + bp.h(i);
        end
        i=i+1;
        bp.h = bp.h(i:end);
        bp.V0 = bp.V0(i:end);
        bp.V = bp.V(:,i:end);
        bp.cu = bp.cu(:,:,i:end);
        bp.cv = bp.cv(:,:,i:end);
        bp.cup = bp.cup(:,:,i:end);
        bp.cvp = bp.cvp(:,:,i:end);
        k=i*2-1;
        rp.h = rp.h(k:end);
        rp.V0 = rp.V0(k:end);
        rp.V = rp.V(:,k:end);
        rp.cu = rp.cu(:,:,k:end);
        rp.cv = rp.cv(:,:,k:end);
        rp.cup = rp.cup(:,:,k:end);
        rp.cvp = rp.cvp(:,:,k:end);
        x_min0 = x;
    end
    if x_max0 > x_max
        %make the integration interval + mesh smaller
        x=x_max0;
        i=length(bp.h)+1;
        while x > x_max
            i=i-1;
            x = x - bp.h(i);
        end
        i=i-1;
        bp.h = bp.h(1:i);
        bp.V0 = bp.V0(1:i);
        bp.V = bp.V(:,1:i);
        bp.cu = bp.cu(:,:,1:i);
        bp.cv = bp.cv(:,:,1:i);
        bp.cup = bp.cup(:,:,1:i);
        bp.cvp = bp.cvp(:,:,1:i);
        k=i*2;
        rp.h = rp.h(1:k);
        rp.V0 = rp.V0(1:k);
        rp.V = rp.V(:,1:k);
        rp.cu = rp.cu(:,:,1:k);
        rp.cv = rp.cv(:,:,1:k);
        rp.cup = rp.cup(:,:,1:k);
        rp.cvp = rp.cvp(:,:,1:k);
        x_max0 = x;
    end
    if x_min0 > x_min
      x = x_min;
      trial_h = bp.h(1);  
      %enlarge the mesh
      i=0;
      while x < x_min0
        i=i+1;
        [h,bp0.V0(i),bp0.V(:,i)] = get_step(x,trial_h,x_min0-x,tol);
        bp0.h(i)=h;
        k=i*2-1;
        rp0.h(k)=h/2;
        rp0.h(k+1)=h/2;
        [V01,V1] = calculate_V(x,h/2);
        [V02,V2] = calculate_V(x+h/2,h/2);
        [bp0.cu(:,:,i),bp0.cup(:,:,i),bp0.cv(:,:,i),bp0.cvp(:,:,i)] = get_coeff(lpm_child,bp0.V(:,i),h); 
        rp0.V0(k)= V01;
        rp0.V(:,k)= V1;
        [rp0.cu(:,:,k),rp0.cup(:,:,k),rp0.cv(:,:,k),rp0.cvp(:,:,k)] = get_coeff(lpm_child,rp0.V(:,k),h/2); 
        k=2*i;
        rp0.V0(k)= V02;
        rp0.V(:,k)= V2;
        [rp0.cu(:,:,k),rp0.cup(:,:,k),rp0.cv(:,:,k),rp0.cvp(:,:,k)] = get_coeff(lpm_child,rp0.V(:,k),h/2);
        x = x + bp0.h(i);
        trial_h = bp0.h(i);
      end 
      bp.h = [bp0.h bp.h];
      bp.V0 = [bp0.V0 bp.V0];
      bp.V = [bp0.V bp.V];
      rp.h = [rp0.h rp.h];
      rp.V0 = [rp0.V0 rp.V0];
      rp.V = [rp0.V rp.V];
      [n1,n2,n3] = size(bp0.cu);
      [m1,m2,m3] = size(bp.cu);
      for i=1:m3
          bp0.cu(:,:,n3+i)=bp.cu(:,:,i);
          bp0.cv(:,:,n3+i)=bp.cv(:,:,i);
          bp0.cup(:,:,n3+i)=bp.cup(:,:,i);
          bp0.cvp(:,:,n3+i)=bp.cvp(:,:,i);
      end
      bp.cu = bp0.cu;
      bp.cup = bp0.cup;
      bp.cv = bp0.cv;
      bp.cvp = bp0.cvp;
      [n1,n2,n3] = size(rp0.cu);
      [m1,m2,m3] = size(rp.cu);
      for i=1:m3
          rp0.cu(:,:,n3+i)=rp.cu(:,:,i);
          rp0.cv(:,:,n3+i)=rp.cv(:,:,i);
          rp0.cup(:,:,n3+i)=rp.cup(:,:,i);
          rp0.cvp(:,:,n3+i)=rp.cvp(:,:,i);
      end
      rp.cu = rp0.cu;
      rp.cup = rp0.cup;
      rp.cv = rp0.cv;
      rp.cvp = rp0.cvp;
    end
    if x_max0 < x_max
        %enlarge the mesh
      x = x_max0;
      i=length(bp.h);
      trial_h = bp.h(end);  
      while x < x_max
        i=i+1;
        [h,bp.V0(i),bp.V(:,i)] = get_step(x,trial_h,x_max-x,tol);
        bp.h(i)=h;
        k=i*2-1;
        rp.h(k)=h/2;
        rp.h(k+1)=h/2;
        [V01,V1] = calculate_V(x,h/2);
        [V02,V2] = calculate_V(x+h/2,h/2);
        [bp.cu(:,:,i),bp.cup(:,:,i),bp.cv(:,:,i),bp.cvp(:,:,i)] = get_coeff(lpm_child,bp.V(:,i),h); 
        rp.V0(k)= V01;
        rp.V(:,k)= V1;
        [rp.cu(:,:,k),rp.cup(:,:,k),rp.cv(:,:,k),rp.cvp(:,:,k)] = get_coeff(lpm_child,rp.V(:,k),h/2); 
        k=2*i;
        rp.V0(k)= V02;
        rp.V(:,k)= V2;
        [rp.cu(:,:,k),rp.cup(:,:,k),rp.cv(:,:,k),rp.cvp(:,:,k)] = get_coeff(lpm_child,rp.V(:,k),h/2); 
        x = x + bp.h(i);
        trial_h = bp.h(i);
      end
    end
end

%rp: reference partition (used to form an estimation of the error in the
%calculated eigenvalues)
[Vmin,bp.imatch] = min(bp.V0);
if bp.imatch == length(bp.h)
    bp.imatch=length(bp.h)-1;
end
rp.imatch=bp.imatch*2;
Vmax = max(bp.V0);


