function [varargout]=get_initConditions(varargin)
% @PPM\GET_INITCONDITIONS
% [yi,yf]=get_initConditions
% [yi,yf,number_interval]=get_initConditions(e,part)
% [yi,yf,number_interval,phase]=get_initConditions(e,part)
% the boundary conditions are converted as initial conditions for the
% forwards and backwards integration of the Schrodinger equation from the
% two endpoints to the matching point.

global schrod_object;
if nargin == 0
  if isa(schrod_object,'schrod')
    a0 = get(schrod_object,'a0');
    b0 = get(schrod_object,'b0');
    a1 = get(schrod_object,'a1');
    b1 = get(schrod_object,'b1');
  elseif isa(schrod_object,'distorted_coulomb')
      a0 = 1;
      b0 = 0;
      a1 = 1;
      b1 = 0;
  end
  sq1 = sqrt(a0^2+b0^2);%*10^300;
  sq2 = sqrt(a1^2+b1^2);%*10^300;
  yi = [-b0 / sq1;a0 / sq1];
  yf = [-b1 / sq2;a1 / sq2];
  if (yi(1) <= 0)
    varargout{1} = -yi;
  else
    varargout{1} = yi;  
  end
  if (yf(1) <= 0)
    varargout{2} = -yf;
  else
    varargout{2} = yf;  
  end
elseif isa(schrod_object,'distorted_coulomb')
    if nargin ~= 2
        error('getInitconditions: 2 inputarguments expected')
    end
    [varargout{2},varargout{3}] = laststep(varargin{1},varargin{2});
    if nargout == 4
      [varargout{1},varargout{4}] = firststep(varargin{1},get(schrod_object,'min')); 
    elseif nargout == 3
      [varargout{1}] = firststep(varargin{1},get(schrod_object,'min'));  
    end
end

%---------------------------------------------------

function [yf,number_intervals]=laststep(e,part)
%LASTSTEP
% [yf,number_intervals]=laststep(e,part)
% For the test value e of the eigenvalue searched for,
% a lower limit r' for rmax is choosen. For that 
% eigenvalue only the interval (r0,r'] is actually involved.
global lpm_child schrod_object;
rmin = get(schrod_object,'min'); 
wkbmax =20;
wkb = 0;
i = part.imatch-1;
while 1
    i = i+1;
    if e <= part.V0(i)
        wkb = wkb + part.h(i)*sqrt(part.V0(i)-e);
    end
    if wkb >= wkbmax || i == length(part.h)
        %als wkb > wkbmax dan wil dit zeggen dat de E-lijn de potentiaal
        %gekruisd heeft en er al beetje voorbij is (hoeveel voorbij? zoveel
        %dat wkb >= wkbmax, 
        dobreak=true;
        for j=i:length(part.h)
            if e > part.V0(j) %dan mag het nog niet afgekapt worden
                %want de energie zal de potentiaal verder nog eens snijden.
                dobreak=false;
                break;
            end
        end
        if dobreak || i == length(part.h)
           break;
        end
    end
end
number_intervals = i;
if part.V0(number_intervals) >= e
    yf = [1;-sqrt(part.V0(number_intervals)-e)];
else %at rmax
    yf = [0;1];  %then: number_intervals = length(part.h) 
end
if wkb >= 1.5*wkbmax
    d = 1.5 * wkbmax - wkb + part.h(number_intervals)*sqrt(part.V0(number_intervals)-e);
    a.h = part.h(number_intervals) * d / (part.h(number_intervals)*sqrt(part.V0(number_intervals)-e));
    number_intervals = number_intervals-1;
    r = rmin + sum(part.h(1:number_intervals));
    [a.V0]=calculate_V(r,a.h);
    wkbnew = a.h * sqrt(a.V0-e);
    if wkbnew < d
        a.h = a.h *d /wkbnew;
        [a.V0,a.V(:,1)]=calculate_V(r,a.h);
    end
    [a.cu,a.cup,a.cv,a.cvp]=get_coeff(lpm_child,a.V(:,1),a.h);
    yf=propagate_solution(e,yf,1,1,a,false);
end
if yf(1) < 0
    yf = -yf;
end
if number_intervals < part.imatch
    warning('number_intervals < part.imatch!')
end

function [varargout]=firststep(e,rr)
%FIRSTSTEP
% [y]=firststep(e,r)
% [y,phase]=firststep(e,r)
% calculates the solution on the interval I_1: (0,r]
% for a distorted Coulomb problem.
%
% r could be a vector. Then the procedure is repeated for every element in
% r.

global lpm_child schrod_object
length_sum = 60;  

vmin = get(lpm_child,'Vmin');
if ~isa(schrod_object,'distorted_coulomb')
    error('firststep could only be applied when the schrodinger object is a distorted_coulomb object')
end
l = get(schrod_object,'l');
if l ~= round(l) || l < 0
    integer = false;
else
    integer = true;
end
Vs = get(schrod_object,'Vs');
if e-vmin > 1
    sn = sqrt(e-vmin);
else
    sn = 1;
end
for nterm=length(Vs):-1:1  
    if Vs(nterm) ~= 0
        break;
    end
end
M = max(nterm-1,1);

len = length(rr);
Y = zeros(2,len);
e0=e;
vs0=Vs(2);
l0=l;
for h=1:len
r=rr(h); 
e = e0;
Vs(2)=vs0;
l=l0;
sch = l*(l+1)/(r*r)-e;
sch = sch + Vs(1:M+1)*(r.^(-1:M-1))';
difev = (e-Vs(2))*r^2;
if(abs(difev) <= 0.00001 || abs(e)<0.00001) %remark 4     
    %ik had hier eerst 0.01 maar 0.00001 geeft betere resultaten, klopt opm4 wel?
      if difev >= 0                                 
          delta= e - (0.01/(r^2));
      else
          delta = e + (0.01/(r^2)); 
      end
      Vs(2) = Vs(2)-delta;
      e = e - delta;
end
if e < 0  %&& integer %remark 5 
    is = -1;
else
    is = 1;
end
%if integer
  x = sqrt(abs(e))*r;
  alpha = 1/sqrt(abs(e));
  %else
 % x = sqrt(e)*r;
 % alpha = 1/sqrt(e);
 %end
v = Vs(1:M+1).*alpha.^(1:M+1);
if imag(l) ~= 0 %avoid complex l-values % does this work???
    v(2)=v(2)+l*(l+1)/x^2; 
    l = 0;
end
if integer
    [csi,eta] = get_eta(-x*x*is,l+length_sum,1);             
end
g(1,:) = zeros(1,M+1);
w(1,:) = zeros(1,M+1);
g(1,1)=1;
g(2,:)=((is.^(1:M+1))./((1:M+1).*(2*l+(0:M)+2))).*((2*l+3)*v(1:M+1)); %=g_{1,j}
w(2,:)=-(1:M+1).*g(2,:);  %=w_{1,j}
y = 0;
yp = 0;
ypp = 0;
xpowin = x^(l-2);
for k=0:length_sum
      s = k+2;
      xpowin = xpowin*is*x;
      xpow = xpowin;
      for j=0:M
         tmp1 = (2*(l+s)+1)/((s+j)*(2*l+j+s+1));  % (3.9) en (3.13) 
         if 2*l + 2*s == 3
             g(3,j+1)=0;
             w(3,j+1)=0;
         else
           tmp2 = (s-j-2)*(2*l+s-j-1)/(2*l+2*s-3); 
           sum = v(1:j+1)*(g(2,j+1:-1:1).*(is.^(-1:1:j-1)))';
           sump = v(1:j+1)*((w(2,j+1:-1:1)-((1:j+1).*g(2,j+1:-1:1))).*is.^(-1:j-1))'; 
           g(3,j+1) = tmp1 * (sum-tmp2*is*g(1,j+1));
           w(3,j+1) = tmp1 * (sump-tmp2*is*w(1,j+1));
         end
         if j <= 1
             tpruf(k+1,j+1)=g(1,j+1);
         end
         xpow = xpow*is*x;
         t = l+k;
         u = t-j;
         if integer
           eta1 = eta(t+1);
           if t > 0
             eta2 = eta(t);
           else
             eta2 = csi;
           end
         else
             etat = get_eta_noninteger(-x*x*is,[t t-1]);
             eta1 = etat(1);
             eta2 = etat(2);
         end
         y = y + g(1,j+1)*eta1*xpow;    % term in phi   
         yp = yp + g(1,j+1)*(-u*eta1+eta2)*xpow;    
         if integer
           if t > 1  
            dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta(t-1))*xpow;      
           elseif t == 1
              dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+csi)*xpow; 
           elseif t == 0
              dypp = g(1,j+1)*((-x*x*is)*(j*eta(2)+eta1)+j*(j*eta1+csi))*xpow;  
           end 
         else
           eta3 =  get_eta_noninteger(-x*x*is,t-2);
           dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta3)*xpow; 
        end
         ypp = ypp+dypp;
      end
      if(abs(ypp-sch*y*r^2) <= 1e-14*abs(ypp))
          break;
      end
      g(1,:) = g(2,:);
      g(2,:) = g(3,:);
      w(1,:) = w(2,:);
      w(2,:) = w(3,:);      
  end

  
%   Finally y and cye are multiplied by x, yp and cype are unchanged
%   and ypp is divided by x. 
  y = y*x;
  ypp = ypp/x;
%   The values are converted in the original r variable. Only yp
%	and ypp are affected.
  yp = yp/alpha;     
  ypp = ypp*e*is;
%   Renormalization by an energy dependent multiplicative factor (fren)
  fnorm = 1e-100;
  fact = fnorm * prod(2*(1:1:l)+1); 
  fren = fact*alpha^(l+1);
  y = fren*y;  
  yp = fren*yp;
  ypp = fren*ypp;
    
  Y(:,h) = [y; yp];
  if nargout == 2
 % estimation of the number of zeros in y     
  nzeros = 0;
  y1 = 1;
  hx = x/10;
  for i=1:10
      x = i*hx;
      %b = get_eta(-x*x*is,l+length_sum,1);
      if integer
        [csi,eta] = get_eta(-x*x*is,l+k+1,1);
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta(l+1:l+k+1)'));
      else
        eta = get_eta_noninteger(-x*x*is,l+(0:k));
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta'));
      end
      xx = is*1/x;
      y2 = (xx*((x*is).^(1:k+1)))*hlp;
      if y1*y2 <= 0
          nzeros = nzeros + 1;
      end
      y1=y2;
  end
  err = 0;
  if k==length_sum+1
      disp('err=1')
      err=1;
  end
  
  
% Generation of the Prufer phase
  tmp = atan(sn*y/yp);
  if (-1)^nzeros*yp < 0
      tmp = pi + tmp;
  end
  phase(h) = nzeros*pi + tmp;  

 %	Error test
 if err == 1
      if tmp/pi >= 0.02 && tmp/pi <= 0.98	
        if l==0
          den=ypp;
        else
	      den=ypp;
          if abs(den)<fnorm*l*(l+1)*r^(l+1)
              den = fnorm*l*(l+1)*r^(l+1);
          end
        end
        relerr = abs((ypp-sch*y)/den);  %relative error
        if relerr >= 1e-14
           disp('accuracy loss on I1');	
        end
      end
 end
end
end%for h
varargout{1}=Y;
if nargout ==2
   varargout{2}=phase; 
end
return;





function [csi,eta] = get_eta(varargin)
%GET_ETA [csi,eta] = get_eta(Z,m)
% this calculates csi, eta_i, i=0..m, for given Z

Z = varargin{1};
m = varargin{2};
tz = 1;
eta=zeros(1,m+1);
mmax = 30;
mmaxm = 29;

if imag(Z)==0

if nargin==3 || abs(Z) < tz 
    %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
    %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
    wght2(1) = 1 / prod(2*(1:m-1)+1); 
    wght1(1) = wght2(1) / (2*m+1);
    wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
    wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
    if nargin == 3
      if wght1(mmax) == 0
        tz = 1;    
      else
        tz = (eps*wght1(1)/wght1(mmax))^(1/(mmaxm));
      end
    end
    if(abs(Z) < tz)
      tmp=(Z.^(0:mmaxm))';  
      eta(m+1) = wght1(1:mmax)*tmp;
      eta(m) = wght2(1:mmax)*tmp;
      for i=(m-1):-1:1
        eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
      end
      csi = Z * eta(2) + eta(1);
      return
    end
end
if Z >= tz  
      sq = sqrt(Z);
      csi = cosh(sq);
      if isinf(csi)
          wght2(1) = 1 / prod(2*(1:m-1)+1); 
          wght1(1) = wght2(1) / (2*m+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(m+1) = wght1(1:mmax)*tmp;
          eta(m) = wght2(1:mmax)*tmp;
          for i=(m-1):-1:1
              eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
          end
          csi = Z * eta(2) + eta(1);
          return;
      end
      eta(1) = sinh(sq)/sq;
else
      sq = sqrt(abs(Z));
      csi = cos(sq);
      eta(1) = sin(sq)/sq;
end    
eta(2)=(csi-eta(1))/Z;
for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
end

 
else %complex
    csi=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi-eta(1))/Z;
    for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
    end
end


    