function [lp]=get_regular_problem(lp,pmax,indices)
% GET_REGULAR_PROBLEM
% transforms a problem with endpoint(s)=inf into a
% problem with finite endpoints = truncation 

schrod=get(lp,'schrod');
lpm_child=lp;
if isa(schrod,'transformed_slp')
   x_max = get(get(schrod,'slp'),'max');
   x_min = get(get(schrod,'slp'),'min');
else
   x_min = get(schrod,'min');
   x_max = get(schrod,'max');
end
V=get(schrod,'V');
infmax=false;
infmin=false;
if isinf(x_max)
    infmax=true;
end
if isinf(x_min)
    infmin=true;
end

if indices
   classif=get(schrod,'classification');
   cutoff=classif.cutoff;
   wkb=max((pmax+1),20); 
   if infmax
     [x_max,emax0,vminx]=get_x_max(V,wkb,0,-1e300,0,V(0));
     %get_estimation_highest_eigenvalue
   end
   if infmin
     [x_min,emax1]=get_x_min(V,wkb,0,-1e300,V(0));
   end
   [bp,Vmin]=get_coarse_mesh(lp,x_min,x_max);  
   if infmax
      nmax0 = locate_eigenvalue(emax0,bp,Vmin,1,length(bp.h));
   else
      nmax0 =1e300; 
   end
   if infmin 
      nmax1 = locate_eigenvalue(emax1,bp,Vmin,1,length(bp.h));
   else
       nmax1 = nmax0;
   end
   flag=0;
   equals=0;
   it=0;
   while min(nmax0,nmax1)<pmax
       %interval moet groter
       it=it+1;
       if flag
           break;
       end
       x_mino=x_min;
       x_maxo=x_max;
       if infmax && nmax0<pmax+1
          [x_max,emax0,vminx]=get_x_max(V,max((pmax-nmax0)/2,1),x_maxo,emax0,vminx,cutoff);
         if x_maxo < x_max
            x = x_maxo;
            i=length(bp.h);
            trial_h = (x_max-x_maxo)/50;  
            while x < x_max
                i=i+1;
                [bp.h(i),bp.V0(i),bp.V(:,i)] = get_step(x,trial_h,x_max-x,1e-3);
                [bp.cu(:,:,i),bp.cup(:,:,i),bp.cv(:,:,i),bp.cvp(:,:,i)] = get_coeff(lp,bp.V(:,i),bp.h(i)); 
                x = x + bp.h(i);
                trial_h = bp.h(i);
            end
         end
       end
       if infmin && nmax1<pmax+1
          [x_min,emax1]=get_x_min(V,max((pmax-nmax1)/2,1),x_mino,emax1,cutoff);
          if x_mino > x_min
             x = x_min;
             trial_h = (x_maxo-x_mino)/50;  
             k=1;
             %enlarge the coarse mesh:
             while x < x_mino
                 [h,Vb,Vt] = get_step(x,trial_h,x_mino-x,1e-3);
                 bp.h = [h bp.h];
                 bp.V0=[Vb bp.V0];
                 bp.V=[Vt' bp.V];
                 [cu(:,:,k),cup(:,:,k),cv(:,:,k),cvp(:,:,k)] = get_coeff(lpm_child,Vt,h); 
                 k=k+1;
                x = x + h;
                trial_h = h;
             end     
             n3=size(bp.cu,3);
             m3=size(cu,3);
             for k=1:n3 
                cu(:,:,m3+k) = bp.cu(:,:,k);
                cup(:,:,m3+k) = bp.cup(:,:,k);
                cv(:,:,m3+k) = bp.cv(:,:,k);
                cvp(:,:,m3+k) = bp.cvp(:,:,k);
             end
             bp.cu=cu;
             bp.cv=cv;
             bp.cup=cup;
             bp.cvp=cvp;   
          end
        end 
        [Vmin,bp.imatch] = min(bp.V0);
        if infmax
           nmax0 = locate_eigenvalue(emax0,bp,Vmin,1,length(bp.h));
        else
           nmax1=1e300; 
        end
        if infmin
           nmax1 = locate_eigenvalue(emax1,bp,Vmin,1,length(bp.h));
        else
           nmax1 = nmax0; 
        end
        if it>25
            if equals==0
                nmax_eq=round(min(nmax0,nmax1));
            end
            if round(min(nmax0,nmax1)) == nmax_eq
                equals=equals+1;
                if equals>10
                    error(['Error!!! :  Sorry, the eigenvalues with indices larger than ' num2str(nmax_eq-1) ' can ''t be calculated'])
                end
            else
                equals=0;
            end
        end
        pause(1e-200) % stop the calculation in the GUI?
        load stop
        if stop
          error('Error!!! :  The calculation was interrupted');
        end
   end
   if floor(min(nmax0,nmax1))==pmax
       if abs(V(x_max)-V(x_max+x_max/5)/(x_max/5))<1e-2
          x_max=x_max+abs(x_max)/5;
       end
   end
   problem = get(lp,'schrod');
   if infmax
      problem = set(problem,'max',x_max);
   end
   if infmin
      problem = set(problem,'min',x_min);
   end
   lp = set(lp,'schrod',problem);
   lp = set(lp,'truncated',true);
   return;
else
 pmax10=abs(pmax/15);   
 if isinf(x_max)
    %find xmax
    if isinf(x_min)
      x=0;         %beter minimum van V
    else
      x=x_min;
    end
    he=0.1;
    vx=V(x);
    vmin=vx;
    vx_prev=vx;
    while vx<pmax || V(x+min(0.1,he))<vx
      x=x+he;
      vx=V(x);
      if (vx-vx_prev)/he >-1e-4  && (vx-pmax)*(vmin-pmax)<0
          break;
      end
      if vx<vmin
          vmin=vx;
      end
      vx_prev=vx;
      he=he*1.5;
    end
    xlow=x-he*2/3;
    xup=x;
    it = 0;
    while true
      it=it+1;  
      xc=(xup+xlow)/2;
      vxc=V(xc);
      if vxc < pmax
        xlow = xc;
      else
        xup = xc;
        if abs(vxc - pmax)<pmax10 
            break;
        end
      end
      if it>10
          break;
      end
    end
    x=xup;
    sums=0;
    he=abs(x/20);
    vx1=V(x);
    while sums<18
      vx0=vx1;  
      x=x+he;
      vx1=V(x);
      vm=(vx0+vx1)/2;
      if vx1==vx0 || ((vx1-vx0)/he < 0 && (vx1-vx0)/he >-1e-3)
          x=x+2*he;
          break;
      end
      if vm-pmax >=0
         sums=sums+he*sqrt(vm-pmax);
      else
          sums=0;
      end
    end
    x_max=x;
 end
 if isinf(x_min)
    %find xmin
    x=min(0,x_max);  %beter ipv 0 de plaats waar het minimum in V ligt
    he=0.1;
    vx=V(x);
    vmin=vx;
    vx_prev=vx;
    while vx<pmax || V(x-min(0.1,he))<vx
      x=x-he;
      vx=V(x);
      if (vx-vx_prev)/he >-1e-4  && (vx-pmax)*(vmin-pmax)<0
          break;
      end
      if vx<vmin
          vmin=vx;
      end
      vx_prev=vx;
      he=he*1.5;
    end
    xlow=x+he*2/3;
    xup=x;
    it=0;
    while true
      it=it+1; 
      xc=(xup+xlow)/2;
      vxc=V(xc);
      if vxc < pmax
        xlow = xc;
      else
        xup = xc;
        if abs(vxc - pmax)<pmax10 
            break;
        end
      end
      if  it>10
          break;
      end
    end
    x=xup;
    sums=0;
    he=abs(x)/20;
    vx1=V(x);
    while sums<18
      vx0=vx1;  
      x=x-he;
      vx1=V(x);
      vm=(vx0+vx1)/2;
      if vx1==vx0 || ((vx1-vx0)/he < 0 && (vx1-vx0)/he >-1e-3)
          x=x-2*he;
          break;
      end
      if vm-pmax >=0
         sums=sums+he*sqrt(vm-pmax);
      else
          sums=0;
      end
    end
    x_min=x;
 end
 problem = get(lp,'schrod');
 if infmax
      problem = set(problem,'max',x_max);
 end
 if infmin
      problem = set(problem,'min',x_min);
 end
 lp = set(lp,'schrod',problem);
 lp = set(lp,'truncated',true);
end

function [x_max,emax,vminx] = get_x_max(V,wkb,x,emax0,vminx,cutoff)
sums = 0;
vmin = min(V(x),cutoff);
he=0.1;
vx_prev=vmin;
x0=x;
while true
       x=x+he;
       vx=V(x);
       if (vx > vmin && vx>vx_prev) || (vx-vx_prev)/he >-1e-3  
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         if vmin == vx
            vminx= x;
         end
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10) || x>x0+max(5e3,abs(x0)*1e2)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x+1)/10);
end
x_max=x;
if abs(vx-V(x+he))<1e-2
    x_max=x_max+abs(x)/10;
end
x0=max(0.7*x_max,vminx);
x=x0;
he=(x_max-x0)/40;
emax=V(x);
xu=x_max;
xl=0;
while true
 xx=x0+(0:39)*he;
 vx=V(xx)-emax;
 tmp=[0 find(vx<0)];
 te=tmp(end);
 sums=sum(sqrt(vx(te+1:end))*he);
  if sums >15
    xl=x0;
     if (xu-xl)/xl <0.25 && sums <50  
       break;
     end
 else
    xu=x0;
 end
 x0=max((xl+xu)/2,vminx);
 x=x0;    
 emax=V(x);
 he=(x_max-x)/40;
end
emax=max(emax,emax0*1.0001);


function [x_min,emax] = get_x_min(V,wkb,x,emax0,cutoff)
sums=0;
vmin = min(V(x),cutoff);
he=0.1;
vx_prev=vmin;
while true
       x=x-he;
       vx=V(x);  
       if (vx > vmin && vx>vx_prev ) || (vx-vx_prev)/he >-1e-3  
          sums=sums+sqrt(vx-vmin)*he;
       else
         vmin=min(vx,cutoff);
         if vmin == vx
             vminx=vmin;
         end
         sums=0;
       end
       if sums > wkb || (abs(vx-cutoff)<1e-15 && abs(vx-vx_prev)/he <1e-10)
         break;
       end
       vx_prev=vx;
       he=min(he*1.2,abs(x-1)/10);
end
x_min=x;
if abs(vx-V(x-he))<1e-2
    x_min=x_min-abs(x)/10;
end
x0=0.7*x_min;
x=x0;
he=abs(x_min-x0)/40;
emax=V(x);
xu=x_min;
xl=0;
while true
 xx=x0-(0:39)*he;
 vx=V(xx)-emax;
 tmp=[1 find(vx<0)];
 te=tmp(end);
 sums=sum(sqrt(vx(te:end))*he);
  if sums >15
    xl=x0;
     if (xu-xl)/xl <0.25 && sums <50  
       break;
     end
 else
    xu=x0;
 end
 x0=(xl+xu)/2; 
 x=x0;    
 emax=V(x);
 he=abs(x_min-x)/40;
end
emax=max(emax,emax0*1.0001);
 

%-------------------------------------------------
function [bp,Vmin]=get_coarse_mesh(lpm_child,x_min,x_max)
x = x_min;
i=0;
trial_h = (x_max-x_min)/50;
if ~isreal(x_max) && imag(x_max) > 0
    error('Error!!! : no success');
end
while x < x_max
    i=i+1;
    [bp.h(i),bp.V0(i),bp.V(:,i)] = get_step(x,trial_h,x_max-x,1e-3);
    [bp.cu(:,:,i),bp.cup(:,:,i),bp.cv(:,:,i),bp.cvp(:,:,i)]  = get_coeff(lpm_child,bp.V(:,i),bp.h(i)); 
    x = x + bp.h(i);
    trial_h = bp.h(i);
end
[Vmin,bp.imatch] = min(bp.V0);