function varargout = propagate_solution(e,y,imin,imax,part,forward)
% @LPM\PROPAGATE_SOLUTION 
% [y,theta] = propagate_solution(e,y,imin,imax,part,forward)
% This propagates the solution from the leftmost point of interval
% imin up to the rightmost point of interval imax if forward = true
% and viceversa if forward = false. 
% Argument part contains the standard data regarding the partition.
global lpm_child;
pruf=false;

if nargout==2
 Vmin = get(lpm_child,'Vmin');
 S = 1;
 if((e-Vmin)> 1) 
      S = sqrt(e-Vmin);
 end
 pruf=true;
 theta=0;
end
h=part.h;
Vbar=part.V0;

if forward
 t = imin:imax;
else
 t = imax:-1:imin;    
end
if imin>imax
    varargout{1}=y;
    return;
end
Tm=get_propagation_matrices(e,part,t,forward);
for i = 1:length(t)
    y1=y;
    T=Tm(:,:,i);
    tmp2 = T*y;
    if abs(tmp2(1))>10^50 || abs(tmp2(2))>10^50 || isnan(tmp2(1))
        %avoid overflow
        m = max(abs(y));
        if m == 0
            m = 10^50;
        end
        y = y/m;
        tmp2 = T*y;
        if abs(tmp2(1))>10^100 || abs(tmp2(2))>10^100 
            y = y/10^100;
            tmp2 = T*y;
        end
    end
    y = tmp2;
    if isinf(y)
        T=T./1e200;
        y = T*y1;
        if isinf(y)  
         error('y = Inf OVERFLOW / It may be better to make the integration interval smaller');
        end
    end
    if isnan(y(1)) || isnan(y(2)) 
       error('Propagate_solution:Nan','y = NaN/ It may be better to make the integration interval smaller'); 
    end
    if pruf
         yr=real(y);
         y1r=real(y1);
      if forward
          [theta0,theta1] = prufer(y1r(1),y1r(2),yr(1),yr(2),e,Vbar(t(i)),h(t(i)),S);
      else
          [theta0,theta1] = prufer(yr(1),yr(2),y1r(1),y1r(2),e,Vbar(t(i)),h(t(i)),S);
      end
      theta=theta+theta1-theta0;
    end
end
varargout{1}=y;
if pruf
    varargout{2}=theta;
end