function [theta0,theta1]=prufer(y0,y0p,y1,y1p,e,v,h,S)
%@CPM\PRUFER [theta0,theta1]=prufer(y0,y0p,y1,y1p,e,v,h,S)
%
% y0, y0p ,y1 ,y1p:     - the values of y and y' at the ends of an interval of stepsize h
%       v         :     - the constant approximation of the potential on that interval
%       e         :     - the energy
% theta0, theta1  :     - the values of the theta function corresponding to the scaled
%                         prufer form (with scale factor S) at the two ends of the interval.

if e > v %(i): e > Vi
    Si = sqrt(e-v); %local scale factor
    if y0p ~= 0
       theta0 = atan(Si*y0/y0p);
    elseif y0 > 0
         theta0 = pi/2;
    elseif y0 < 0
         theta0 = -pi/2;
    else
        theta0=0;
    end
    nphi = floor((theta0 + Si*h)/pi); 
    phibar = theta0 + Si*h - nphi*pi; %between -pi/2 and pi/2
    if y1p ~= 0
        phistar = atan(Si*y1/y1p);
    elseif y1 > 0
        phistar = pi/2;
    elseif y1 < 0
        phistar = -pi/2;
    else
        phistar = 0;
    end
    deltaphi = phistar - phibar;
    if deltaphi > pi/2
        deltaphi = deltaphi-pi; 
    elseif deltaphi < -pi/2
        deltaphi = deltaphi + pi;
    end
    theta1 = theta0 + Si*h + deltaphi;
    if Si == S
        return;
    end
%   else rescaling procedure
    sigma = S/Si;
    theta0 = theta0 + atan2((sigma-1)*sin(theta0)*cos(theta0),...
        1+(sigma-1)*sin(theta0)^2);
    theta1 = theta1 + atan2((sigma-1)*sin(theta1)*cos(theta1),...
        1+(sigma-1)*sin(theta1)^2);
    return;
%(ii): e <= Vi   
elseif y0p ~= 0
    theta0 = atan(S*y0/y0p);
elseif y0 > 0 
    theta0 = pi/2;
elseif y0 < 0
    theta0 = -pi/2;
else
    theta0=0;
end
if y1p ~= 0
    theta1 = atan(S*y1/y1p);   %ylp is soms Inf
elseif y1 > 0 
    theta1 = pi/2;
elseif y1 < 0
    theta1 = -pi/2;
else
    theta1=0;
end
if y0*y1 >= 0 
    if theta0 > 0 && theta1 < 0
        theta1 = theta1 + pi;
    elseif theta0 < 0 && theta1 > 0
        theta1 = theta1 - pi;
    end
    return
elseif theta0*theta1 > 0 
    theta1 = theta1 + pi;
    return
end