function [x,y,yp,status]=calculate_eigenfunction(varargin)
%@PPM\CALCULATE_EIGENFUNCTION
% [x,y,yp] = calculate_eigenfunction(e)
%   This calculates the normalized eigenfunction (vector yn), 
%   associated to some input eigenvalue e, and its first 
%   derivative (vector ynp) at the meshpoints (x) of the partition.
%
%      part              - the standard data concerning the partition 
%                          of the domain of the equation.

global ppm_child schrod_object
e = varargin{1};
part = varargin{2};
status = 0;
if isa(schrod_object,'distorted_coulomb')
    distorted=true;
else
    distorted=false;
end
if distorted
    [yi,yei,yf,yef,number_intervals,phase] = get_ppm(ppm_child,'init_conditions',e,part);
else
    [yi,yf]=get_ppm(ppm_child,'init_conditions');
    yef=[0;0];
    yei=[0;0];
    number_intervals = length(part.h);
end


yn = zeros(2,number_intervals+1);

imatch = part.imatch;

i=1;
while imatch <= length(part.h)
  [yl,yel] = propagate_solution3(e,yi,yei,1,imatch,part,true);
  [yr,yer] = propagate_solution3(e,yf,yef,imatch+1,number_intervals,part,false);
  qr = [yer(2) -yer(1)]*yr;
  ql = [-yel(2) yel(1)]*yl;
  if (isinf(qr) || isinf(ql) || isnan(ql) || isnan(qr)) && ~distorted
      if nargin >2
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2},varargin{3},varargin{4});
      else
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2});
      end
     return;
  end
  % to avoid problems when in matchpoint yl and yr are near 0, and it isn't
  % clear whether the sign of the right part of the eigenfunction has to be 
  % changed or not.
  if imatch == length(part.h)
      break;
  end
  if ((abs(yr(1)) < 0.2 || abs(yl(1)) < 0.2) && abs(yr(2)) < 0.2) || (abs(yr(1)/yr(2))<1e-13 && abs(yl(1)/yl(2))<1e-13)
    imatch = min(imatch + i,length(part.h));
    i = i*2;
  else
    break;
  end
end
if abs(yr(1)) < 0.1 || (abs(yr(2))+abs(yl(2)) - (abs(yr(1))+abs(yl(1))) > (abs(yr(2))+abs(yl(2))))  
       gap = yl(2)/yr(2);
else
     gap = yl(1)/yr(1);
end
fnorm = ql+qr*gap*gap;
if isinf(fnorm) && ~distorted
     if nargin >2
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2},varargin{3},varargin{4});
      else
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2});
     end
     return;
end
sqfnorm = 1;
if fnorm  <= 0 
	warning('warning:eigenfunction',' calculate_eigenfunction: There are problems with the normalization.')	
    status =1; 
else	
    sqfnorm = sqrt(fnorm);  
end
if distorted
    nadd = max(round(10 * phase / 3.14),4);
    h = get(schrod_object,'min') / nadd;
    ya(:,1) = [0;0];
    x = 0:h:(get(schrod_object,'min')-h);
    if get(schrod_object,'l') == 0
        ya(2,1) = 1e-100/sqfnorm;
    end
    [ya(:,2:nadd)]=firststep(e,x(2:nadd));
    ya = ya./sqfnorm;
else 
    x=[];
    ya=[];
end
x_min = get(schrod_object,'min');
x_max = get(schrod_object,'max');
V = get(schrod_object,'V'); 
symm = true;
mid = (x_max+x_min)/2;
for i=mid:(x_max-x_min)/200:x_max
 if V(mid+i) ~= V(mid-i)
      symm =false;
      break;
  end
end
if symm
    gap = sign(gap);
end
yi = yi./sqfnorm;
yf = yf*gap/sqfnorm;
yn(:,1) = yi;
q = 0;

for i=1:imatch
    [yn(:,i+1),ye] = propagate_solution3(e,yn(:,i),[0;0],i,i,part,true);
    q = q + [yn(2,i+1) -yn(1,i+1)]*ye;	
end 
ymatch = yn(:,imatch+1);
yn(:,number_intervals+1) = yf;
qr=0;
for i=number_intervals:-1:(imatch+1)  
	[yn(:,i),ye] = propagate_solution3(e,yn(:,i+1),[0;0],i,i,part,false);
  	qr = qr + [-yn(2,i) yn(1,i)]*ye;	
end
if ymatch(2)*yn(2,imatch+1)<0 && abs(ymatch(2))>1e-1
    yn(:,imatch+1:number_intervals+1)=yn(:,imatch+1:number_intervals+1)*-1;
end
q=q+qr;
if isinf(q) && ~distorted
    if nargin >2
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2},varargin{3},varargin{4});
    else
        [x,y,yp]=calculate_eigenfunction_overflow(varargin{1},varargin{2});
    end
    return;
end
if q <= 0
   sq = 1;
   warning('warning:eigenfunction1',' calculate_eigenfunction: There are problems with the normalization.' )
   status = 1;
else	
   sq=1/sqrt(q);
end	


yn = [ya yn].*sq;
for i=1:size(yn,2) 
    if yn(1,i) > 0.1
        break;
    end
    if yn(1,i) < -0.1
        yn = (-1).*yn;
        break;
    end
end
y = yn(1,:);
yp = yn(2,:);

xp = x_min;
xx = cumsum(part.h(1:number_intervals)) + xp;
x = [x xp xx];

if nargin>2
    a = varargin{3};
    b = varargin{4};
    for i=1:length(x)
        if x(i)>=a || abs(x(i)-a)<1e-12
            x = x(i:end);
            y = y(i:end);
            yp= yp(i:end);
            break;
        end
    end
    for i=length(x):-1:1
        if x(i)<=b || abs(x(i)-b)<1e-12
            x = x(1:i);
            y = y(1:i);
            yp= yp(1:i);
            break;
        end
    end
end


function [y,ye] = propagate_solution3(e,y,ye,imin,imax,part,forward)
% @CPM\PROPAGATE_SOLUTION3 
% [y,ye] = propagate_solution3(e,y,ye,imin,imax,part,forward)
% This propagates the solution from the leftmost point of interval
% imin up to the rightmost point of interval imax if forward = true
% and viceversa if forward = false. y(1), y(2), ye(1) and ye(2) 
% represent y, y',y_e and y'_e both in the input and in the output.
% Argument part contains the standard data regarding the partition.
global ppm_child
if forward
 k = imin:imax;
else
 k = imax:-1:imin;    
end
if isempty(k)
    return;
end
[Tm,Tem]=get_ppm(ppm_child,'propagation_matrices',e,part,k,forward);
for i = 1:length(k)
    T=Tm(:,:,i);
    Te=Tem(:,:,i);
    ye = Te*y+T*ye;
    y = T*y;
end



function [varargout]=firststep(e,rr)
%@CPM/FIRSTSTEP
% [y,ye]=firststep(e,r)
% [y,ye,phase]=firststep(e,r)
% calculates the solution on the interval I_1: (0,r]
% for a distorted Coulomb problem.
%
% r could be a vector. Then the procedure is repeated for every element in
% r.

global cpm_child schrod_object
length_sum = 60;  

vmin = get(cpm_child,'Vmin');
if ~isa(schrod_object,'distorted_coulomb')
    error('firststep could only be applied when the schrodinger object is a distorted_coulomb object')
end
l = get(schrod_object,'l');
if l ~= round(l) || l < 0
    integer = false;
else
    integer = true;
end
Vs = get(schrod_object,'Vs');
if e-vmin > 1
    sn = sqrt(e-vmin);
else
    sn = 1;
end
for nterm=length(Vs):-1:1  
    if Vs(nterm) ~= 0
        break;
    end
end
M = max(nterm-1,1);

len = length(rr);
Y = zeros(2,len);
Ye = zeros(2,len);
e0=e;
vs0=Vs(2);
l0=l;
for h=1:len
r=rr(h); 
e = e0;
Vs(2)=vs0;
l=l0;
sch = l*(l+1)/(r*r)-e;
sch = sch + Vs(1:M+1)*(r.^(-1:M-1))';
difev = (e-Vs(2))*r^2;
if(abs(difev) <= 0.00001 || abs(e)<0.00001) %remark 4     
    %ik had hier eerst 0.01 maar 0.00001 geeft betere resultaten, klopt opm4 wel?
      if difev >= 0                                 
          delta= e - (0.01/(r^2));
      else
          delta = e + (0.01/(r^2)); 
      end
      Vs(2) = Vs(2)-delta;
      e = e - delta;
end
if e < 0  %&& integer %remark 5 
    is = -1;
else
    is = 1;
end
x = sqrt(abs(e))*r;
alpha = 1/sqrt(abs(e));
v = Vs(1:M+1).*alpha.^(1:M+1);
if imag(l) ~= 0 %avoid complex l-values % does this work???
    v(2)=v(2)+l*(l+1)/x^2; 
    l = 0;
end
if integer
    [csi,eta] = get_eta(-x*x*is,l+length_sum,1);             
end
g(1,:) = zeros(1,M+1);
w(1,:) = zeros(1,M+1);
g(1,1)=1;
g(2,:)=((is.^(1:M+1))./((1:M+1).*(2*l+(0:M)+2))).*((2*l+3)*v(1:M+1)); %=g_{1,j}
w(2,:)=-(1:M+1).*g(2,:);  %=w_{1,j}
y = 0;
yp = 0;
ypp = 0;
cye = 0;
cype = 0;
xpowin = x^(l-2);
for k=0:length_sum
      s = k+2;
      xpowin = xpowin*is*x;
      xpow = xpowin;
      for j=0:M
         tmp1 = (2*(l+s)+1)/((s+j)*(2*l+j+s+1));  % (3.9) en (3.13) 
         if 2*l + 2*s == 3
             g(3,j+1)=0;
             w(3,j+1)=0;
         else
           tmp2 = (s-j-2)*(2*l+s-j-1)/(2*l+2*s-3); 
           sum = v(1:j+1)*(g(2,j+1:-1:1).*(is.^(-1:1:j-1)))';
           sump = v(1:j+1)*((w(2,j+1:-1:1)-((1:j+1).*g(2,j+1:-1:1))).*is.^(-1:j-1))'; 
           g(3,j+1) = tmp1 * (sum-tmp2*is*g(1,j+1));
           w(3,j+1) = tmp1 * (sump-tmp2*is*w(1,j+1));
         end
         if j <= 1
             tpruf(k+1,j+1)=g(1,j+1);
         end
         xpow = xpow*is*x;
         t = l+k;
         u = t-j;
         if integer
           eta1 = eta(t+1);
           if t > 0
             eta2 = eta(t);
           else
             eta2 = csi;
           end
         else
             etat = get_eta_noninteger(-x*x*is,[t t-1]);
             eta1 = etat(1);
             eta2 = etat(2);
         end
         y = y + g(1,j+1)*eta1*xpow;    % term in phi   
         yp = yp + g(1,j+1)*(-u*eta1+eta2)*xpow;    
         cype = cype+w(1,j+1)*(-u*eta1+eta2)*xpow;
         cye = cye+w(1,j+1)*eta1*xpow;
         if integer
           if t > 1  
            dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta(t-1))*xpow;      
           elseif t == 1
              dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+csi)*xpow; 
           elseif t == 0
              dypp = g(1,j+1)*((-x*x*is)*(j*eta(2)+eta1)+j*(j*eta1+csi))*xpow;  
           end 
         else
           eta3 =  get_eta_noninteger(-x*x*is,t-2);
           dypp = g(1,j+1)*(u*(u+1)*eta1+...
                (-2*u+1)*eta2+eta3)*xpow; 
        end
         ypp = ypp+dypp;
      end
      if(abs(ypp-sch*y*r^2) <= 1e-14*abs(ypp))
          break;
      end
      g(1,:) = g(2,:);
      g(2,:) = g(3,:);
      w(1,:) = w(2,:);
      w(2,:) = w(3,:);      
end

  
%   Finally y and cye are multiplied by x, yp and cype are unchanged
%   and ypp is divided by x. 
  y = y*x;
  cye = cye*x;
  ypp = ypp/x;
%	The partial derivatives against e are constructed
  ye = (x*yp+cye)/(2*e);   %(3.11)
  ype = (x*ypp+cype)/(2*e);  %(3.12)
%   The values are converted in the original r variable. Only yp,ype
%	and ypp are affected.
  yp = yp/alpha;     
  ype = ype/alpha;
  ypp = ypp*e*is;
%   Renormalization by an energy dependent multiplicative factor (fren)
  fnorm = 1e-100;
  fact = fnorm * prod(2*(1:1:l)+1); 
  fren = fact*alpha^(l+1);
  ye = fren*(-(l+1)*y/(2*e)+ye);
  ype = fren*(-(l+1)*yp/(2*e)+ype);   
  y = fren*y;  
  yp = fren*yp;
  ypp = fren*ypp;
    
  Y(:,h) = [y; yp];
  Ye(:,h) =[ye; ype];
 if nargout == 3
 % estimation of the number of zeros in y     
  nzeros = 0;
  y1 = 1;
  hx = x/10;
  for i=1:10
      x = i*hx;
      %b = get_eta(-x*x*is,l+length_sum,1);
      if integer
        [csi,eta] = get_eta(-x*x*is,l+k+1,1);
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta(l+1:l+k+1)'));
      else
%         for s=0:k
%           eta(s+1) = get_eta_noninteger(-x*x*is,l+s);
%         end
        eta = get_eta_noninteger(-x*x*is,l+(0:k));
        hlp =((tpruf(1:k+1,1)+(x*is*tpruf(1:k+1,2))).*(eta'));
      end
      xx = is*1/x;
      y2 = (xx*((x*is).^(1:k+1)))*hlp;
      if y1*y2 <= 0
          nzeros = nzeros + 1;
      end
      y1=y2;
  end
  err = 0;
  if k==length_sum+1
      disp('err=1')
      err=1;
  end
  
  
% Generation of the Prufer phase
  tmp = atan(sn*y/yp);
  if (-1)^nzeros*yp < 0
      tmp = pi + tmp;
  end
  phase(h) = nzeros*pi + tmp;  

 %	Error test
 if err == 1
      if tmp/pi >= 0.02 && tmp/pi <= 0.98	
        if l==0
          den=ypp;
        else
	      den=ypp;
          if abs(den)<fnorm*l*(l+1)*r^(l+1)
              den = fnorm*l*(l+1)*r^(l+1);
          end
        end
        relerr = abs((ypp-sch*y)/den);  %relative error
        if relerr >= 1e-14
           disp('accuracy loss on I1');
        else
	       err=0;	
        end
      end
 end
end
end%for h
varargout{1}=Y;
varargout{2}=Ye;
if nargout ==3
   varargout{3}=phase; 
end
return;



function [csi,eta] = get_eta(varargin)
%GET_ETA [csi,eta] = get_eta(Z,m)
% this calculates csi, eta_i, i=0..m, for given Z
Z = varargin{1};
m = varargin{2};
tz = 1;
eta=zeros(1,m+1);
mmax = 30;
mmaxm = 29;
if imag(Z)==0
if nargin==3 || abs(Z) < tz 
    %wght1(q+1) = Coefficient of Z^q in series expansion for eta_m.
    %wght2(q+1) = Coefficient of Z^q in series expansion for eta_{m-1}.
    wght2(1) = 1 / prod(2*(1:m-1)+1); 
    wght1(1) = wght2(1) / (2*m+1);
    wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
    wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
    if nargin == 3
      if wght1(mmax) == 0
        tz = 1;    
      else
        tz = (eps*wght1(1)/wght1(mmax))^(1/(mmaxm));
      end
    end
    if(abs(Z) < tz)
      tmp=(Z.^(0:mmaxm))';  
      eta(m+1) = wght1(1:mmax)*tmp;
      eta(m) = wght2(1:mmax)*tmp;
      for i=(m-1):-1:1
        eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
      end
      csi = Z * eta(2) + eta(1);
      return
    end
end
if Z >= tz  
      sq = sqrt(Z);
      csi = cosh(sq);
      if isinf(csi)
          wght2(1) = 1 / prod(2*(1:m-1)+1); 
          wght1(1) = wght2(1) / (2*m+1);
          wght1(2:mmax) = wght1(1)./ (cumprod(2*(2*(1:mmax-1)+2*m+1).*(1:mmaxm)));
          wght2(2:mmax) = wght2(1)./ (cumprod(2*(2*(1:mmax-1)+2*m-1).*(1:mmaxm)));
          tmp=(Z.^(0:mmaxm))';
          eta(m+1) = wght1(1:mmax)*tmp;
          eta(m) = wght2(1:mmax)*tmp;
          for i=(m-1):-1:1
              eta(i) = Z * eta(i+2) + (2*i+1) * eta(i+1);
          end
          csi = Z * eta(2) + eta(1);
          return;
      end
      eta(1) = sinh(sq)/sq;
else
      sq = sqrt(abs(Z));
      csi = cos(sq);
      eta(1) = sin(sq)/sq;
end    
eta(2)=(csi-eta(1))/Z;
for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
end

 
else %complex
    csi=cos(j*Z^(1/2));
    eta(1)= sin(j*Z^(1/2))/(j*Z^(1/2));
    eta(2)=(csi-eta(1))/Z;
    for i=3:m+1
     eta(i)=(eta(i-2)-(2*i-3)*eta(i-1))/Z;
    end
end


function eta = get_eta_noninteger(x,s)
% used when index of eta-function is not an integer
% expresses this function as a besselfunction
x = sqrt(-x);
if length(s)>1
  for i=1:length(s)   
    eta(i) = sqrt(pi/2)*(x)^(-(s(i)+1/2))*besselj(s(i)+1/2,x);
  end
else
  eta = sqrt(pi/2)*(x)^(-(s+1/2))*besselj(s+1/2,x);
end
%if imag(s) == 0
%  eta = sqrt(pi/2)*(x)^(-(s+1/2))*besselj(s+1/2,x);
  %else
%b = str2double(char(maple('evalf',maple('BesselJ',s+1/2,x))));
%eta = sqrt(pi/2)*(x)^(-(s+1/2))*b;



