function [x,y,yp,status]=calculate_eigenfunction_overflow(varargin)
%@PPM\CALCULATE_EIGENFUNCTION_overflow
% [x,y,yp] = calculate_eigenfunction_overflow(e,part)
%   This calculates the normalized eigenfunction (vector yn), 
%   associated to some input eigenvalue e, and its first 
%   derivative (vector ynp) at the meshpoints (x) of the partition.
%
%  This is the same as calculate_eigenfunction, but here is tried
%  to avoid overflow.
%
%      part              - the standard data concerning the partition 
%                          of the domain of the equation.
global ppm_child schrod_object
e= varargin{1};
part=varargin{2};
status =0;
[yi,yf]=get_ppm(ppm_child,'init_conditions');
yef=[0;0];
yei=[0;0];
number_intervals = length(part.h);

yn = zeros(2,number_intervals+1);

imatch = part.imatch;
i=1;
while imatch <= length(part.h)
  n=0;
  [yl,yel,yr,yer,number_rescales_l,number_rescales_r] = propagate_solution4(e,yi,yei,yf,yef,imatch,number_intervals,part);
  if number_rescales_l > number_rescales_r
      n = number_rescales_l - number_rescales_r;
      yr = yr/(n*10^150);
       yer = yer/(n*10^150);
  elseif number_rescales_l < number_rescales_r
      n = number_rescales_r - number_rescales_l;
      yl = yl/(n*10^150);
      yel = yel/(n*10^150);
  end
  qr = [yer(2) -yer(1)]*yr;
  ql = [-yel(2) yel(1)]*yl;
  % to avoid problems when in matchpoint yl and yr are near 0, and it isn't
  % clear whether the sign of the right part of the eigenfunction has to be 
  % changed or not.
  if imatch == length(part.h)
      break;
  end
  if n >0 && ((abs(yr(1)) < 2/(n*10^150) || abs(yl(1)) < 2/(n*10^150)) && abs(yr(2)) < 1/(n*10^150))
    imatch = min(imatch + i,length(part.h));
    i = i*2;
  else
    break;
  end
end
if abs(yr(1)) < 0.1  
   gap = yl(2)/yr(2);
else
    gap = yl(1)/yr(1);
end
fnorm = ql+qr*gap*gap;
sqfnorm = 1;
if fnorm  <= 0 
	warning('warning:eigfoverflow',' calculate_eigenfunction: There are problems with the normalization.')	
    status = 1;
elseif isinf(fnorm)
    error('calculate_eigenfunction: fnorm = Inf OVERFLOW')
else
    sqfnorm = sqrt(fnorm);  
end	
yi = yi./sqfnorm;
yf = yf*gap/sqfnorm;

yn(:,1) = yi;
q = 0;
number_rescales_f = 0;
for i=1:imatch
    [yn(:,i+1),ye(:,i+1),rescale] = propagate_solution_overflow(e,yn(:,i),i,part,true);
    if rescale
        number_rescales_f = number_rescales_f +1;
        yn(:,1:i) = yn(:,1:i)./10^150;
        ye(:,2:i) = ye(:,2:i)./10^150;
    end
end 
number_rescales_b = 0;
yn(:,number_intervals+1) = yf;
for i=number_intervals:-1:(imatch+1)  
	[yn(:,i),ye(:,i),rescale] = propagate_solution_overflow(e,yn(:,i+1),i,part,false);
    if rescale
       number_rescales_b = number_rescales_b +1;
       yn(:,i+1:number_intervals+1) =  yn(:,i+1:number_intervals+1)./10^150; 
       ye(:,i+1:number_intervals) =  ye(:,i+1:number_intervals)./10^150; 
    end
end 
if number_rescales_f > number_rescales_b
      n = number_rescales_f - number_rescales_b;
      yn(:,imatch+1:end) = yn(:,imatch+1:end)./(n*10^150);
      ye(:,imatch+1:end) = ye(:,imatch+1:end)./(n*10^150);
elseif number_rescales_f < number_rescales_b
      n = number_rescales_b - number_rescales_f;
       yn(:,1:imatch+1) = yn(:,1:imatch+1)./(n*10^150);
       ye(:,2:imatch+1) = ye(:,2:imatch+1)./(n*10^150);
end
for i=1:imatch
  q = q + [yn(2,i+1) -yn(1,i+1)]*ye(:,i+1);
end
for i=number_intervals:-1:(imatch+1)  
   q = q + [-yn(2,i) yn(1,i)]*ye(:,i);	
end
if q <= 0
   sq = 1;
   warning('warning:eigfoverflow1',' calculate_eigenfunction: There are problems with the normalization.' )
   status =1;
else	
   sq=1/sqrt(q);
end	
yn = yn.*sq;

if yn(1,1) < 0 || (yn(1,1) == 0 && yn(2,1) < 0) || (yn(1,1) == 0 && yn(2,1) == 0 && yn(1,2) <= 0)
     yn = (-1).*yn;
end
y = yn(1,:);
yp = yn(2,:);
xp = get(schrod_object,'min'); 
xx = cumsum(part.h(1:number_intervals)) + xp;  
x = [xp xx];
if nargin>2
    a = varargin{3};
    b = varargin{4};
    for i=1:length(x)
        if x(i)>=a
            x = x(i:end);
            y = y(i:end);
            yp= yp(i:end);
            break;
        end
    end
    for i=length(x):-1:1
        if x(i)<=b
            x = x(1:i);
            y = y(1:i);
            yp= yp(1:i);
            break;
        end
    end
end

%----------------------------------------------------
function [yl,yel,yr,yer,number_rescales_l,number_rescales_r] = ...
    propagate_solution4(e,yi,yei,yf,yef,imatch,number_intervals,part)
global ppm_child
y = yi;
ye = yei;
number_rescales_l = 0;
number_rescales_r = 0;
k=1:imatch;
%[Tm,Tem]=get_propagation_matrices(ppm_child,e,part,k,true);
[Tm,Tem]=get_ppm(ppm_child,'propagation_matrices',e,part,k,true);
for i=1:length(k)
    T=Tm(:,:,i);
    Te=Tem(:,:,i);
    tmp1 = Te*y+T*ye;
    tmp2 = T*y;
    if tmp2(1)>10^150 || tmp2(2)>10^150 || tmp1(1)>10^150 || tmp1(2)>10^150
        m = 10^150;
        y = y/m;
        ye = ye/m;
        tmp1 = Te*y+T*ye;
        tmp2 = T*y;
        number_rescales_l = number_rescales_l +1;
    end
    y = tmp2;
    ye = tmp1;
end
yl = y;
yel = ye;
y = yf;
ye = yef;
k=number_intervals:-1:imatch+1;
%[Tm,Tem]=get_propagation_matrices(ppm_child,e,part,k,false);
[Tm,Tem]=get_ppm(ppm_child,'propagation_matrices',e,part,k,false);
for i=1:length(k)
    T=Tm(:,:,i);
    Te=Tem(:,:,i);
    tmp1 = Te*y+T*ye;
    tmp2 = T*y;
    if tmp2(1)>10^150 || tmp2(2)>10^150 || tmp1(1)>10^150 || tmp1(2)>10^150
        m = 10^150;
        y = y/m;
        ye = ye/m;
        tmp1 = Te*y+T*ye;
        tmp2 = T*y;
        number_rescales_r = number_rescales_r +1;
    end
    y = tmp2;
    ye = tmp1;
end
yr = y;
yer = ye;

function [yt,yet,rescale] = propagate_solution_overflow(e,y,i,part,forward)
% same as propagate_solution but only used in the calculation of the
% eigenvector (calculate_eigenfunction_overflow and calculate_eigenfunction_overflow2). 
% avoids overflow.
global ppm_child
rescale = false;
ye =[0;0];
[T,Te]=get_ppm(ppm_child,'propagation_matrices',e,part,i,forward);
yet = Te*y+T*ye;
yt = T*y;
if yt(1) > 10^150 || yt(2) > 10^150 || yet(1) > 10^150 ||yet(2) > 10^150  
      rescale = true;
      y = y/10^150;
      ye = ye/10^150;
      yet = Te*y+T*ye;
      yt = T*y;
end
if isnan(yt)
      error('Difficulties in the calculation of the eigenvector. (Maybe you should make the integration interval smaller');
end
