 function [e] = convert_n2e(e,he,part,vmin,vmax,n)
% @PPM\CONVERT_N2E
% for given index n, this produces a value of e such as e_{n-1} < e < e_n
% if n > 0 or such as e < e_0 if n=0.
% 
%    INPUT:
%      e                  - a value such as e < e_n.
%      he                 - an initial increment along the e axis.
%      part               - the data concerning the partition
%                           of the equation domain.
%      vmin /vmax         - approximation of the min. and max. of the potential  
%      n                  - the index n.
global ppm_child
truncated=get(ppm_child,'truncated');

if he == 0
    he = 1;
end
if truncated
      et=e;
      [trunca,truncb]=get_truncation_points(part,e);
else
       trunca=1;
       truncb=length(part.h);
end
rn = get_ppm(ppm_child,'prufer',e,part,vmin,trunca,truncb);
p1 = n+1-rn-eps;
while p1 <= 0
    e = e - he;
    et=e;
    if truncated
       [trunca,truncb]=get_truncation_points(part,et);
    end
    rn = get_ppm(ppm_child,'prufer',e,part,vmin,trunca,truncb);	         	
    p1 = n+1-rn-eps;
end
while true
  if truncated && (trunca >1 || truncb<length(part.h))  
    while e > et
         et=e;
         [trunca,truncb]=get_truncation_points(part,et);
    end
  end
  rn = get_ppm(ppm_child,'prufer',e,part,vmin,trunca,truncb);         	
  p1 = n+1-rn-eps;
  if p1 > 0 
	  e0 = e;
	  p0 = p1;
	  e = e0 + he;
	  he = 2*he;
  else
	  e1 = e;
	  break;
  end
end
et=e0+(e1-e0)*0.95;
if truncated
  [trunca,truncb]=get_truncation_points(part,et);
end
rn =  get_ppm(ppm_child,'prufer',et,part,vmin,trunca,truncb); 
pt = n+1-rn-eps;
if pt>0
    e0=et;
    p0=pt;
else
    e1=et;
end
while true
   if p0 + eps > 1 
	  et = (e0 + e1)/2;
	  scalev = max(1,max(abs(et-vmin),abs(et-vmax)));	
	  if abs(e1-e0) < (eps*scalev)
          e = e0;
          break;
      else
          if truncated
             [trunca,truncb]=get_truncation_points(part,et);
          end
	      rn =  get_ppm(ppm_child,'prufer',et,part,vmin,trunca,truncb);       
	      pt = n+1-rn-eps;
	      if p0*pt < 0 
	        e1 = et;
	      else
	        e0 = et;
	        p0 = pt;
	      end
	  end
   else
	  e = e0;
	  break;
   end
end 


function [trunca,truncb]=get_truncation_points(bp,E)
trunca=1;
truncb=length(bp.h);
  V0=bp.V0;
  h=bp.h;
  sums=0;
  for i=bp.imatch:length(h)-1
    v0i=V0(i);  
    if v0i>=E && V0(i+1) > v0i
      sums=sums+sqrt(v0i-E)*h(i);
    else
        sums=0;
    end
    if sums > 15
        truncb=i;
        break;
    end
  end
  sums=0;
  for i=bp.imatch:-1:2
    v0i=V0(i);  
    if v0i>=E && V0(i-1) > v0i
      sums=sums+sqrt(v0i-E)*h(i);
    else
        sums=0;
    end
    if sums > 15
        trunca=i;
        break;
    end
  end
