function [emin,emax,nmin,nmax,err]=get_intervals(pmin,pmax,indices,imax)
% @PPM\GET_INTERVALS
% [emin,emax,nmin,nmax,err]=get_intervals(min,max,indices)
% returns in which energy-range and which index-range the required eigenvalues are
% situated

global ppm_child
bp = get(ppm_child,'bp');
if isempty(bp)
    error('The Problem is singular, no eigenvalues can be calculated.');
end
vmin = get(ppm_child,'Vmin');
vmax = get(ppm_child,'Vmax');
tol = get(ppm_child,'tol');
truncated=get(ppm_child,'truncated');
s=get(ppm_child,'schrod');

%accepte= energy value < lowest eigenvalue
accepte = -2*abs(vmin);
if accepte < -100 && indices
    V=get(s,'V');
    xmin=get(s,'min');
    xmax=get(s,'max');
    if V(xmin)<vmin || V(xmax)<vmin
       %-inf in a || -inf in b
         accepte =-1.1*abs(vmin);
         if truncated
            [trunca,truncb]=get_truncation_points(bp,accepte);
         else
            trunca=1;
            truncb=length(bp.h);
         end
         acceptn = get_ppm(ppm_child,'prufer',accepte,bp,vmin,1,length(bp.h));
         it=1;
         dif=(acceptn - pmin -1);
         while ((acceptn - pmin -1) > eps)
             accepte=0.75*accepte;
             acceptn = get_ppm(ppm_child,'prufer',accepte,bp,vmin,trunca,truncb);
             it=it+1;
             if it>25
                err.fail = true; 
                err.msg = sprintf('The minimal eigenvalue index to be accepted is %d.',floor(acceptn));
                emin = 0;
                emax = 0;
                nmin = 0;
                nmax = 0;
                return;
             end
             if (acceptn - pmin -1)>dif
                 accepte=-2*abs(vmin);
             end
             dif=(acceptn - pmin -1);
         end
    end
end
if abs(vmin) < 2 && indices
       V=get(s,'V');
       xmin=get(s,'min');
       vm=min(V(xmin+sum(bp.h(1:bp.imatch))),vmin);
       vm1=V(xmin+sum(bp.h(1:bp.imatch-1)));
       try
        vm2=V(xmin+sum(bp.h(1:bp.imatch+1)));
       catch
        vm2=vm1;
       end
       vm=min(vm1,vm);
       vm=min(vm2,vm);
       accepte = vm-2*max([abs(vmin-vm),abs(vmin-vm1),abs(vmin-vm2),abs(vm1-vm2)]);
end
err.fail = false;

if indices
    % the energy range [emin,emax] consistent with the input index range is
    % determined
    if truncated
            [trunca,truncb]=get_truncation_points(bp,accepte);
    else
            trunca=1;
            truncb=length(bp.h);
    end   
    acceptn = get_ppm(ppm_child,'prufer',accepte,bp,vmin,trunca,truncb);
    if ((acceptn - pmin -1) > eps)
          accepte=accepte-2;
          acceptn = get_ppm(ppm_child,'prufer',accepte,bp,vmin,trunca,truncb);
          if ((acceptn - pmin -1) > eps)
           err.fail = true; 
           err.msg = sprintf(' The minimal eigenvalue index to be accepted is %d.',floor(acceptn));
           emin = 0;
           emax = 0;
           nmin = 0;
           nmax = 0;
           return;
          end
    end
    e = accepte;
    scal = min(abs(e)/(pmax+1-pmin),200);
    emin = convert_n2e(e,scal,bp,vmin,vmax,pmin);
    nmin = pmin;
    c=get(s,'classification');
    if ~isempty(c)
     if c.cutoff<1e200
        scal = min(abs(c.cutoff-emin)/(pmax+1-pmin),200);
     end
    end
    emax = convert_n2e(emin,scal,bp,vmin,vmax,pmax+1);
    if emax == 0
        emax = eps;
    end
    nmax = pmax;
else
    % produce the index of the lowest and of the highest eigenvalue in the
    % given energy interval (= nmin, nmax resp. )
    emin = pmin;
    emax = pmax;
    if truncated
            [trunca,truncb]=get_truncation_points(bp,emin);
    else
           trunca=1;
           truncb=length(bp.h);
    end
    nmin0 = get_ppm(ppm_child,'prufer',emin,bp,vmin,trunca,truncb); 
    nmin = floor(nmin0);
    if(abs(nmin-nmin0)<tol)
     warning('getIntervals:warning','it may be safer to decrease tol')    
    end
    nmax = get_ppm(ppm_child,'prufer',emax,bp,vmin,1,length(bp.h));
    nmax = floor(nmax - 1);
end


function [trunca,truncb]=get_truncation_points(bp,E)
trunca=1;
truncb=length(bp.h);
  V0=bp.V0;
  h=bp.h;
  sums=0;
  for i=bp.imatch:length(h)-1
    if V0(i)>=E && V0(i+1) > V0(i)
      sums=sums+sqrt(V0(i)-E)*h(i);
    else
        sums=0;
    end
    if sums > 17
        truncb=i;
        break;
    end
  end
  sums=0;
  for i=bp.imatch:-1:2
    if V0(i)>=E && V0(i-1) > V0(i)
      sums=sums+sqrt(V0(i)-E)*h(i);
    else
        sums=0;
    end
    if sums > 17
        trunca=i;
        break;
    end
  end
