function c = classification(V,a,b,bcs)
%SLEDGE classification algorithm

% This routine classifies the Schrodinger problem.  Note:
% (1) any computational algorithm must be based on a finite
%     amount of information; hence, there will always be cases that
%     any algorithm misclassifies.  In addition, some problems are
%     inherently ill-conditioned, in that a small change in the 
%     coefficients can produce a totally different classification.
% (2)  The maximum number of points sampled for singular problems
%      is given by the variable kmax.  By increasing this number, the
%      reliability of the classification may increase; however, the
%      computing time may also increase.  The values we have chosen
%      seem to be a reasonable balance for most problems.

%zo kan ik meteen weten als een bepaalde gevraagde waarde in het continu spectrum
%valt

for j=1:2 %for each endpoint
  if j==1 %first endpoint
      endpoint = a;
      direction(j) = 1;
  else %second endpoint
      endpoint = b;
      direction(j) = -1;
  end
  kmax = 16; %number of points sampled
  if isinf(endpoint)
      base = 8;
%      kmax=14;
  else
      base = 1/8;
 %     kmax = 16;
  end
  z(:,j) = base.^(1:kmax); %sample points
  kused(j) = kmax;
  for k=1:kmax
     if isinf(endpoint)
         z(k,j) = -direction(j)*z(k,j);
     else
         z(k,j) = endpoint + direction(j)*z(k,j);
     end
   %  z(k,j)
     Q(k,j)=V(z(k,j));
     if log(abs(Q(k,j)))>300 || isnan(Q(k,j)) || abs(endpoint-z(k,j))<1e-14
         kused(j)=k-1;
         break;
     end
  end
end %for

warning on;
c.bcs = bcs;
[cev1,cont_spectrum1,regular1,limit_circle1,oscillatory1,BC,uncertain1,enda,c.err(1)]=classify_endpoint(z(1:kused(1),1),Q(1:kused(1),1),V,a,direction(1));
if (cont_spectrum1 || ~oscillatory1 ) && ~regular1
    c.bcs(1)=BC(1);
    c.bcs(2)=-BC(2);
end
[cev2,cont_spectrum2,regular2,limit_circle2,oscillatory2,BC,uncertain2,endb,c.err(2)]=classify_endpoint(z(1:kused(2),2),Q(1:kused(2),2),V,b,direction(2));
if (cont_spectrum2 || ~oscillatory2 ) && ~regular2
    c.bcs(3)=BC(1);
    c.bcs(4)=-BC(2);
end
warning off;
c.cutoff = min(cev1,cev2);
c.end=[enda endb];

% if uncertain1 || uncertain2
%     warning('classification of the endpoint is not certain');
% end

%find the number of eigenvalues below the start of the continuous spectrum
lastev=-5; %lastev=number of eigenvalues below the continuous spectrum
if (~oscillatory1 && ~limit_circle2 && oscillatory2) || ...
    (~oscillatory2 && ~limit_circle1 && oscillatory1)
    lastev=0; %There are no eigenvalues below the start of the continuous spectrum.
elseif (oscillatory1 && ~limit_circle1 && oscillatory2 && limit_circle2) || ...
    (oscillatory2 && ~limit_circle2 && oscillatory1 && limit_circle1)
    lastev=0;
elseif (cont_spectrum1 && oscillatory2) || (cont_spectrum2 && oscillatory1)
    lastev=0;
elseif (cont_spectrum1 && ~oscillatory2) ||(cont_spectrum2 && ~oscillatory1) || ...
        cont_spectrum1 && cont_spectrum2
    end1=enda; 
    end2=endb;
    %if singular point, find truncated point:
    while (isinf(V(end1))||isnan(V(end1))) && ~isinf(enda)
        end1=end1+0.01;
    end
    while (isinf(V(end2))||isnan(V(end2))) && ~isinf(endb)
        end2=end2-0.01;
    end
    if cont_spectrum2 %our interval must be large enough
      v2=V(end2);
      if isinf(b) && v2 < c.cutoff 
        while c.cutoff-v2 > 0.1
          end2 = end2*2;
          v2 = V(end2);
        end
      end
    end
    if cont_spectrum1 %our interval must be large enough
      v1=V(end1);
      if isinf(a) && v1 < c.cutoff 
        while c.cutoff-v1 > 0.1
          end1 = end1*2;
          v1 = V(end1);
        end
      end
    end
    s=schrod(char(V),end1,end2,bcs(1),bcs(2),bcs(3),bcs(4),'x',false);
    cp=cpm16_14(s,1e-5); %construct mesh
    bp=get(cp,'bp');
    n1n=get_ppm(cp,'prufer',c.cutoff,get(cp,'bp'),get(cp,'Vmin'),1,length(bp.h));
    n1=floor(n1n);
    if isinf(a) || isinf(b) %enlarge interval
        if cont_spectrum1 && isinf(a)
            end1 = end1 - abs(end1)/2;
        end
        if cont_spectrum2 && isinf(b)
            end2 = end2 + abs(end2)/2;
        end
        tol=1e-5;
    else %lower tolerance
        tol=1e-8;
    end
    %kan goedkoper door mesh aan te
    %passen:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %zal dat veel goedkoper zijn?
    s=schrod(char(V),end1,end2,bcs(1),bcs(2),bcs(3),bcs(4),'x',false);
    cp=cpm16_14(s,tol);
     bp=get(cp,'bp');
    n2n=get_ppm(cp,'prufer',c.cutoff,get(cp,'bp'),get(cp,'Vmin'),1,length(bp.h));
    n2=floor(n2n);
    if n1 >= n2
        if (isinf(a) || isinf(b)) && n1==n2
            if cont_spectrum1 && isinf(a)
                end1 = end1-abs(end1)/2;
                h=abs(end1);
                if abs(V(end1 -h)-V(end1))<0.5 && (V(end1- h)-V(end1))/(h) < 0.5 && (V(end1- h)-V(end1-h*0.9))/(0.1*h) > 0
                    end1 = end1- h;
                end
                v2=V(end1);
                it=0;
                if v2 < c.cutoff 
                 while c.cutoff-v2 > 1e-3 && it<3
                    end1 = end1*2;
                    v2 = V(end1);
                    it=it+1;
                 end
                end
            end
            if cont_spectrum2 && isinf(b)
                end2 = end2+ abs(end2)/2;
                h=end2;
                if abs(V(end2+ h)-V(end2))<0.5 && (V(end2+ h)-V(end2))/(h) < 0.5 && (V(end2+ h)-V(end2+h*0.9))/(0.1*h) > 0
                    end2 = end2+ h;
                end
                v2=V(end2);
                it=0;
                if v2 < c.cutoff 
                 while c.cutoff-v2 > 1e-3 && it<3
                    end2 = end2*2;
                    v2 = V(end2);
                    it=it+1;
                 end
                end
            end
            s=schrod(char(V),end1,end2,bcs(1),bcs(2),bcs(3),bcs(4),'x',false);
            cp=cpm16_14(s,tol);
            bp=get(cp,'bp');
            vmin=get(cp,'Vmin');
            n3n=get_ppm(cp,'prufer',c.cutoff,bp,vmin,1,length(bp.h));
            n3=floor(n3n);
            if n3>n2
                lastev=-5;
            else
                if n3n<n2n
                    if n2n<n1n && n1n-floor(n1n)>0.9
                        lastev=round(n3n);
                    else
                        lastev=n3; 
                    end
                else
                   lastev=n2;
                end
            end
        else
            lastev=n2;
            if n1 ~=n2
                warning('The eigenvalue count is uncertain');
            end
        end
    else
       lastev=-5;
    end
end
c.lastev=lastev;
c.regular=[regular1 regular2];
c.limit_circle=[limit_circle1 limit_circle2];
c.cont_spectrum=[cont_spectrum1 cont_spectrum2];
c.cev=[cev1 cev2];
if cont_spectrum1
    oscillatory1=false;
end
if cont_spectrum2
    oscillatory2=false;
end
c.oscillatory=[oscillatory1 oscillatory2];
%classify the problem in one of the 10 categories
if ~oscillatory1 && ~oscillatory2 && ~cont_spectrum2 && ~cont_spectrum1
    category=1;
elseif (~oscillatory1 && ~cont_spectrum1 && cont_spectrum2) || (~oscillatory2 && ~cont_spectrum2 && cont_spectrum1)
    category=2;
elseif (~oscillatory1 && ~cont_spectrum1 && oscillatory2 && limit_circle2 && ~cont_spectrum2) ||...
        (~oscillatory2 && ~cont_spectrum2 && oscillatory1 && limit_circle1 && ~cont_spectrum1)
    category=3;
elseif (~oscillatory1 && ~cont_spectrum1 && ~limit_circle2 && oscillatory2 && ~cont_spectrum2) ||...
        (~oscillatory2 && ~cont_spectrum2 && ~limit_circle1 && oscillatory1 && ~cont_spectrum1)
    category=4;
elseif limit_circle1 && oscillatory1 && limit_circle2 && oscillatory2
    category=5;
elseif oscillatory1 && ~limit_circle1 && ~cont_spectrum1 && oscillatory2 && ~limit_circle2 && ~cont_spectrum2
    category=6;
elseif oscillatory1 && oscillatory2 && ~(cont_spectrum1 || cont_spectrum2) && ((limit_circle1 && ~limit_circle2) ||(limit_circle2 && ~limit_circle1))
    category=7;
elseif (limit_circle1 && oscillatory1 && cont_spectrum2)||(limit_circle2 && oscillatory2 && cont_spectrum1)
    category=8;
elseif (~limit_circle1 && oscillatory1 && ~cont_spectrum1 && cont_spectrum2) || (~limit_circle2 && oscillatory2 && ~cont_spectrum2 && cont_spectrum1)
    category=9;
elseif cont_spectrum1 && cont_spectrum2
    category=10;
end
c.category=category;



%-----------------------------------------------------------------
function [cev,cont_spectrum,regular,limit_circle,oscillatory,BC,uncertain,ende,err]=classify_endpoint(z,qz,V,endpoint,direction)
%information about the problem at the singular endpoint (called "endpoint")
%is passed through the variables cev,cont_spectrum,regular,...

%initializations:
U=realmin*10; %small value
cont_spectrum=false;
irreg=false;
oscillatory=false;
limit_circle=false;
KCLASS = 0;
cev=1/U;
BC=[0;0];
ende=endpoint;
err=0;

%seek monomial approximation 
[eq,cq,qosc,exact,uncertain]=powerf(z,qz,endpoint,direction);
if abs(cq) <= 1e-14
    cq=0;
end
if cq == 0
    eq = 0;
end

if qosc
  %disp(' q(x) is not well approximated by a power potential, classification is uncertain!');
  err=1;
  uncertain=true;
  if isinf(endpoint)
      %oscillatory coefficient function
      KCLASS=1;
      %warning('oscillatory coefficient function');
      regular =false;
      limit_circle=false;
      cont_spectrum=true;
      oscillatory = false;
      BC(1)=1;
      BC(2)=0;
      kmax = length(qz);
      cev = qz(kmax-1);
      delta = (z(kmax)-z(kmax-1))/(41);
      [cq,mini] = min(V(z(kmax)-(0:40).*delta));
      cev = min(cev,cq);
      mini = mini-1;
      if (U*abs(cev) >= 1) 
              cont_spectrum = false;
      elseif eq > 2 %%%%%%%%%%%%%%%%%%%%%%%%%%????%%%%%%%????
             %het stijgt, er is geen continu spectrum
               cont_spectrum=false;
      else %als het oscillatorisch is en niet stijgt
        if z(kmax)<0
          tmp1=max(-1e13,z(kmax));
        else
          tmp1=min(1e13,z(kmax)); 
        end
        ceve=V(tmp1);
        try
         tmp=tmp1-0.01;
         vm=V(tmp);   
         while vm<ceve
          ceve=vm;
          tmp=tmp-0.01;
          vm=V(tmp);          
         end
        catch
        end
        try
         tmp=tmp1+0.01;
         vm=V(tmp);   
         while vm<ceve
          ceve=vm;   
          tmp=tmp+0.01;
          vm=V(tmp);           
         end
        catch
        end
        cev=min(cev,ceve);
      end
      if abs(cev) < 5e-12
          cev=0;
      end
      if U*abs(cev)>=1
          cont_spectrum=false;
      end
      ende=99; 
      if endpoint<0
        ende=-99;
      end
      return  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%die stond er niet
  else
%       regular=true;
%       if isinf(endpoint) || eq<=-1
%           regular=false;
%       end
      limit_circle=true;
      oscillatory = false;
  end
 % eqlnf=0;
end

%analyze this endpoint
if eq >-1 && ~isinf(endpoint)
    %If the endpoint is finite, then it is regular if and only if Eq>-1
    %Theorem 1
    regular=true;
    limit_circle=true;
    oscillatory=false;
    cont_spectrum=false;
    return
end
regular=false;
eta(1)=1; %always 1 for LNF
eqlnf=eq;
if eqlnf>=0
  iqlnf=eqlnf+0.5*eqlnf;
else
  iqlnf=eqlnf-0.5*eqlnf;
end
if abs(iqlnf-eqlnf)<1e-13
    eqlnf=iqlnf;
end
c1=cq;
if c1 == 0
    eqlnf=0;
end
c2=0; %always zero for LNF
%disp(['C1: ' num2str(c1) ' C2: ' num2str(c2) ' eqlnf: '  num2str(eqlnf) ]);
if isinf(endpoint)
   %make an initial estimate for "infinity" 
   if eq<0 || cq == 0
       gamma=eq;
       delta=abs(cq);
       if delta == 0
           delta=1;
       end
   else
       if eq >0 
           gamma=eq;
           delta=abs(cq);
       else
           gamma=0;
           delta=abs(1-cq);
       end
   end
   if gamma >0.5
       if gamma < 2 %potentiaal stijgt geleidelijk
         ende=80;
       else
         ende=min(max(64/((2*gamma-3)*delta^(1/(gamma+2))),1),80);
         if gamma > 24  %sterke stijging in potentiaal
            ende=12;
            if gamma > 40
               ende = 10; 
            end
            while V(-direction*ende)-V(0) >1e7
               ende=ende-1;
            end
         end
       end
   else
       if gamma <-0.5
          ende=max(min(600*delta^(1/gamma)*5^gamma,120),2);
          endn=ende;
          while true
              if V(endn+1)<V(endn)
                  break;
              end
              endn=endn+2;
              if endn>=ende+10;
                  break;
              end
          end
          ende=endn;
       else
          ende=12;
       end
       if gamma == 0 && cq ~=0
          ende =40;
       end
   end
   if endpoint<0 && ende>0
      ende = -ende;
   end
else
    %test for finite irregular singular points
    if eq>=0
        k=eq+0.5;
    else
        k=eq-0.5;
    end
    irreg=true;
    if cq ~= 0
        if 0<=eq 
            irreg = false;
        elseif 0>eq && k>=-2 
            irreg = false;
        end
    else
        irreg=false;
    end
    if irreg
       % disp('This is a irregular singular point');
        KCLASS=5;
    end
end

if ~isinf(endpoint)
    %finite endpoint
    %Carry out the Case 1 tests.
    osc = 0;
    %Theorem 4
    if eqlnf == -2 || abs(eqlnf+2)<1e-14
         if abs(c1+0.25) <= 1e-13 
%            warning('  WARNING: borderline nonoscillatory/oscillatory classification.')
               osc = -1;
               uncertain = true;
               err=2;
         else
               if (c1 < -0.25-1e-13) 
                   osc = 1; %OSC
               end
               if (c1 > -0.25) 
                   osc = -1; %NONOSC
               end
         end
    elseif eqlnf < -2
            if cq<0
                osc = 1; %OSC
            elseif cq > 0
                osc = -1; %NONOSC
            end
    elseif eqlnf > -2 
                osc= -1; %NONOSC
    end
    if osc==1 
            oscillatory = true;
    elseif osc==-1 
            oscillatory = false;
    else
             %warning('  NO INFORMATION on osc/nonosc class.')
             err=5;
    end
    lc = 0;
    if eqlnf == -2 || abs(eqlnf+2)<1e-14 
            if (abs(c1-3/4) <= 1e-13) 
               lc = -1;
%               warning('  WARNING: borderline Lc/Lp classification.')
               err=3;
               uncertain=true;
            elseif (c1 >= 3/4) 
                lc = -1; %limit point (theorem 3)
            elseif (abs(c1) < 3/4-1e-13) 
                lc = 1; %limit circle
            end
            if (c1 < -1e-13)
                lc = 1;
            end
    elseif  eqlnf < -2
            if (cq > 0) 
                lc = -1; %limit point
            elseif cq < 0
                lc = 1; %limit circle
            end
    elseif eqlnf > -2 
            lc = 1; %limit circle
    end
    if (lc == 1) 
            limit_circle = true;
    elseif lc == -1
             limit_circle = false;
    else
             limit_circle = -1;
             %warning('  NO INFORMATION on Lp/Lc class.')
             err=4;
    end
end

%%%%%%%%%%%%%


     if isinf(endpoint) 
        %infinite endpoint
        %Carry out the Case 2 tests.
         osc = 0;
         if ((eqlnf > 0) && (cq < 0)) 
             osc = 1; %OSC
         elseif ((eqlnf > 0) && (cq > 0)) 
             osc = -1; %NONOSC
         end
         if (eqlnf == 0) %NONOSC/OSC with cutoff value 
            osc = -1;
            cev = cq; %then cutoff value= cq (Theorem 4)
            cont_spectrum = true;
            if (U*abs(cev) >= 1) 
                cont_spectrum = false;
            end
         end
         if (eqlnf < 0) %NONOSC/OSC with cutoff value
            osc = -1;
            cev = 0; %cutoff
            cont_spectrum = true;
         end
         if (osc == 1) 
            oscillatory = true;
         else
            if (osc == -1) 
               oscillatory = false;
            else
             %  warning('  NO INFORMATION on Osc/Nonosc class.')
               err=5;
            end
         end
         lc = 0;
         if ((eqlnf > 2) && (cq > 0)) || eqlnf <=2  || cq ==0
             lc = -1; %limit point
         end
         if ((eqlnf > 2) && (cq < 0)) 
             lc = 1; %limit circle
         end
         if (lc == 1) 
            limit_circle = true;
         else
            if (lc == -1) 
               limit_circle = false;
            else
               limit_circle = -1; 
               %warning('  NO INFORMATION on Lp/Lc class.')
               err=4;
            end
         end
     end
%%%%%%%%%%%%%
%  Case 3 and 4 don't occur for LNF
%%%%%%%%%%%%%

      if (abs(cev) <= 1e-13) 
          cev = 0;
      end

%     Calculate the Friedrichs boundary condition (if appropriate).

      if cont_spectrum 
         BC(1) = 1;
         BC(2) = 0;
      end
      if (~cont_spectrum && ~oscillatory ) 
           sgn=1;
           if isinf(endpoint)
               sgn=-1;
           end
           if sgn*eq < 0 
                  BC(1) = 1;
                  BC(2) = 0;
           end
      end
      if (~oscillatory) 
         if isinf(endpoint) && (eqlnf == -1) 
           %disp('infinite endpoint, eqlnf=-1')
           KCLASS=3;
         end
         i = 0;
         if cq < 0 
            i = min(i,eq);
         end
         if (~isinf(endpoint) && (i < 0)) 
%              Transform some nonoscillatory problems for which Tau
%              is unbounded near a finite singular endpoint.
               if irreg
               %    disp('"hard" irregular, eta(1)>0');
                   KCLASS = 10;
               end
               if ((eqlnf == -2||abs(eqlnf+2)<1e-14) || (c1 == 0)) 
                  if ~irreg
                %      disp('Q is unbounded (like 1/t^2) near a nonoscillatory finite end');
                      KCLASS = 9;
                  end
               else
                  if ~irreg 
                      %disp('Q is unbounded (<1/t^2) near a nonoscillatory finite end');
                      KCLASS = 8;
                  end
               end
         end
         if (KCLASS >= 9) 
               if (~exact) 
                   warning('not exact')
                 %  LFLAG(5) = .TRUE.
               end
               %warning('  This problem has unbounded [Ev*r(x)-q(x)]/p(x).')
         end
         if (~isinf(endpoint) && (~regular) && KCLASS == 0)
             KCLASS = 4;
         end
      end
      if qosc && isinf(endpoint) 
          ende = 99.0;
      end
      %disp(['  Classification type (KCLASS) is: ',num2str(KCLASS)]);
   
    
%-------------------------------------------------------------------
function [EF,CF,OSC,exact,uncertain]=powerf(x,f,e,direction)
% Find the power function which "dominates" the tabled
% coefficient function.  The output is Cf and Ef such that
%           f(x)  is asymptotic to  Cf*x^Ef or Cf*(x-e)^Ef.
%     The vectors X(*) and F(*) hold the N input points:
%           F(I) = f(X(I)) I = 1,...,N.  
%     e=the value of the endpoint


uncertain=false;
OSC=false;

 %Estimate the exponent.
n = length(f);
if isinf(e)
  for k = 1:n-1
    if (f(k) ~= 0) && (f(k+1) ~= 0) 
         y(k) = log(abs(f(k+1)/f(k)))/log(abs(x(k+1)/x(k)));
    else
         y(k) = 0;
    end
  end   
 else
  for k = 1:n-1
    if (f(k) ~= 0) && (f(k+1) ~= 0)  
         y(k) = log(abs(f(k+1)/f(k)))/log(abs((x(k+1)-e)/(x(k)-e)));
    else
         y(k) = 0;
    end
  end
end
[EF,err] = aitken(1e-13,n-1,y); % Use Aitken's algorithm to accelerate convergence of the sequence in y
k = EF+sign(EF)*0.5;
if EF == 0
    k = EF+0.5;
end
if abs(k-EF) <= 1e-6
    EF = k;
end
if abs(err) > 1e-13*max(1,abs(EF)) 
%       There is uncertainty in the exponent.
        uncertain = true;
        disp('classification is not certain');
end
if (abs(EF) <= 10*eps) 
    EF = 0;
end

% Estimate the coefficient.
if isinf(e)
  for k = 1:n-1
   y(k) = f(k)/abs(x(k))^EF; 
  end  
else
  for k = 1:n-1
   y(k) = f(k)/abs(x(k)-e)^EF;
  end
end
[CF,err] = aitken(1e-13,n-1,y);
if isinf(e)
    direction=direction*(-1);
    e=0;
end
% CF=CF
% EF=EF
% abs(f(n)-CF*(direction*(x(n)-e))^EF) 
% 20.0*1e-11*abs(f(n)) 
if (EF > 20) && (abs(CF) <= 1e-13) 
%   Coefficient probably has exponential behaviour.
       if CF >= 0
         CF = 1;
       else
         CF = -1;
       end
elseif ((abs(err) > 1e-13*max(1,abs(CF))) || ...
       ((abs(f(n)-CF*(direction*(x(n)-e))^EF) > 20.0*1e-11*abs(f(n))) && (EF ~= 0 ))) 
%      There is uncertainty in the coefficient; call such
%      cases oscillatory.
       uncertain=true;
       OSC = true;
end
if (abs(CF) > 1e7) 
    exact = false;
    return
end
k = CF+0.5;
if abs(k-CF) <= 1e-7  && k ~= 0 
    CF = k;
end
exact = true;
for k = 1:n
    if (abs(f(k)-CF*(direction*(x(k)-e))^EF) > 1e-11*abs(f(k))) 
        exact = false;
    end
end


%-------------------------------------------------------------
function [xlim,err] = aitken(tol,n,x)
%     Use Aitken's algorithm to accelerate convergence of the sequence
%     in x(*).

if n <= 2 
   xlim = x(n);
   err = 0;
   return;
end
xold = 1e30;
for i = 1:n-2
   denom = x(i+2)-2*x(i+1)+x(i);
   if denom ~= 0 
        xlim = x(i)-(x(i+1)-x(i))^2/denom;
        err = xlim-xold;
   else
        err = x(i+2)-x(i+1);
        xlim = x(i+2);
   end
   if (abs(err) < max(1,abs(xlim))*1e-13) 
       return
   end
   xold = xlim;
end