function transf = Liouvilletransform(sl)
% LIOUVILLETRANSFORM: Liouville's transformation
% usage: transf = liouvilletransform(sl), 
%        transf is a transformed_slp-object
global slfev
syms x; %all inline functions have x as unknown variable
chp=char(sl.p);
chw=char(sl.w);

%function needed to convert x to r:
fcn = char(simple(sym(sprintf('sqrt((%s)/(%s))',chw,chp))));
fcn = vectorize([fcn '+0*x']); % makes it possible to do fcn(X) where X is a vector
fcn = inline(fcn);

if strcmp('0',char(sl.w))
    error('indefinite problem');
end
m = inline(char(simple(sym(eval(sprintf('(%s)^(-1/4)*(%s)^(-1/4)',chp,chw))))));
dmr = inline(char(simple(diff(sprintf('((%s)^(1/4)*(%s)^(1/4))',chp,chw),x))));
s = eval(sprintf('(((%s)^(1/2))/((%s)^(1/2)))*(%s)',chp,chw,char(dmr)));
if s == 0
    ddm = '0';
else
    dm = char(simple(s));
    ddm = char(diff(dm,x));
    ddm = char(simple(eval(sprintf('(((%s)^(1/2))/((%s)^(1/2)))*(%s)',chp,chw,ddm))));
end
try
V = char(simple(eval(sprintf('(%s)/(%s) + (%s)*(%s)',char(sl.q),chw,char(m),ddm))));
% this V operates on the unknown variable (r) of the Sturm-Liouville problem 
% apply first the function x2r on x (schrodinger-variable) before using this V. 
catch
 V = char(eval(sprintf('(%s)/(%s) + (%s)*(%s)',char(sl.q),chw,char(m),ddm)));
 if (strcmp(ddm,'0') && strcmp(char(sl.q),'0')) 
     V = '0';
 end
end

if ~strcmp(char(sl.p),char(sl.w)) && (isinf(fcn(sl.min)) || isinf(fcn(sl.max)) || isnan(fcn(sl.min)) || isnan(fcn(sl.max)) )
    transf=transformed_slp(schrod(V,sl.min,sl.max),sl,[],[],[]);
    transf=set(transf,'singularities',[sl.min sl.max]);
    return; %singularities, it is too early to do the Liouville transformation, the interval must me truncated first
end

inf_min=false;
inf_max=false;
if isinf(sl.min)||isinf(sl.max)
    %to handle an infinite integration interval
    if isinf(sl.min) && isinf(sl.max)
       sl.min=-1;
       inf_min=true;
       sl.max=1;
       inf_max=true;
    elseif isinf(sl.max)
        sl.max=sl.min+1;
        inf_max=true;
    elseif isinf(sl.min)
        sl.min = sl.max-1;
        inf_min=true;
    end
end

[min,max,rG,xG,q] = sl2sch(fcn,sl.min,sl.max);

pause(1e-200) % stop the calculation in the GUI?
load stop
if stop
     error('The calculation was interrupted');
end


t = sprintf('(%d)*(%s)*(%s)*(%s)',sl.b0,char(sl.p),char(dmr),char(m));
t = inline(char(simple(sym(eval(sprintf('(%d)*(%s)*(%s)+(%s)',sl.a0,char(m),char(m),t))))));
a0 = t(sl.min);
slfev = slfev +1;
b0 = sl.b0;
if isinf(a0) || (a0==0 && b0==0)
     a0=1;
     b0=0;
end

t = sprintf('(%d)*(%s)*(%s)*(%s)',sl.b1,char(sl.p),char(dmr),char(m));
t = inline(char(simple(sym(eval(sprintf('(%d)*(%s)*(%s)+(%s)',sl.a1,char(m),char(m),t))))));
a1 = t(sl.max);
slfev = slfev +1;
b1 = sl.b1;
if isinf(a1) || a1==0 && b1==0
     a1=1;
     b1=0;
end
if inf_min
    min=0;  
    sl.min = -inf;
end
if inf_max
    max=inf;
    sl.max = inf;
end
pot=inline(V);
rs = schrod(V,min,max,a0,b0,a1,b1,'x',false);
if ~isempty(get(rs,'singularities'))
    rmin=sl.min;
    rmax=sl.max;
    if ~(isinf(rmin) || isinf(rmax) || isinf(pot(rmin)) || isinf(pot(rmax)) || isnan(pot(rmin)) || isnan(pot(rmax)))
        rs=set(rs,'singularities',[]);
    end
end
transf = transformed_slp(rs,sl,rG,xG,q); 
return;


function [xmin,xmax,rG,xG,q] = sl2sch(fcn,rmin,rmax)
% For given Sturm-Liouville problem, this returns
% the suitable x_min and x_max for the corresponding
% Schrodinger- problem. Also the rG - xG -points are
% returned.

i = 1;
xmin = 0;
rG(1) = rmin;
xG(1) = 0;
h = (rmax-rmin)/10;
hmin = 10^4*eps*abs(rmax-rmin); 
while true
  rG(i+1) = rG(i) + h;
  if (rG(i+1) > (rmax-hmin))
    rG(i+1) = rmax;
    h = rG(i+1) - rG(i);
  end
  hi = h;
  accurate = false;
  while ~(accurate) 
    mid = (rG(i)+rG(i+1))/2;
    q0 = gauss(fcn,rG(i),rG(i+1));
    q1 = gauss(fcn,rG(i),mid); 
    q2 = gauss(fcn,mid,rG(i+1));
    aq = abs(q1+q2);
    diffaq = abs(q1+q2-q0);
    tol=eps*max(aq,1); 
    if(diffaq < tol)  
       accurate = true;
    else
       hi = hi/2;
       rG(i+1) = rG(i) + hi;
    end
  end %while ~(accurate)
  q(i) = q1 + q2;
  xG(i+1) = xG(i) + q(i); 
  if(rG(i+1) >= rmax) 
    xmax = xG(i+1);
    return;
  end
  i = i+1;
end %while



function res = gauss(func,a,b)
%SLP\GAUSS res = gauss(func,a,b)
% calculates the integral of func by the 
% twelve-point Gauss quadrature formula.
global slfev 
x = [0.125233408511468915472441369464,...
   0.36783149899818019375269153664,...
   0.5873179542866174472967024189,...
   0.7699026741943046870368938332,...
   0.904117256370474856678465866,...
   0.9815606342467192506905490901];
w = [0.24914704581340278500056243604,...
   0.233492536538354808760849899,...
   0.2031674267230659217490645,...
   0.160078328543346226334653,...
   0.106939325995318430960255,...
   0.047175336386511827194616];

m = (b+a)/2;
d = (b-a)/2;
dx = d*x;
ff=func([m+dx m-dx]);
fps=ff(1:6);
fms=ff(7:end);
lastwarn('');
res = d*(w*(fps+fms)');
slfev = slfev +12;
[msgstr,msgid] = lastwarn;
if strcmp(msgid,'MATLAB:divideByZero')
      error('Divide by Zero')
end


