function sl = slp(varargin)
%SLP Sturm-Liouville class constructor
% sl = slp(p,q,w,min,max)
% sl = slp(p,q,w,min,max,a0,b0,a1,b1)
% sl = slp(p,q,w,min,max,a0,b0,a1,b1,var).
% sl = slp(p,q,w,min,max,a0,b0,a1,b1,var,classification).
% If less than 10 arguments, then the independent variable 
% is supposed to be 'x'. In this case p,q and w are strings 
% representing the functions p(x), q(x), w(x) resp.
stop = 0;
save stop.mat stop;
switch nargin
case 0
    % if no input arguments, create a default object
   sl.p = inline('1');
   sl.q = inline('1');
   sl.w = inline('1');
   sl.classification=[];
   sl.min = 0;
   sl.max = pi;
   sl.a0 = 1;
   sl.b0 = 0;
   sl.a1 = 1;
   sl.b1 = 0;
   sl.classification=[];  
   sl = class(sl,'slp');
case 1    
    % copy-constructor
 if isa(varargin{1},'slp')
     sl = varargin{1};
 else
     disp('Input argument is not a Sturm-Liouville object')
 end
case {3,5,9,10,11}
    if ~isa(varargin{1},'char')
         error('slp-constructor: arg1: You need to pass the function as string ');  
    end
    if ~isa(varargin{2},'char')
         error('slp-constructor: arg2: You need to pass the function as string ');  
    end
    if ~isa(varargin{3},'char')
         error('slp-constructor: arg3: You need to pass the function as string ');  
    end
    sl.p = inline(varargin{1});
    sl.q = inline(varargin{2});
    sl.w = inline(varargin{3}); 
    sl.classification=[];  
    switch nargin
        case 3
          sl.min = 0;
          sl.max = pi;
          sl.a0 = 1;
          sl.b0 = 0;
          sl.a1 = 1;
          sl.b1 = 0;  
        case {5,9,10,11}
          classif = true;  
          if ~isa(varargin{4}, 'double')
             error('slp-constructor: arg4: You need to pass a double-precision floating-point number');  
          end
          if ~isa(varargin{5}, 'double')
             error('slp-constructor: arg5: You need to pass a double-precision floating-point number');  
          end
          sl.min = varargin{4};
          sl.max = varargin{5};
          if nargin==5
            sl.a0 = 1;
            sl.b0 = 0;
            sl.a1 = 1;
            sl.b1 = 0;
          else  
            if (~isa(varargin{6}, 'double')...
                || ~isa(varargin{7}, 'double')...
                || ~isa(varargin{8}, 'double')...
                || ~isa(varargin{9}, 'double'))
              error('slp-constructor: arg6-arg9: You need to pass double-precision floating-point numbers');  
            end
            sl.a0 = varargin{6};
            sl.b0 = varargin{7};
            sl.a1 = varargin{8};
            sl.b1 = varargin{9};
          end  
          if nargin == 11
            classif = varargin{11};
          end
          if classif
               bcs = [sl.a0 sl.b0 sl.a1 sl.b1];
               c = classification(sl.p,sl.q,sl.w,sl.min,sl.max,bcs); %SLEDGE CLASSIFICATION ALGO
               if ~(c.limit_circle(1))% && ~strcmp(char(sl.p),char(sl.w)))
                 sl.a0 = c.bcs(1);  %new boundary conditions
                 sl.b0 = c.bcs(2);
               end
               if ~(c.limit_circle(2))% && ~strcmp(char(sl.p),char(sl.w)))
                 sl.a1 = c.bcs(3);
                 sl.b1 = c.bcs(4);  
               end
               s = print_endpoint_info(c,sl.min,sl.max);
               c.infostring = s;
               sl.classification=c;
          else
               sl.classification=[];  
          end
    end
    sl = class(sl,'slp');
otherwise
    error('Wrong number of input arguments')
end;
if sl.min >= sl.max
    error('Error!!! : INVALID INTEGRATION INTERVAL')
end

