function [varargout] = transform_eigenfunction(varargin)
% @TRANSFORMED_SLP\TRANSFORM_EIGENFUNCTION
% [r,z,zprime]=transform_eigenfunction(transf_slp,x,y,yprime)
% calculating the original Sturm-Liouville-eigenfunction 'z'
% out of the Schrodinger-eigenfunction 'y'

% x = vector with the values of the meshpoints
% r = idem but for original variable

if nargin < 3
    error 'transformEigenfunction must have at least 3 input arguments'
    return;
end  
r = x2r(varargin{1},varargin{2});
p = get(varargin{1},'p');
w = get(varargin{1},'w');
varargout{1}=r;
m=zeros(1,length(varargin{2}));
switch nargout
    case 1
    case 2
        for i = 1:length(m)  
          m(i) = ((p(r(i)))*(w(r(i))))^(-1/4);
        end
        y = varargin{3} .* m;
        varargout{2} = y;
    case 3
        syms x;
        mp = inline(char(diff(sprintf('((%s)*(%s))^(-1/4)',char(p),char(w)),x)));   
        md=zeros(1,length(m));
        mt=zeros(1,length(m));
        for i = 1:length(m)
          ri=r(i);  
          pr=p(ri);
          m(i) = (pr*(w(ri)))^(-1/4);
          md(i) = mp(ri);
          mt(i) = m(i)*pr;
        end
        y = varargin{3} .* m;
        yprime = md .* varargin{3} + varargin{4}./(mt);
        varargout{2} = y;
        varargout{3} = yprime;
    otherwise
        warning('transformEigenfunction returns maximal 3 output arguments')
end

