function r = x2r(transf_slp,x)
% r = x2r(transf_slp,x)
%X2R For input x (schrodinger) this returns the corresponding 
% value of r (Sturm-Liouville).
% x can be a vector

fcn = get(transf_slp,'fcn');
rG = get(transf_slp,'rG');
xG = get(transf_slp,'xG');

%preallocation:
r=zeros(1,length(x));

for j = 1:length(x)
  %determination of the interval
  tmp=find(xG>x(j));
  if ~isempty(tmp)
    i=tmp(1)-1;
  else
    i=length(xG)-1;  
  end
  
  r1 = rG(i);
  r2 = rG(i+1);
  g1 = xG(i)-x(j);
  gcc = fcn([r1 r2]);
  gc1 = gcc(1);
  g2 = xG(i+1)-x(j);
  gc2 = gcc(2);
  loop1 = true;
  its=0;
  while loop1
    its=its+1;
    if its>5 && mod(its,4)==0
      pause(1e-200) % stop the calculation in the GUI?
      load stop; 
      if stop
        error('The calculation was interrupted');
      end
    end
    gleft = g1;
    n = 0;
    while true
     n = n+1;
     rn = 0.1*n;
     rn2 = rn*rn;
     a1 = rn2*(3-2*rn);
     a0 = 1-a1;
     b0 = rn-rn2*(2-rn);
     b1 = rn2*(rn-1);
     gright = a0*g1+a1*g2+(b0*gc1+b1*gc2)*(r2-r1);
     if (gleft*gright <= 0)
        break;
     else
        gleft=gright;
     end
    end
    %startpoint for Newton procedure
    rs = r1+(rn-0.1*gright/(gright-gleft))*(r2-r1);
    tol = 10*eps*max(abs(rs),1); 
    if (abs(r2-r1) < tol)
      r(j) = rs;
      break; 
    end 
    it=0;
    while true
    %Newton procedure
      it = it+1;
      if it >= 100
        r(j) = rs;
        loop1 = false;
        break;
      end
      q = gauss(fcn,rG(i),rs);
      gs = q - x(j) + xG(i);
      gsp = fcn(rs);    
      drnewton = -gs/gsp;
      rnewton = rs+drnewton;
      if ((rnewton-r1)*(rnewton-r2)<=0)
        rs = rnewton;
        tol = 10*eps*max(abs(rs),1); 
        if (abs(drnewton) <= tol) 
            r(j) = rs;
            loop1 = false;
            break;
        end
      else
        if(gs*g1 < 0) 
          g1 = gs;
          r1 = rs;
          gc1 = gsp;
        else
          g2 = gs;
          r2 = rs;
          gc2 = gsp;
        end
      end
    end %while
 end %while
end %for


